/******************************************************************************/
/* The information contained in this file is confidential and proprietary to  */
/* Broadcom Corporation.  No part of this file may be reproduced or           */
/* distributed, in any form or by any means for any purpose, without the      */
/* express written permission of Broadcom Corporation.                        */
/*                                                                            */
/* (c) COPYRIGHT 2001 Broadcom Corporation, ALL RIGHTS RESERVED.              */
/*                                                                            */
/*  Name: M E D I A _ R G N . H                                               */
/*  Author : Kevin Tran                                                       */
/*                                                                            */
/* Module Description:  This file contains definition of Serial EEPROM/Flash  */
/* layout.                                                                    */
/*                                                                            */
/* History:                                                                   */
/*    04/20/00 Kevin Tran       Incarnation.                                  */
/******************************************************************************/

#ifndef __MEDIA_REGIONS_H__
#define __MEDIA_REGIONS_H__

#define MAGIC_VALUE     0x669955aa

#ifndef TYPE_DEFINED
#include "types.h"
#endif

/**************************************************************************
 *      media_code_offset_area_t
 ***************************************************************************/

#define MEDIA_MAX_LOAD_STAGES   8


typedef struct media_code_offset_region 
{
  u32 sram_start_addr;  /* where to locate code region (byte addr) */
  /* The top 8 bits are used for TYPE field */

  /* We use 22 bit for length so that we accumulate up to 16 Mbyte */
#define CODE_IMAGE_LENGTH(x)       (x & 0x3fffff)
  u32 code_len;         /* code region length (in words) */
  u32 code_start_addr;  /* location of code on media (media byte addr) */
} media_code_offset_region_t, * pMEDIA_DIR;

typedef struct media_manufact_region 
{
  char manuf_format_rev;     /* Version format of this data structure */
  u08  reserved1;            /* reserved, must be 0 */
  u16  length;               /* length of this structure in bytes */
  u32  phy_id;               /* PHY ID of physical device */
  u08 mac_address[8];       /* MAC address */
  char part_number[16];      /* part number. Printable string ending in '\0'. */
  char part_revision[2];     /* part revision.  Two printable ascii char */
  u16  bootcode_fw_revision; /* single byte acending rev number */
  char manuf_data[4];        /* ascii manufact date, wwyy, \0 by next field */
  u32  oem_area_p;           /* pointer to 32 byte opaque OEM area or 0*/
  u16  pci_dev_id;           /* pci device id */
  u16  pci_vend_id;          /* pci vendor id */
  u16  pci_sub_id;           /* alternate pci subsystem id */
  u16  pci_sub_vend_id;      /* alternate pci subsystem vendor id*/
  u16  cpu_clk;              /* cpu clock speed in Mhz rounded to the */
  u16  reserved3;
  char sw_key[16];           /* software key, used to enable */
  u08  power_dissipated[4];
  u08  power_consumed[4];  
  u32  pxe_wol_admin;
  NIC_HW_CONFIG hw_config;
  u08 reserved4[48];
  u32 cksum;        /* 32-bit CRC */
} media_manufact_region_t;

#define MAX_VPD_R_LENGTH       128
#define MAX_VPD_W_LENGTH       128

typedef struct media_vpd_read_t
{
  u08 data[MAX_VPD_R_LENGTH];
}media_vpd_read_t;

typedef struct media_vpd_write_t
{
  u08 data[MAX_VPD_W_LENGTH];
}media_vpd_write_t;

typedef struct media_vpd_t
{
  media_vpd_write_t vpd_r;
  media_vpd_write_t vpd_w;
}media_vpd_t;



#endif /* __MEDIA_REGIONS_H__ */
