/******************************************************************************
 * The information contained in this file is confidential and proprietary to  
 * Broadcom Corporation.  No part of this file may be reproduced or           
 * distributed, in any form or by any means for any purpose, without the      
 * express written permission of Broadcom Corporation.                        
 *                                                                            
 * (c) COPYRIGHT 2000 Broadcom Corporation, ALL RIGHTS RESERVED.              
 *                                                                            
 *  FileName: $Header: /Source/Bcm44xx/b44diag/screen.c 3     3/07/02 5:05p Ahui $
 *
 * Module Description:                                                        
 *                                                                            
 *  $Log: /Source/Bcm44xx/b44diag/screen.c $
 * 
 * 3     3/07/02 5:05p Ahui
 * Added sromutil support
 ******************************************************************************/
#include "t3_scrn.h"
#include <stdarg.h>
#ifndef EFI
#include <conio.h>
#endif
#include <ctype.h>
#include "t3_gdata.h"
#include "t3_comm.h"

/* There are three kinds of coordinate used in this module:
    1. AbsPoint     This is aboslute point of monitor screen. For a stand 80X24
                    screen, the valid range is 0-24, 0-79.
                    
    2. ScrnPoint    This is each virtual screen's point. It refrence to each 
                    screen's base position. 
                    To conver to AbsPoint, it will have to add the base postion
                    
                    For example,
                        AbsPoint.r = r + scrn.pos.r;
                        AbsPoint.c = c + scrn.pos.c;
                    
    3. UserPoint    This is application's point. Application can set a work area
                    in the virtual screen. To convert to ScrnPoint, user will have
                    to add the position of virtual screen. 
                    
                    For Example, 
                        ScrnPoint.r = r + scrn.area.ul.r;
                        ScrnPoint.c = c + scrn.area.ul.c;
                        AbsPoint.r =  r + scrn.area.ul.r + scrn.pos.r;
                        AbsPoint.c = c + scrn.area.ul.c + scrn.pos.c;
*/                        

#ifdef EFI
#define ABS_SCREEN_BASE (ScreenPtr) (&curr_scrn[0][0])
#else
#define ABS_SCREEN_BASE (ScreenPtr) 0xb8000
#endif
#define MAX_COL 80
#define MAX_ROW 25

ScreenNode curr_scrn[MAX_ROW][MAX_COL];

Screen __scrn = {
//    {0,0},          /* pos */
    ABS_SCREEN_BASE,    /* buffer */
    ABS_SCREEN_BASE,    /* cursor */
    {0,0},
    {{0,0},{MAX_ROW-1,MAX_COL-1}}, /* area */
    MAX_ROW,        /* rsize */
    MAX_COL,        /* csize */
    GRAY,           /* attrib */
};

static CHAR single_line_set[] = {BOXDRAW_DOWN_RIGHT, BOXDRAW_HORIZONTAL,
                                BOXDRAW_DOWN_LEFT, BOXDRAW_VERTICAL,
                                BOXDRAW_UP_RIGHT, BOXDRAW_UP_LEFT};
static CHAR double_line_set[] = {BOXDRAW_DOWN_RIGHT_DOUBLE, 
                                BOXDRAW_DOUBLE_HORIZONTAL,
                                BOXDRAW_DOWN_LEFT_DOUBLE, 
                                BOXDRAW_DOUBLE_VERTICAL,
                                BOXDRAW_UP_RIGHT_DOUBLE, 
                                BOXDRAW_UP_LEFT_DOUBLE};

/********************************************************************
 * private routines 
 *******************************************************************/ 
 
void scrn_scroll(pScreen s, int lines)
{
    int cnt = s->area.lr.r - s->area.ul.r - lines;
    ScreenPtr dest = s->buffer + s->area.ul.r * s->csize + s->area.ul.c;
    ScreenPtr src = dest + lines * s->csize;
    int size = (s->area.lr.c - s->area.ul.c + 1) * sizeof(ScreenPtr);
    
    while (cnt--)
    {
        memcpy(dest,src,size);
        dest += s->csize;
        src += s->csize;
    }
        
    size /= sizeof(ScreenPtr);
    while (lines--)
    {
        dest = src;
        for (cnt = 0 ; cnt <= size; cnt++)
            (src++)->ch = ' ';
        src = dest + s->csize;
    }
};

void scrn_moveCursor(pScreen s, int step, int scroll)
{
    int width = s->area.lr.c - s->area.ul.c + 1;
    int height;
        
    s->cursor.c += step;

    if (s->cursor.c < 0 || s->cursor.c >= width)
    {
        if (!scroll)
        {
            s->cursorPtr = NULL;
            return;
        }
        
        while (s->cursor.c < 0) /* went off left limit */
        {
            if (!s->cursor.r)
                s->cursor.c = 0;
            else
            {
                s->cursor.r--;
                s->cursor.c += width;
            }
        }
        if (s->cursor.c >= width) /* went off right limit */
        {
            s->cursor.r += s->cursor.c / width;
            s->cursor.c %= width;
            height = s->area.lr.r - s->area.ul.r;
            if (s->cursor.r > height)
            {
                if (scroll)
                    scrn_scroll(s, s->cursor.r - height);
                s->cursor.r = height;
            }
        }
        scrn_gotoxy(s,s->cursor.r,s->cursor.c);
    }
    else
        s->cursorPtr += step;
}    

void scrn_writeCh(pScreen scrn, CHAR ch, int scroll)
{
    if (ch == '\n')
        scrn_moveCursor(scrn,scrn->area.lr.c-scrn->cursor.c+1,scroll);
    else
    {
        if (scrn->cursorPtr!=NULL)
        {
            scrn->cursorPtr->ch = ch;
            scrn->cursorPtr->attrib = scrn->attrib;
#ifdef EFI
            efi_settextcolor(scrn->attrib);
            efi_put_unicode(ch);
#endif
        }
        scrn_moveCursor(scrn,1,scroll);
    }
}

static boxTopBottom(pScreen scrn, int x1,int y1,int x2,int y2,CHAR st[])
{
    scrn_writeXYCh(scrn,x1,y1,st[0]);
    scrn_replicate(scrn,st[1],y2-y1-1);
    scrn_writeCh(scrn,st[2],0);
    scrn_writeXYCh(scrn,x2,y1,st[4]);
    scrn_replicate(scrn,st[1],y2-y1-1);
    scrn_writeCh(scrn,st[5],0);
}

static void boxOnly(pScreen scrn, int x1,int y1,int x2,int y2,CHAR st[])
{
    int i;

    boxTopBottom(scrn, x1,y1,x2,y2,st);
    for (i=x1+1;i<x2;i++) 
    {
        scrn_writeXYCh(scrn,i,y1,st[3]);
        scrn_writeXYCh(scrn,i,y2,st[3]);
    }
}

static void box(pScreen scrn, int x1,int y1,int x2,int y2,CHAR st[])
{
    int i;

    boxOnly(scrn, x1,y1,x2,y2,st);
    for (i=x1+1;i<x2;i++) 
    {
        scrn_gotoxy(scrn,i,y1+1);
        scrn_replicate(scrn,L' ',y2-y1-1);
    }
}

void refreshScreen(int x1, int y1, int x2, int y2)
{
    int i,j;
    CHAR ch;
    u08 old_attrib = SCREEN.attrib;
    ScreenPtr s_ptr = SCREEN.cursorPtr;
    for (i=x1; i<x2; i++)
        for (j=y1; j<y2; j++)
        {
            scrn_gotoxy(pSCREEN, i, j);
            ch = SCREEN.cursorPtr->ch;
            SCREEN.attrib = SCREEN.cursorPtr->attrib;
            scrn_writeCh(pSCREEN, ch, 0);
        }
    SCREEN.attrib = old_attrib;
    SCREEN.cursorPtr = s_ptr;
}


/********************************************************************
        Public routines 
 *******************************************************************/

/*
 * this routine returns the user scrn buffer pointer based on user point
 *
 * the parameter p is user point
 * 
 */
void scrn_userPointToScrnPoint(pScreen s, pScreenPoint up)
{
    up->r += s->area.ul.r;
    up->c += s->area.ul.c;
}

/*
void scrn_userPointToAbsPoint(pScreen s, pScreenPoint up)
{
    up->r += s->area.ul.r + s->pos.r;
    up->c += s->area.ul.c + s->pos.c;
} 

void scrn_scrnPointToAbsPoint(pScreen s, pScreenPoint sp)
{
    sp->r += s->pos.r;
    sp->c += s->pos.c;
} */
  
scrnPtr scrn_scrnPtr(pScreen s, ScreenPoint up)
{
    return (scrnPtr) s->buffer + (up.r+s->area.ul.r) * s->csize + up.c+s->area.ul.c;
}

void scrn_replicate(pScreen scrn,CHAR c,int len)
{
    while (len-->0) scrn_writeCh(scrn,c,0);
}

void scrn_printf(pScreen scrn, s08 *fmt, ...)
{
	va_list arglist;

    va_start (arglist, fmt);
    vsprintf (global.pbuf, fmt, arglist);
    scrn_print(scrn,global.pbuf);
    va_end (arglist);
}

void scrn_xyPrintf(pScreen scrn, int x, int y, s08 *fmt, ...)
{
	va_list arglist;

    scrn_gotoxy(scrn,x,y);
    va_start (arglist, fmt);
    vsprintf (global.pbuf, fmt, arglist);
    scrn_print(scrn,global.pbuf);
//    scrn_printNoScroll(scrn,global.pbuf);
    va_end (arglist);
}

void scrn_xyPrintfPad(pScreen scrn, int x, int y, int len, s08 *fmt, ...)
{
	va_list arglist;

    va_start (arglist, fmt);
    vsprintf (global.pbuf, fmt, arglist);
    scrn_xyPrintPad(scrn,x,y,global.pbuf,len);
    va_end (arglist);
}

void scrn_xyPrintPad(pScreen scrn, int row,int col,s08 * msg, int len)
{
    scrn_gotoxy(scrn,row,col);
    scrn_printPad(scrn,msg,len);
}

u08 scrn_setAttrib(pScreen scrn, u08 a)
{
  u08 o = scrn->attrib;

  scrn->attrib = a;
  return o;
}

void scrn_gotoxy(pScreen scrn, int x,int y)
{
    scrn->cursor.r = x;
    scrn->cursor.c = y;
    
    x += scrn->area.ul.r;
    y += scrn->area.ul.c;
    if (x<0 || x>=scrn->rsize || y<0 || y >= scrn->csize)
        scrn->cursorPtr = NULL;
    else
        scrn->cursorPtr = scrn->buffer + (x * scrn->csize + y);
#ifdef EFI
    efi_settextposition(x, y);
#endif
}

/*
 * in scrn - screen pointer
 *    p - absolute point
 */
/*
int scrn_inScreen(pScreen scrn, ScreenPoint p)
{
    / * convert to scrnPoint position * /
    p.r -= scrn->pos.r;
    p.c -= scrn->pos.c;
    return ((p.r >= 0) && (p.r < scrn->rsize) &&
            (p.c >= 0) && (p.c < scrn->csize));
} */

void scrn_clear(pScreen scrn)
{
    int r=scrn->area.lr.r-scrn->area.ul.r+1;
    int width = scrn->area.lr.c-scrn->area.ul.c+1;

    while (r)
    {
        scrn_gotoxy(scrn,--r,0);
        scrn_replicate(scrn,L' ',width);
    }
    scrn_gotoxy(scrn,0,0);
}

void scrn_setConsole(pScreen scrn, int r1, int c1, int r2, int c2)
{
    scrn->area.ul.r = r1;
    scrn->area.lr.r = r2;
    scrn->area.ul.c = c1;
    scrn->area.lr.c = c2;
    scrn_homeCursor(scrn);
}

void scrn_print(pScreen scrn, s08 *str)
{
    while (*str) scrn_writeCh(scrn, (CHAR)(u08)*str++, 1);
}

void scrn_printNoScroll(pScreen scrn, s08 *str)
{
    while (*str) scrn_writeCh(scrn, (CHAR)(u08)*str++, 0);
}

void scrn_xyPrint(pScreen scrn,int x, int y, s08 * str)
{
    scrn_gotoxy(scrn,x,y);
    scrn_printNoScroll(scrn,str);
}

void scrn_printPad(pScreen scrn, s08 * st, int len)
{
    int l=0;
    
    while (*st) 
    {
        if (l++==len)
            break;
        scrn_writeCh(scrn, (CHAR)(u08)*st++,0);
    }
    while (l++<len) scrn_writeCh(scrn, L' ',0);
}

void scrn_init()
{
    struct videoconfig v;
#ifdef EFI
    int i,j;
#endif
    
    _getvideoconfig(&v);
    __scrn.rsize = v.numtextrows;
    __scrn.csize = v.numtextcols;
    __scrn.area.lr.r = v.numtextrows - 1;
    __scrn.area.lr.c = v.numtextcols - 1;
    scrn_showCursor();
#ifdef EFI
    for (i=0;i<MAX_ROW;i++)
        for (j=0;j<MAX_COL;j++)
        {
            curr_scrn[i][j].ch = L' ';
            curr_scrn[i][j].attrib = ATTRIB(WHITE,BLACK);
        }
#endif
}

void scrn_box(pScreen scrn, line_t type, int ur,int uc, int lr, int lc)
{
    CHAR * boxstr = type == SINGLE_LINE ? single_line_set : double_line_set;
#if 0
    s08 * boxstr = type == SINGLE_LINE ? "\xda\xc4\xbf\xb3\xc0\xd9" : 
                                          "\xc9\xcd\xbb\xba\xc8\xbc";
#endif
    box(scrn,ur,uc,lr,lc,boxstr);
}

void scrn_boxOnly(pScreen scrn, line_t type, int ur,int uc, int lr, int lc)
{
    CHAR * boxstr = type == SINGLE_LINE ? single_line_set : double_line_set;
#if 0
    CHAR * boxstr = type == SINGLE_LINE ? "\xda\xc4\xbf\xb3\xc0\xd9" : 
                                          "\xc9\xcd\xbb\xba\xc8\xbc";
#endif
    boxOnly(scrn,ur,uc,lr,lc,boxstr);
}
void scrn_boxWithShadow(line_t type, int ur,int uc, int lr, int lc)
{
    scrn_box(pSCREEN,type,ur,uc,lr,lc);
    scrn_addShadow(ur,uc,lr,lc);
}

void scrn_boxOnlyWithShadow(line_t type, int ur,int uc, int lr, int lc)
{
    scrn_boxOnly(pSCREEN,type,ur,uc,lr,lc);
    scrn_addShadow(ur,uc,lr,lc);
}
/*
void scrn_center(int row,s08 *string,int w)
{
  int start_pos = 40 - w/2;
  int string_pos = 40 - strlen(string)/2;
  scrn_gotoxy(row,start_pos);
  scrn_replicate(L' ',string_pos-start_pos);
  scrn_print(string);
  scrn_replicate(L' ',string_pos-start_pos);
}

void scrn_right(int row,s08 *string,int w)
{
  scrn_gotoxy(row,w - strlen(string));
  scrn_print(string);
} */

/*
void scrn_print(s08 * string)
{
    while (*string) writeCh((CHAR)(u08)*string++);
}
*/

ScreenPtr scrn_dispMsg(s08 * st)
{
    int l = strlen(st);
    int attr = SCREEN.attrib;
    ScreenPtr buffer;

    if (global.Rs232)
    {
        com_puts(st);
    }
    buffer=scrn_saveScreen(MSGPOSITIONS);
    scrn_setAttrib(pSCREEN,RED*16+WHITE);
    scrn_boxWithShadow(SINGLE_LINE,5,38-l/2,7,42+l/2);
    scrn_xyPrint(pSCREEN,6,40-l/2,st);
    scrn_setAttrib(pSCREEN,attr);
    return buffer;
}

s08 scrn_dispWait(s08 *fmt, ...)
{
    ScreenPtr buffer;
    s08 rv;
	va_list arglist;

    va_start (arglist, fmt);
    vsprintf (global.pbuf, fmt, arglist);
    va_end (arglist);
    buffer=scrn_dispMsg(global.pbuf);
    rv=util_getch();
    scrn_restoreScreen(MSGPOSITIONS,buffer);
    free(buffer);
    return rv;
}

scrn_yes(s08 * msg)
{
    return (toupper(scrn_dispWait(msg))=='Y');
}


void scrn_showCursor()
{
#ifdef EFI
    efi_showcursor();
#else
   __asm
     {
        mov ah,1
        mov cx,0x090a
        int 0x10
     }
#endif
}

void scrn_showThickCursor()
{
#ifdef EFI
    efi_showcursor();
#else
   __asm
     {
        mov ah,1
        mov cx,0x070a
        int 0x10
     }
#endif
}

void scrn_hideCursor()
{
#ifdef EFI
    efi_hidecursor();
#else
   __asm
     {
        mov ah,1
        mov cx,0x2020
        int 0x10
     }
#endif
}

ScreenPtr scrn_saveScreen(int x1,int y1,int x2,int y2)
{
    int i;
    int unit = y2-y1+1;
    ScreenPtr buff,obuff;

    obuff = buff = (ScreenPtr) malloc((x2-x1+1)*unit*sizeof(ScreenNode));
    for (i=x1;i<=x2;i++) 
    {
        memcpy(buff,SCREEN.buffer+(i*SCREEN.csize)+y1,unit*sizeof(ScreenNode));
        buff = buff + unit;
    }
    return obuff;
}

ScreenPtr scrn_save()
{
    return scrn_saveScreen(0,0,SCREEN.rsize-1,SCREEN.csize-1);
}

void scrn_restoreScreen(int x1,int y1,int x2,int y2,ScreenPtr buff)
{
    int unit = y2-y1+1;
    int i;

    for (i=x1;i<=x2;i++) 
    {
        memcpy(SCREEN.buffer+(i*SCREEN.csize)+y1,buff,unit*sizeof(ScreenNode));
        buff = buff + unit;
    }
#ifdef EFI
    refreshScreen(x1, y1, x2, y2);
#endif
}

void scrn_restore(ScreenPtr buff)
{
    scrn_restoreScreen(0,0,SCREEN.rsize-1,SCREEN.csize-1,buff);
}

void scrn_addShadow(int r1, int c1, int r2, int c2)
{
    
    ScreenPtr ptr = SCREEN.buffer + ++r1 * SCREEN.csize + ++c2;

    if (c2 < SCREEN.csize-1)
    {
        while (r1++<=r2) 
        {
            if (r1>=SCREEN.rsize)
                break;
            ptr->attrib = 0x8;
            if (c2 < SCREEN.csize-2)
               (ptr+1)->attrib = 0x8;
            ptr += SCREEN.csize;
        }
    }
    
    if (++r2 < SCREEN.rsize)
    {
        ptr = SCREEN.buffer + r2 * SCREEN.csize + c1 + 1;
        if (++c2 >= SCREEN.csize)
            c2 = SCREEN.csize-1;
        while (c1++ < c2)
            (ptr++)->attrib = 0x8;
    }
}

void scrn_writeXYCh(pScreen scrn, int x1,int y1,CHAR c)
{
    scrn_gotoxy(scrn,x1,y1);
    scrn_writeCh(scrn,c,0);
}



#ifdef EFI
void scrn_updateScrnBuf(int r, int c, CHAR s[], u08 attrib)
{
    int i=0;
    while (*s)
    {
        curr_scrn[r][c+i].ch = *s;
        curr_scrn[r][c+i].attrib = attrib;
        s++;
        i++;
    }
}
#endif
