/********************************************************************
 * Copyright(c) 2000 Broadcom Corporation, all rights reserved.
 * Contains proprietary and confidential information.
 *
 * This source file is the property of Broadcom Corporation, and
 * may not be copied or distributed in any isomorphic form without 
 * the prior written consent of Broadcom Corporation. 
 *
 *  FileName: $Header: /Source/Bcm44xx/b44diag/t3_cli.h 7     4/25/02 11:08a Ahui $
 *
 *  Description: This file contains command line interface 
 *
 *  $Log: /Source/Bcm44xx/b44diag/t3_cli.h $ 
 * 
 * 7     4/25/02 11:08a Ahui
 * 
 * 3     3/07/02 5:05p Ahui
 * Added sromutil support
 ********************************************************************/ 
 
#ifndef __T3_CLI_H__
#define __T3_CLI_H__

#include "general.h"
#include "t3_cmd.h"

#define MAX_PARAMETER 20
#define MAX_PROMPT_LENGTH 40
#define COMMAND_LENGTH MAX_LINE_LENGTH

/****************************************************************************
 * cmd_line_interface()
 *    It prompt user the enter the command, and execute as nessary. The 
 *    concept is similar to a DOS prompt
 * 
 * input - command string. If the command string is NULL, it will 
 *         stay in the loop to get the prompt and excute. Otherwise, it use 
 *         the input string as command and only excute that parameter 
 * 
 * output - none
 * 
 * return - exit code 
 **************************************************************************/
int cmd_line_interface(s08 * para);


/****************************************************************************
 * cmd_setPrompt
 *    set new command prompt
 * 
 * input - new prompt
 *
 **************************************************************************/
void cmd_setPrompt(s08 * newPrompt);

/****************************************************************************
 * cmd_optionValue
 *    returns the option value
 * 
 * input : option character
 *
 * return the option value 
 *
 **************************************************************************/
optionValue_t cmd_optionValue(command_ptr cmdPtr, s08 option);

/****************************************************************************
 * cmd_optionEntered
 *    returns true the option is entered by user
 * 
 * input : option character
 *
 * return the option value 
 *
 **************************************************************************/
u08 cmd_optionEntered(command_ptr cmdPtr, s08 option);

/****************************************************************************
 * cmd_help
 *    print out command help
 * 
 * input : command structure 
 *
 **************************************************************************/
int cmd_help(command_ptr cmdPtr);

/****************************************************************************
 * cmd_prTitle
 *    print out command title
 * 
 * input : command structure 
 *
 **************************************************************************/
void cmd_prTitle(command_ptr cmdPtr);

/*
 * cmd_str2Parameter 
 *      convert input string to C parameter style argc and argu
 *
 * input: buff - input buffer string
 *
 * output: argc - argument acount
 *         argu - argument pointer
 */
void cmd_str2Parameter(s08 * buff, int * argc, s08 **argu);
 

/* 
 * cmd_executeString
 * 
 * input - command string
 *         cmd set
 *
 * return - exit Code
 */
int cmd_executeString(s08 * buff, command_ptr * cmds);

/*
 * cmd_disp_helps
 *    print out all commands available
 * 
 * input : command structure
 *
 * return : ERR_INVALID
 */
int cmd_disp_helps(command_ptr * Cmds, int startGrp, int endGrp);

/*
 * executeSelect
 *    execute the function based on global.blastCntl
 * 
 * input : the address of the function to be executed
 *         global.blastCntl 
 *
 * return : HAS_ERROR if error deteced
 *        : NO_ERROR if no error
 */
//int runSelect(int * (initFunc)(void),int * (loopFunc)(void),int * (cleanUpFunc)(void), int);

/*
 * cmd_SetIteration
 *    set the iteration number
 */
void cmd_SetIteration(command_ptr pCmd, int itr);

/****************************************************************************
 * cmd_clearOption
 *    clear all option value
 * 
 * input : command structure 
 *
 **************************************************************************/
void cmd_clearOption(command_ptr cmdPtr);

int executeSelect(int * (func)(void));

int cmd_getPrompt(s08 * buffer, u32 len);

#endif
