/********************************************************************
 * Copyright(c) 2000 Broadcom Corporation, all rights reserved.
 * Contains proprietary and confidential information.
 *
 * This source file is the property of Broadcom Corporation, and
 * may not be copied or distributed in any isomorphic form without 
 * the prior written consent of Broadcom Corporation. 
 *
 *  FileName: $Header: /Source/Bcm44xx/b44diag/t3_cmd.c 16    7/19/02 10:07a Ahui $
 *
 *******************************************************************/

#include <time.h>
#include <fcntl.h>
#include <stdio.h>
#include <sys/stat.h>
#include <ctype.h>
#ifndef EFI
#include <conio.h>
#include <io.h>
#endif
#include "Cl4xsb.h"
#include "t3_cli.h"
#include "t3_gdata.h"
#include "t3_eprom.h"
#include "t3_tests.h"
#include "protocol.h"
#include "fw_img.h"
#include "tigon3.h"
#include "t3_tx.h"
#include "t3_mii.h"
#include "t3_eecfg.h"
#include "t3_win.h"
#include "t3_intr.h"
#include "t3_intr.h"
#include "t_io.h"
#include "t3_diag.h"
#include "t3_flash.h"
#include "t3_util.h"
#include "t3_util.h"
#include "t3_var.h"
#include "sromutl.h"
#include "pci.h"


#define MEMORY_TYPE_CFG      "!"
#define MEMORY_TYPE_CFGB     "X"
#define MEMORY_TYPE_CFGW     "S"
#define MEMORY_TYPE_REG      "#"
#define MEMORY_TYPE_SRAM     "*"
#define MEMORY_TYPE_NVRAM    "$"
#define MEMORY_TYPE_MII      "m"
#define MEMORY_TYPE_VPD      "~"
#define MEMORY_TYPE_SCRATCH  "^"
#define MEMORY_TYPE_DIRECT32 "l"
#define MEMORY_TYPE_DIRECT16 "s"
#define MEMORY_TYPE_DIRECT8  "x"
#define MEMORY_TYPE_INDIRECT "I"

#define MEMORY32_TYPE MEMORY_TYPE_CFG\
		      MEMORY_TYPE_REG\
		      MEMORY_TYPE_SRAM\
		      MEMORY_TYPE_NVRAM\
		      MEMORY_TYPE_VPD\
		      MEMORY_TYPE_SCRATCH\
		      MEMORY_TYPE_DIRECT32\
		      MEMORY_TYPE_INDIRECT

#define MEMORY16_TYPE MEMORY_TYPE_MII\
		      MEMORY_TYPE_CFGW\
		      MEMORY_TYPE_DIRECT16
		      
#define MEMORY8_TYPE  MEMORY_TYPE_DIRECT8\
		      MEMORY_TYPE_CFGB

#define MEMORY_TYPE MEMORY32_TYPE""MEMORY16_TYPE""MEMORY8_TYPE

#define MEMORY_STRING_READ \
    "  "MEMORY_TYPE_CFG     " = Configuration space (32)\n"\
    "  "MEMORY_TYPE_CFGW    " = Configuration space (16)\n"\
    "  "MEMORY_TYPE_CFGB    " = Configuration space (8)\n"\
    "  "MEMORY_TYPE_REG     " = Registers (32) (default)\n"\
	"  "MEMORY_TYPE_MII     " = MII registers (16)\n"\
    "  "MEMORY_TYPE_DIRECT32" = direct access (32)\n"\
    "  "MEMORY_TYPE_DIRECT16" = direct access (16)\n"\
    "  "MEMORY_TYPE_DIRECT8 " = direct access (8)\n"
#define MEMORY_STRING_WRITE \
    "  "MEMORY_TYPE_CFG     " = Configuration space (32)\n"\
    "  "MEMORY_TYPE_CFGW    " = Configuration space (16)\n"\
    "  "MEMORY_TYPE_CFGB    " = Configuration space (8)\n"\
    "  "MEMORY_TYPE_REG     " = Registers (32) (default)\n"\
    "  "MEMORY_TYPE_DIRECT32" = direct access (32)\n"\
    "  "MEMORY_TYPE_DIRECT16" = direct access (16)\n"\
    "  "MEMORY_TYPE_DIRECT8 " = direct access (8)\n"



#define GET_NUMBER(p,v,ptr) \
{\
    if (p>=argc)\
    {\
	eprintf("missing parameter\n");\
	return ERR_INVALID;\
    }\
    if (!util_isDigit(*(ptr))) \
	return test_failed1(ERR_INVALID,"parameter, a number is expected");\
    v=strtoul(ptr,&(ptr),global.radix);\
}
s8* SromTable[] = {
						"functionEnabled",
						"commonPower",
						"SubsystemID",
						"SubVendorID",
						"deviceID",
						"classCode",
						"classCode",
						"PME/Pow",
						"b0l/pme/b1En&Sz",
						"bar0Value",
						"CISPointer",
						"CISPointer",
						"deviceID",
						"classCode",
						"classCode",
						"PME/Pow",
						"b0l/pme/b1En&Sz",
						"bar0Value",
						"CISPointer",
						"CISPointer",
						"deviceID",
						"classCode",
						"classCode",
						"PME/Pow",
						"b0l/pme/b1En&Sz",
						"bar0Value",
						"CISPointer",
						"CISPointer",
						"deviceID",
						"classCode",
						"classCode",
						"PME/Pow",
						"b0l/pme/b1En&Sz",
						"bar0Value",
						"CISPointer",
						"CISPointer",
						"wl0mac",
						"wl0mac",
						"wl0mac",
						"enet0mac",
						"enet0mac",
						"enet0mac",
						"wl1mac",
						"wl1mac",
						"wl1mac",
						"enetphy",
						"BdRev/ccLock",
						"wl0 PA b0",
						"wl0 PA b1",
						"wl0 PA b2",
						"wl0 PA b3",
						"wl0 PA b4",
						"wl0 Ant/MaxPower",
						"wl1 PA b0",
						"wl1 PA b1",
						"wl1 PA b2",
						"wl1 PA b3",
						"wl1 PA b4",
						"wl1 Ant/MaxPower",
						"OEM0",
						"OEM1",
						"OEM2",
						"OEM3",
						"Rev/CRC8",

};
s08 * cmd_grp_str[MAX_CMD_GRP] = {      
						"vpd",
						"seeprom",
						"cpu",
						"dma",
						"packet",
						"mii",
						"mem",
						"test",
						"power",
						"irq",
						"mac",
						"misc",
				    "asf"};
fn_info_t fn_list[] = {
	{ 0x4402, CLASS_ETHER, CLASS_NET, PME_4402_ENET, PMEREP_4402_ENET, 0 },
	{ 0xffff },
	{ 0xffff },
	{ 0xffff },
	{ 0x4401, CLASS_ETHER, CLASS_NET, PME_4402_ENET, PMEREP_4402_ENET, 0 },
	{ 0xffff },
	{ 0xffff },
	{ 0xffff },
	};
chip_info_t chip_info[] = {
	{ "BCM4402", 4, VENDOR_BROADCOM, 0x4402, 0xffff, HW_PI4402, HW_FUN4401, 0, 0, 0, 1, &fn_list[0] },
	{ "BCM4401", 4, VENDOR_BROADCOM, 0x4401, 0xffff, HW_PI4402, HW_FUN4401, 0, 0, 0, 1, &fn_list[4] },
	{ "UNKNOWN", 0, 0xffff, 0xffff, 0xffff, 0, 0, 0, 0, 0, 0, 0 }
	};                
/***************************************************************************
 ***************************************************************************
  Private functions 
 ***************************************************************************
***************************************************************************/

static void checkMemoryAlign(u08 type, u32 * addr)
{
    if (strchr(MEMORY32_TYPE,type))
	*addr &= 0xfffffffc;
    else if (strchr(MEMORY16_TYPE,type))
	*addr &= 0xfffffffe;
}

static int checkMemoryType(s08 ** l, u08 * type, u32 * addr)
{
    *type = '#';
    if (!util_isDigit (**l))
    {
	if(strchr(MEMORY_TYPE,**l)==NULL)
	{
	    eprintf ("bad memory specifier\n");
	    return ERR_INVALID;
	}
	*type = **l;
	(*l)++;
    }
    *addr = strtoul(*l,NULL,16);
    checkMemoryAlign(*type, addr);
    return NO_ERROR;
}


/*
 * checkRange 
 *    decode addresses
 *       possible format: nnnn             one token    (use default length)
 *                        nnnn-nnnn        one token    (range)
 *                        xxxx -xxxx       two tokens   (range)
 *                        xxxx- xxxx       two tokens   (range)
 *                        xxxx - xxxx      three tokens (range)
 *                        xxxx xxxx        two tokens   (second argument is length)
 * 
 *     in   pos - argument position start to check
 *         argc - number of arguments
 *         argu - argument list
 *         *len - default length value 
 *
 *   out   *addr - begin address
 *         *len  - length
 * 
 * return next position after check
 *        returns 0 if the begin address is not valid
 *
 */
static int checkRange(u08 * type, int pos, int argc, s08 **argu, u32 *addr, u32 *len, int base)
{
    s08 * str = argu[pos];
    int is_range = 0;
    
    if (!util_isDigit(*str))
    {
	eprintf("Invalid begin address\n");
	return 0;
    }    
    *addr = strtoul (str, &str, base);
    *len = 1;
    if (!*str) 
    {
	if (++pos == argc)
	    return pos;
	str = argu[pos];
    }
    if (*str=='-')
    {
	is_range = 1;
	str++;
    }
    if (!*str) 
    {
	if (++pos == argc)
	    return pos;
	str = argu[pos];
    }
    if (util_isDigit(*str))
    {
	*len = strtoul (str, &str, base);
	if (is_range)
	    *len = *len - *addr + 1;
	pos++;    
    }
    checkMemoryAlign(*type,addr);
    if (strchr(MEMORY32_TYPE,*type))
    {
	if (*len & 3)
	    *len = (*len & 0xfffffffc) + 4;
    }    
    else if (strchr(MEMORY16_TYPE,*type))
    {
	if (*len & 1)
	    *len = (*len & 0xfffffffe) + 2;
    }
    return pos;
}

/***************************************************************************
 ***************************************************************************
  command defines
 ***************************************************************************
***************************************************************************/

/************************************************************
 * command 'quit' 1  
 ************************************************************/

static command_t cmd_quit = {
    "quit",             /* command      */
    "",                 /* syntax       */
    NULL,               /* option       */
    "Exit the System",  /* title        */
    "",
    NULL,               /* command      */
    CMD_GRP_MISC
};

/************************************************************
 * command 'exit'  2 
 ************************************************************/

static command_t cmd_exit = {
    "exit",             /* command      */
    "",                 /* syntax       */
    NULL,               /* option       */
    "Exit the System",  /* title        */
    "",
    NULL,               /* command      */
    CMD_GRP_MISC
};

/************************************************************
 * command 'semode' 3  
 ************************************************************/

/************************************************************
 * command 'seread'  4 
 ************************************************************/

/************************************************************
 * command 'sewrite'  5 
 ************************************************************/
 
/************************************************************
 * command 'secfg'  6 
 ************************************************************/

/************************************************************
 * command 'vpdwrite' 7  
 ************************************************************/

/************************************************************
 * command 'vdpread'  8 
 ************************************************************/

/************************************************************
 * command 'vdptest'  9 
 ************************************************************/

/************************************************************
 * command 'regtest'  10 
 ************************************************************/

static int func_regtest(int argc, s08 **argu);
static option_t opt_regtest[] = {
    { 'i', OPT_BOOL, "Also run indirect memory test"},
    { 'n', OPT_LOOP,  "iteration"},
    END_OF_OPTION
};

static command_t cmd_regtest = {
    "regtest",          /* command */
    "",                 /* syntax  */
    opt_regtest,        /* option  */
    "Run Register Test",
    "The default iteration is 1. 0 means run forever",
    func_regtest,
    CMD_GRP_TEST
};

static int func_regtest(int argc, s08 **argu)
{
	return SB4xDiagTestRegisters(1, cmd_optionEntered(&cmd_regtest, 'i'));
}    

/************************************************************
 * command 'seprg'    11
 ************************************************************/

static int func_seprg(int argc, s08 **argu);
static option_t opt_seprg[] = {
    { 'o', OPT_U32,  "offset of serial eeprom",0},
    { 'l', OPT_U32,  "length in bytes (Default = size of input file)",NO_DEFAULT},
    { 'f', OPT_STR,  "filename",NULL},
    END_OF_OPTION
};

static command_t cmd_seprg = {
    "seprg",            // command
    "<filename>",       // syntax  
    opt_seprg,          // option  
    "Program SEEPROM",
    "The function reads data from file and program into seeprom. The file name\n must be specified in the parameter.",
    func_seprg,
    CMD_GRP_SEEPROM
};

static int func_seprg(int argc, s08 **argu)
{
    BYTE * buffer = NULL;
    u32 offset = Value32(cmd_seprg,'o');
    u32 len = Value32(cmd_seprg,'l');
    int retc;
    s08 * fn;
    
    GetFilename(cmd_seprg, fn, 1);
    if ((retc=util_readFileMemAlloc(fn, offset, len, SEEPROM_TOTAL_SIZE, 
			   &len, &buffer, 1))==NO_ERROR)
    {
		eecfg_write(2, buffer, offset, len);
		free(buffer);
		printf("\n");
    }

    return NO_ERROR;    
}

/************************************************************
 * command 'memsearch'  12 
 ************************************************************/

/************************************************************
 * command 'miitest'  13 
 ************************************************************/
static int func_miitest(int argc, s08 **argu);

static option_t opt_iter[] = {
    { 'n', OPT_LOOP,  "iteration"},
    END_OF_OPTION
};

static command_t cmd_miitest = {
    "miitest",                          /* command */
    "",                                 /* syntax  */
    opt_iter,                           /* option  */
    "Run MII Memory Test",
    "The default iteration is 1. 0 means run forever",
    func_miitest,
    CMD_GRP_TEST
};

static int func_miitest(int argc, s08 **argu)
{
	return SB4xDiagTestMii (1);
}

/************************************************************
 * command 'debug'    14
 ************************************************************/

/************************************************************
 * command 'maclpk'  15 
 ************************************************************/

/************************************************************
 * command 'msi'   16
 ************************************************************/

/************************************************************
 * command 'memtest' 17  
 ************************************************************/

/************************************************************
 * command 'intrtest'  18
 ************************************************************/

static int func_intrtest(int argc, s08 **argu);
static command_t cmd_intrtest = {
    "intrtest",                        /* command */
    "",                     /* syntax  */
    opt_iter,                          /* option  */
    "Run Interrupt Test",
    "The default iteration is 1. 0 means run forever",
    func_intrtest,
    CMD_GRP_TEST
};

static int func_intrtest(int argc, s08 **argu)
{
    return test_runTest(1, GROUP_REG, REG_INTERRUPT, NULL);
}

/************************************************************
 * command 'bist'     19
 ************************************************************/
static int func_bist(int argc, s08 **argu);
static command_t cmd_bist = {
    "bist",                        /* command */
    "",                            /* syntax  */
    NULL,                          /* option  */
    "Run BIST",
    "The default iteration is 1. 0 means run forever",
    func_bist,
    CMD_GRP_TEST
};

static int func_bist(int argc, s08 **argu)
{
    return SB4xDiagTestIntMemory(1);
}

/************************************************************
 * command 'setest'   20
 ************************************************************/
static int func_setest(int argc, s08 **argu);
static command_t cmd_setest = {
    "setest",                        /* command */
    "",                              /* syntax  */
    opt_iter,                        /* option  */
    "Run SEEPROM Test",
    "The default iteration is 1. 0 means run forever",
    func_setest,
    CMD_GRP_TEST
};

static int func_setest(int argc, s08 **argu)
{
    return SB4xDiagTestEEPROM(1);
}

/************************************************************
 * command 'cmd_nictest' 21
 ************************************************************/

static int func_nictest(int argc, s08 **argu);
static command_t cmd_nictest = {
    "nictest",                     // command 
    "",                 // syntax  
    opt_iter,                      // option  
    "",//"Run a set of NIC Tests",
    "",//"The default iteration is 1. 0 means run forever",
    func_nictest,
    CMD_GRP_TEST
};

static int func_nictest(int argc, s08 **argu)
{
	int temp;
    global.manuf = !global.manuf;
	temp= test_doAllTests(1, 1 << get_currcard_index(), 0, 0);
	global.manuf = !global.manuf;
	return temp;
}

/************************************************************
 * command 'nicstats'  22
 ************************************************************/

static int func_nicstats(int argc, s08 **argu);
static option_t opt_nicstats[] = {
    { 'c', OPT_BOOL, "Clear Statististics"},
    END_OF_OPTION
};

static command_t cmd_nicstats = {
    "nicstats",                     // command 
    "",                             // syntax  
    opt_nicstats,                   // option  
    "",//"Display NIC Statistics",
    "",
    func_nicstats,
    CMD_GRP_PACKET
};


static int func_nicstats(int argc, s08 **argu)
{
    if (cmd_optionEntered(&cmd_nicstats,'c'))
	{
		resetcountersBlast();
	}
	else
	{
		displaycountersBlast(0);
	}
	return(NO_ERROR);
	
}        

/************************************************************
 * command 'ringindex'  23
 ************************************************************/

/************************************************************
 * command 'blast'  24  -- doesn't work......
 ************************************************************/

static int func_blast(int argc, s08 **argu);
option_t opt_blast[] = {
	{ 'p', OPT_DEC ,  "packet type: 0 = all 0s, 1 = all 1s\n             2 = 5555, 3 = AAAA\n             4 = 0F0F, 5 = F0F0\n             6 = FF00, 7 = 00FF\n             8 = FFFF0000, 9 = 0000FFFF\n            10 = Inc, 11 = Random"},
    { 'l', OPT_DEC , "Length of Tx packet",60},
    { 't', OPT_BOOL, "Enable Tx"},
    { 'r', OPT_BOOL, "Enable Rx"},
    { 'h', OPT_BOOL, "Enable Host Loopback"},
    { 'i', OPT_BOOL, "Increment length"},
    { 'n', OPT_DEC , "Number of packets to be transmitted",0},
    { 'e', OPT_DEC , "Upper Limit of Tx buffer in incremental packet size",MAX_ETHERNET_PACKET_SIZE_NO_CRC},

    END_OF_OPTION
};

static command_t cmd_blast = {
    "blast",                     // command 
    "",                          // syntax  
    opt_blast,                   // option  
    "",//"Blast Packets in Poll Mode",
    "",
    func_blast,
    CMD_GRP_PACKET
};

int func_blast(int argc, s08 **argu)
{   

	return SB4xDiagTestBlast(cmd_optionValue(&cmd_blast,'p').valueInt);
}

/************************************************************
 * command 'tprot'  25
 ************************************************************/

/************************************************************
 * command 'intrctrl' 26
 ************************************************************/

/************************************************************
 * command 'gpiowrite'  27
 ************************************************************/

/************************************************************
 * command 'gpioread'  28
 ************************************************************/

/************************************************************
 * command 'pxecpy'  29
 ************************************************************/

/************************************************************
 * command 'pkttest'  30
 ************************************************************/

/************************************************************
 * command 'loadfw'  31
 ************************************************************/

/************************************************************
 * command 'cputest'  32
 ************************************************************/

/************************************************************
 * command 'dmaw'  33
 ************************************************************/

/************************************************************
 * command 'dmatest' 34
 ************************************************************/

/************************************************************
 * command 'dmar'   35
 ************************************************************/

/************************************************************
 * command 'mbuf' 36
 ************************************************************/

/************************************************************
 * command 'upgfrm'  37
 ************************************************************/

/************************************************************
 * command 'txpkt'   38
 ************************************************************/

/************************************************************
 * command 'device'   39
 * command 'switch'   109
 ************************************************************/
static int func_device(int argc, s08 **argu);

static option_t opt_device[] = {
    { 'n', OPT_U32 , "Device Number"},
    END_OF_OPTION
};

static command_t cmd_device = {
    "device",               /* command */
    "<dev>",                /* syntax  */
    opt_device,                   /* option  */
    "Show or Switch Device",
    "If no parameter is entered, it will display all device available.",
    func_device,
    CMD_GRP_MISC
};

static int func_device(int argc, s08 **argu)
{
    u32 dev;
    
    if (cmd_optionEntered(&cmd_device,'n'))
	dev = cmd_optionValue(&cmd_device,'n').value32;
    else if (argc > 1)
	dev = atoi(argu[1]);
    
    if (argc == 1 && !cmd_optionEntered(&cmd_device,'n'))
	util_select_devices();
    else
	{
#ifdef WIN_CLI
	if (!isdigit(argu[1][0]) || !util_activateDevice(atoi(argu[1])))
	    scrn_dispWait("Cannot switch to device %s",argu[1]);
#else            
	util_switch_card (dev);
		util_show_device(get_currcard_index());
#endif        
	}
    return NO_ERROR;
}

/************************************************************
 * command 'version'   40
 ************************************************************/
static int func_version(int argc, s08 **argu);
static command_t cmd_version = {
    "version",               /* command */
    "",
    NULL,                   /* option  */
    "Display Program Version",
    "",
    func_version,
    CMD_GRP_MISC
};

static int func_version(int argc, s08 **argu)
{
	util_print_banner();
	return NO_ERROR;
}

/************************************************************
 * command 'loaddrv'   41
 ************************************************************/

/************************************************************
 * command 'unloaddrv'   42
 ************************************************************/

/************************************************************
 * command 'help'   43, 44
 ************************************************************/
static int func_help(int argc, s08 **argu);
command_t cmd_help1 = {
    "help",               /* command */
    "[seeprom|mii|test|mac|misc]",
	//"[vpd|seeprom|cpu|dma|packet|mii|mem|test|power|irq|mac|misc]",
    NULL,                   /* option  */
    "Display the Commands Available",
    "Enter command group discribed above for the list of commands available.\n"\
    "If no parameter is entered, all commands are displayed. Example: help seeprom\n"\
    "For each command help, type the command and then '?'. Example: sromutil ?",
    func_help,
    CMD_GRP_MISC
};


static command_t cmd_help2 = {
    "?",  /* command */
	"[seeprom|mii|test|mac|misc]",
    //"[vpd|seeprom|cpu|dma|packet|mii|mem|test|power|irq|mac|misc]",
    NULL,                   /* option  */
    "Alternate Help Command",
    "This is same command as 'help' command",
    func_help,
    CMD_GRP_MISC
};

static int func_help(int argc, s08 **argu)
{
    int cnt;
    int grp = 0, endGrp=MAX_CMD_GRP-1;
    int len;
    
    if (argc==2)
    {
	len = strlen(argu[1]);
	for (cnt=0; cnt<MAX_CMD_GRP; cnt++)
	    if (!strncmp(argu[1],cmd_grp_str[cnt],len))
	    {
		grp = endGrp = cnt;
		break;
	    }
	if (cnt==MAX_CMD_GRP)
	    return test_failed1(ERR_INVALID,"parameter");     
    }    
    
    return cmd_disp_helps(cmds, grp, endGrp);
}

/************************************************************
 * command 'phyctrl'   45
 ************************************************************/

static int func_phyctrl(int argc, s08 **argu);
static option_t opt_phyctrl[] = {
    { 's', OPT_U32 , "0:10 Mbps, 1:100 Mbps",T3_SPEED_1000MBPS},
    { 'h', OPT_BOOL,  "force half duplex"},
    { 'r', OPT_BOOL, "reset PHYs"},
    { 'f', OPT_STR , "file contains initialization scripts"},
    END_OF_OPTION
};

static command_t cmd_phyctrl = {
    "phyctrl",                   
    "",
    opt_phyctrl,                   
    "Configure Speeds/Duplex",
    "",
    func_phyctrl,
    CMD_GRP_PACKET
};

static int func_phyctrl(int argc, s08 **argu)
{

    u16 reg;
	u16 val16_19;
    u32 offset;
    FILE *ifile;
    s08 string[81];
	u16 LinkUp = 0x4;

    u32 speed = Value32(cmd_phyctrl,'s');  // max_number_fragments 
    
    if (argc > 1)
		return cmd_help(&cmd_phyctrl);

    if (speed > 3)
		return test_failed1(ERR_INVALID,"speed paramters");

    if (cmd_optionEntered(&cmd_phyctrl,'r')) // reset phy? 
    {
		if (util_resetPhy()!=NO_ERROR)
			return ERR_INVALID;
    }

    if (cmd_optionEntered(&cmd_phyctrl,'f')) // file option ? 
    {
		if ((ifile = fopen(cmd_optionValue(&cmd_phyctrl,'f').valueStr,"r")) == NULL)
			return test_failed1(ERR_FILE_OPEN,cmd_optionValue(&cmd_phyctrl,'f').valueStr);
        
	
		// Set pointer to beginning of file: 
		fseek(ifile, 0L, SEEK_SET );
		while( !feof( ifile ) )     
		{
			fscanf( ifile, "%s", string );
			if(strcspn(string,"//") == 0)
				fscanf( ifile, "%s", string );
			else
			{   
				offset = strtoul(string,NULL,16);
				fscanf( ifile, "%s", string );
				reg = strtoul(string,NULL,16);
				if (offset > 31)
					break;

				printf("0x%02x <-- 0x%04x\n",offset,reg);
				miiwr(offset, reg);
			}
		}
		fclose(ifile);
    }
    
    switch (speed)
    {
	case T3_SPEED_10MBPS:
		if (cmd_optionEntered(&cmd_phyctrl,'h')) // HALF_DUPLEX 
		{
			miiwr(4,0x21);
			miiwr(0,0x1300);
			RegWr(REGOFFSET(txcontrol), RegRd(REGOFFSET(txcontrol)) & ~TC_FD);
		}
		else
		{
			miiwr(4,0x61);
			miiwr(0,0x1300);
			RegWr(REGOFFSET(txcontrol), RegRd(REGOFFSET(txcontrol)) | TC_FD);
		}
	    break;

	case T3_SPEED_100MBPS:
	    if (cmd_optionEntered(&cmd_phyctrl,'h')) // HALF_DUPLEX 
		{
			miiwr(4,0x81);
			miiwr(0,0x1300);
			RegWr(REGOFFSET(txcontrol), RegRd(REGOFFSET(txcontrol)) & ~TC_FD);
		}
		else
		{
			miiwr(4,0x1E1);
			miiwr(0,0x1300);
			RegWr(REGOFFSET(txcontrol), RegRd(REGOFFSET(txcontrol)) | TC_FD);
		}
	    break;
	    break;

	default:
	    break;
    }
    
	val16_19 = ((miird(0x19) & 0x700) >> 8);
	miird(1);
	if (miird(1) & LinkUp)
	{
		printf("Phy Link Status: Link Up\n");
		switch (val16_19)
		{
			case 1:
				printf("Phy Link Speed: 10 BT Half Duplex\n");
				break;
			case 2:
				printf("Phy Link Speed: 10 BT Full Duplex\n");
				break;
			case 3:
				printf("Phy Link Speed: 100 BT Half Duplex\n");
				break;
			case 4:
				
				break;
			case 5:
				printf("Phy Link Speed: 100 Full Duplex\n");
				break;
			default:
				break;
		}
	}
	else
		printf("Phy Link Status: Link Down\n");

    return NO_ERROR;
}

/************************************************************
 * command 'statusblk'   46
 ************************************************************/

/************************************************************
 * command 'dma_h'   47
 ************************************************************/

/************************************************************
 * command 'dma_d'   48
 ************************************************************/

/************************************************************
 * command 'machalt'   49
 ************************************************************/

/************************************************************
 * command 'read'   50
 *  Prints out memory (posiibly range).
 *  Format is "read [~!#*$%^]<begin_addr> [- end_addr | num_bytes]"
 *  where ~ = Flash space
 *        ! = Configuration space
 *        # = Registers (default)
 *        * = SRAM
 *        $ = Serial EEPROM
 *        % = parallel EEPROM
 *        m = MII registers
 *        ^ = internal scratchpad
 *        d = direct access
 *  Examples:
 *        read *4 16    : read sram starting at 4 ending at 20 (16 bytes)
 *        read ~4-16    : read sram starting at 4 ending at 16 (12 bytes)
 *        read 0        : read register 0
 ************************************************************/
static int func_read(int argc, s08 **argu);
static command_t cmd_read = {
    "read",               /* command */
    "["MEMORY_TYPE"]<begin> [-end | len]",
    NULL,                   /* option  */
    "Read Memory",
    MEMORY_STRING_READ\
    "\nexample: read *4 16    : read sram starting at 4 ending at 20 (16 bytes)\n"\
      "         read ~4-16    : read flash starting at 4 ending at 16 (12 bytes)\n"\
      "         read 0        : read register 0",
    func_read,
    CMD_GRP_MEM
};


static int func_read(int argc, s08 **argu)
{
	u32 addr, end, len=1;
	s08 type;
    
    if (argc>=2 && argc<4)
    {
	if (checkMemoryType(&argu[1], &type, &addr) != NO_ERROR)
	    return ERR_INVALID;
	
	if (type=='I')
	{
	    addr = t_readmem32(addr);
	    type = 'l';
	    len = (argc==3) ? strtoul(argu[2],NULL,16) : 4;
	} 
	else 
	{
	    if (!checkRange(&type, 1, argc, argu, &addr, &len, global.radix))
		return ERR_INVALID;
	}
	    
	end = addr + len - 1;
	if (type == '#')
		util_print_tabular(addr,end,util_read_reg);
	else if (type == '!')
	    util_print_tabular(addr, end, (u32 (*)(u32))t_readconf32);
	else if (type == 'S')
	    util_printTabularShort(addr, end,(u32 (*)(u32))t_readconf16);
	else if (type == 'X')
	    util_printTabularByte(addr, end,(u32 (*)(u32))t_readconf08);    
	else if (type == '$')
	    util_print_tabular (addr, end, eprom_read_seprom);
	else if (type == 'm')
	    util_print_miiregs(addr, end);
	else if (type == 'l')
	    util_print_tabular(addr, end, util_read_direct_mem);
	else if (type == 's')
	    util_printTabularShort(addr, end, util_read_direct_mem_short);
	else if (type == 'x')
	    util_printTabularByte(addr, end, util_read_direct_mem_byte);    
	return NO_ERROR;
    }
    return cmd_help(&cmd_read);
}

/************************************************************
 * command 'write'   51
 *  Prints out memory (posiibly range).
 *  Format is "write [~!#*$%^]<begin_addr> [- end_addr | num_bytes] data"
 *  where ~ = Flash space
 *        ! = Configuration space
 *        # = Registers (default)
 *        * = SRAM
 *        $ = Serial EEPROM
 *        % = parallel EEPROM
 *        m = MII registers
 *        ^ = internal scratchpad
 *        d = direct access
 ************************************************************/
static int func_write(int argc, s08 **argu);
static command_t cmd_write = {
    "write",               /* command */
    "["MEMORY_TYPE"]<address> [-end | len] data",
    NULL,                   /* option  */
    "Write Memory",
    MEMORY_STRING_WRITE,
    func_write,
    CMD_GRP_MEM
};


static int func_write(int argc, s08 **agu)
{
    s08 * argu[MAX_PARAMETER];
	u32 begin, end, len=1, writeval;
	s08 type;
    s08 pos;
    
    for (pos=0;pos<argc;pos++)
	argu[pos] = agu[pos];
	
    if (argc>=3 && argc<5)
    {
	argc--;
	GET_NUMBER(0, writeval, argu[argc]);

	if (checkMemoryType(&argu[1], &type, &begin) != NO_ERROR)
	    return ERR_INVALID;

	if (type=='I')
	{
	    begin = t_readmem32(begin);
	    type = 'l';
	    len = (argc==3) ? strtoul(argu[2],NULL,16) : 4;
	} 
	else 
	{
	    if (!checkRange(&type, 1, argc, argu, &begin, &len, global.radix))
		return ERR_INVALID;
	}

	end = begin + len - 1;
	if (type == '#')
	   util_fill_reg (begin, end, writeval);
	else if (type == '!')
	   util_fill_conf (begin, end, writeval, 4);
	else if (type == 'S')
	   util_fill_conf (begin, end, writeval, 2);
	else if (type == 'X')
	   util_fill_conf (begin, end, writeval, 1);
	else if (type == '$')
	   eprom_fill_blk (begin, end, writeval);
	else if (type == 'l')
	   util_fill_direct_mem(begin, end,writeval,4);
	else if (type == 's')
	   util_fill_direct_mem(begin, end,writeval,2);
	else if (type == 'x')
	   util_fill_direct_mem(begin, end,writeval,1);
		
	return NO_ERROR;
    }
    return cmd_help(&cmd_write);
}

/************************************************************
 * command 'radix'   52
 *
 * sets input global.radix (octal=8, binary=2, decimal=10, hex=16, note that these
 * numbers are all expressed in decimal).  Setting global.radix to zero, allows
 * reading of hex (prepended "0x"), octal (prepended 0) or decimal (default).
 *
 ************************************************************/
static int func_radix(int argc, s08 **argu);
static command_t cmd_radix = {
    "radix",               /* command */
    "[new radix]",
    NULL,                   /* option  */
    "Change System Radix",
    "Radix must be 0, 2-36. Radix used for number entry. 16 means enter number\n"\
    "in hex, and 10 means in decimal.",
    func_radix,
    CMD_GRP_MISC
};

static int func_radix(int argc, s08 **argu)
{
    int val;
    
    if (argc>2)
	return cmd_help(&cmd_radix);
    if (argc==2)
    {
	if (!util_isDigit(*argu[1]))
	    return test_failed1(ERR_INVALID,"parameter, a number is expected!");
	val = atoi(argu[1]);
	if (val && (val > 36 || val < 2))
	{
	    printf ("Radix must be 2-36, or 0\n");
	    return ERR_INVALID;
	}
	if (val && val != 10 && val != 16 && val != 8 && val != 2)
	{
	    printf("you are setting the global.radix to %u!?!(decimal) Ok, but that's wierd.\n",val);
	    printf ("(BTW 'global.radix 0' in any global.radix restores a sane state)\n\n");
	}
	global.radix = val;
    }
    printf("Radix = %u\n",global.radix);
    return NO_ERROR;
}

/************************************************************
 * command 'nolog'   53
 ************************************************************/
static int func_nolog(int argc, s08 **argu);
static command_t cmd_nolog = {
    "nolog",               /* command */
    "",
    NULL,                   /* option  */
    "Close the Current Logfile",
    "",
    func_nolog,
    CMD_GRP_MISC
};

static int func_nolog(int argc, s08 **argu)
{
	if (global.logfile)
	{
	time_t t = time(NULL);
	fprintf (global.logfile, "\n---------------------------------------------\n");
	printf ("logfile closed at %s",asctime(localtime(&t)));
	fclose (global.logfile);
	global.logfile = NULL;
	}
	return NO_ERROR;
}

/************************************************************
 * command 'log'   54
 ************************************************************/
static option_t opt_filename[] = {
    { 'f', OPT_STR,  "filename (for bcmediag compatibility only)",NULL},
    END_OF_OPTION
};
static int func_log(int argc, s08 **argu);
static command_t cmd_log = {
    "log",               /* command */
    "<filename>",
    opt_filename,        /* option  */
    "Open Logfile",
    "",
    func_log,
    CMD_GRP_MISC
};

static int func_log(int argc, s08 **argu)
{
    s08 * fn;
    
	if (global.logfile)
	{
	    printf ("Logfile already open!\n");
	    return 1;
	}
    GetFilename(cmd_log, fn, 1);
    return util_openLog(fn);
}

/************************************************************
 * command 'pciinit'   55
 ************************************************************/
static int func_pciinit(int argc, s08 **argu);
static command_t cmd_pciinit = {
    "pciinit",               /* command */
    "",
    NULL,                   /* option  */
    "Initialize PCI Configuration Registers",
    "",
    func_pciinit,
    CMD_GRP_MISC
};

static int func_pciinit(int argc, s08 **argu)
{
    return util_pcicfg_init();
}

/************************************************************
 * command 'pciscan'   56
 ************************************************************/
static int func_pciscan(int argc, s08 **argu);
static command_t cmd_pciscan = {
    "pciscan",               /* command */
    "",
    NULL,                   /* option  */
    "Scan for All PCI Devices",
    "",
    func_pciscan,
    CMD_GRP_MISC
};

static int func_pciscan(int argc, s08 **argu)
{
    return util_pci_scan();
}

/************************************************************
 * command 'sever'   57
 ************************************************************/

/************************************************************
 * command 'sever'   58
 ************************************************************/

/************************************************************
 * command 'dos'   59
 ************************************************************/
command_t cmd_dos = {
    "dos",               /* command */
    "<dos command>",
    NULL,                   /* option  */
    "Execute DOS command",
    "If no parameter is entered, DOS shell is entered.",
    NULL,
    CMD_GRP_MISC
};

/************************************************************
 * command 'diagcfg'   60
 ************************************************************/

/************************************************************
 * command 'ftq'   61
 ************************************************************/

/************************************************************
 * command 'addmc'   62
 ************************************************************/

/************************************************************
 * command 'delmac'   63
 ************************************************************/

/************************************************************
 * command 'mwrite'   64
 ************************************************************/
static int func_mwrite(int argc, s08 **argu);
static command_t cmd_mwrite = {
    "mwrite",               /* command */
    "<addr> <data>",
    NULL,                   /* option  */
    "Write PHY registers via MII Management interface",
    "",
    func_mwrite,
    CMD_GRP_MII
};

static int func_mwrite(int argc, s08 **argu)
{
    u32 ad, writeval;
    if (argc==3)
    {
	ad = strtoul(argu[1], &argu[1], global.radix);
	writeval = strtoul(argu[2], &argu[2], global.radix);
	util_fill_miiregs(ad, ad, writeval);
	return 0;
    }
    return cmd_help(&cmd_read);
}

/************************************************************
 * command 'mread'   65
 ************************************************************/
static int func_mread(int argc, s08 **argu);
static command_t cmd_mread = {
    "mread",               /* command */
    "<begin> [-<end> | <len>]",
    NULL,                   /* option  */
    "Read PHY registers via MII Management interface",
    "",
    func_mread,
    CMD_GRP_MII
};

static int func_mread(int argc, s08 **argu)
{
    u32 val, len;
    
    if (argc==1)
	util_print_miiregs(0,0x1f); /* display all */
    else if (argc>=2 && argc<4)
    {
	if (!checkRange(MEMORY_TYPE_MII, 1, argc, argu, &val, &len, global.radix))
	    return 1;
	util_print_miiregs(val, val + len - 1);
    }
    else 
	return cmd_help(&cmd_read);
    return NO_ERROR;
}

/************************************************************
 * command 'mdev'   66
 ************************************************************/

/************************************************************
 * command 'miimode'   67
 ************************************************************/

/************************************************************
 * command 'reset'   68
 ************************************************************/

static int func_reset(int argc, s08 **argu);
static command_t cmd_reset = {
    "reset",               
    "",
    NULL,                   
    "Reset Chip",
    "",
    func_reset,
    CMD_GRP_PACKET
};

static int func_reset(int argc, s08 **argu)
{
    reset();
	return NO_ERROR;
}

/************************************************************
 * command 'stsblk'   69
 ************************************************************/

/************************************************************
 * command 'cpudtt'   70
 ************************************************************/

/************************************************************
 * command 'cpudrt'   71
 ************************************************************/

/************************************************************
 * command 'txcfg'   72
 ************************************************************/

/************************************************************
 * command 'rxcfg'   73
 ************************************************************/

/************************************************************
 * command 'intr'   74
 ************************************************************/

/************************************************************
 * command 'pmdcfg'   75
 ************************************************************/

/************************************************************
 * command 'pmpcfg'   76
 ************************************************************/

/************************************************************
 * command 'pmpd'   77
 ************************************************************/

/************************************************************
 * command 'teste'   78
 ************************************************************/
static int func_teste(int argc, s08 **argu);
static command_t cmd_teste = {
    "teste",               /* command */
    "[<tests> [<tests> ...]]",
    NULL,                   /* option  */
    "Enable Test",
    "The command enables tests. It effects nictest, regtest, pkttest, and memtest\n"\
    "commands. The test must starts with an alpa. If no number is entered, all tests\n"
    "in that group are enabled. For example,\n\n"\
    "teste a12bc        -- Enable test a1, a2, all tests in group b and c\n"\
    "teste ab cd        -- Enables all tests\n"\
    "teste              -- Display enabled tests",
    func_teste,
    CMD_GRP_TEST
};

static void showTests(int enable)
{
    int grp, id;
    
    for (grp = 0; grp < MAX_TEST_GROUP; grp++)
	for (id = 0; id < test_maxTest(grp); id++)
	    if ((test_enabled(grp,id) && enable) ||
		(!test_enabled(grp,id) && !enable))
	    {    
		printf("%c%d. %s Test\n",grp+'A', id+1, test_title(grp,id));
	    }
}

static int func_teste(int argc, s08 **argu)
{
    int pos = 1;
    
    while (pos < argc)
    {
	if (test_checkTestStr(1,argu[pos++])!=NO_ERROR)
	    return 1;
    }
    printf("\nEnabled Tests:\n\n");
    showTests(1);
    return 0;
}

/************************************************************
 * command 'testd'   79
 ************************************************************/
static int func_testd(int argc, s08 **argu);
static command_t cmd_testd = {
    "testd",               /* command */
    "[<tests> [<tests> ...]]",
    NULL,                   /* option  */
    "Disable Test",
    "The command disables tests. It effects nictest, regtest, pkttest, and memtest\n"\
    "commands. The test must starts with an alpa. If no number is entered, all tests\n"
    "in that group are disabled. For example,\n\n"\
    "testd a12bc        -- disable test a1, a2, all tests in group b and c\n"\
    "testd ab c        -- disables all tests\n"\
    "testd              -- display disabled tests",
    func_testd,
    CMD_GRP_TEST
};

static int func_testd(int argc, s08 **argu)
{
    int pos = 1;
    
    while (pos < argc)
    {
	if (test_checkTestStr(0,argu[pos++])!=NO_ERROR)
	    return 1;
    }
    printf("\nDisabled Tests:\n\n");
    showTests(0);
    return 0;
}

/************************************************************
 * command 'reg'   80
 ************************************************************/

/************************************************************
 * command 'intt'   81
 ************************************************************/

/************************************************************
 * command 'intt'   82
 ************************************************************/

/************************************************************
 * command 'intt'   82
 ************************************************************/

/************************************************************
 * command 'lbertram'   83
 ************************************************************/

/************************************************************
 * command 'dbertram'   84
 ************************************************************/

/************************************************************
 * command 'berstats'   85
 ************************************************************/

/************************************************************
 * command 'bustest'   86
 ************************************************************/

/************************************************************
 * command 'cls'   87
 ************************************************************/
static int func_cls(int argc, s08 **argu);
static command_t cmd_cls = {
    "cls",               /* command */
    "",
    NULL,               /* option  */
    "Clear Screen",
    "",
    func_cls,
    CMD_GRP_MISC
};

static int func_cls(int argc, s08 ** argu)
{
    win_clearscreen();
    return NO_ERROR;
}

/************************************************************
 * command 'mread'   88
 ************************************************************/

/************************************************************
 * command 'loop'   89
 ************************************************************/
/************************************************************
 * command 'loop'   89
 ************************************************************/
static int func_loop(int argc, s08 **argu);
command_t cmd_loop = {
    "loop",               /* command */
    "[iteration] <cmd> [<parameter> ...]",
    NULL,                   /* option  */
    "loop on command",
    "",
    func_loop,
    CMD_GRP_MISC,
    SINGLE_ARGUMENT|NOT_DEVICE_CMD,
};

static int func_loop(int argc, s08 **argu)
{
    s08 loop_buff[COMMAND_LENGTH+1], * buff = argu[1];
    u32 loop = 0;
    u32 loopCnt = 0;
    int retc = NO_ERROR;

    if (argc>1)
    {    
        while (*buff==' ') buff++;
    
        if (util_isDigit(*buff))
            loop = strtoul(buff,&buff,10);
        
        /* buff get distroyed, so we save to to loop_buff */    
        strcpy(loop_buff,buff);
        do 
        {
            strcpy(buff,loop_buff);
            win_setTextColor(BROWN);
            printf("Loop %d",++loopCnt);
            if (loop)
                printf(" of %d",loop);
            win_setTextColor(WHITE);
            printf(" cmd:%s\n",buff);
            win_setTextColor(GRAY);
        } while (((retc=cmd_executeString(buff, NULL))==NO_ERROR) &&  
                 !util_user_abort() &&
                 (loopCnt<loop || !loop));
        test_setError(NO_ERROR);
    }
    return retc;
}


/************************************************************
 * command 'mrloop'   90
 ************************************************************/
static int func_mrloop(int argc, s08 **argu);
static command_t cmd_mrloop = {
    "mrloop",               /* command */
    "<addr>",
    NULL,                   /* option  */
    "loop on MII read",
    "This is special test routine for MII read.\nIt loops on MII register read util user abort or if value is zero",
    func_mrloop,
    CMD_GRP_MII
};

static int func_mrloop(int argc, s08 **argu)
{
    u32 addr;
    u16 data;
    u32 cnt=0;
    
    if (argc == 2)
    {
	addr = strtoul(argu[1],&argu[1],global.radix);
	while (1)
	{
	    data=miird(addr);
			
	    printf("%u: %04x\n", ++cnt, data);
	    if (util_user_abort() || !data)
		break;    
	}
	return NO_ERROR;
    }
    return cmd_help(&cmd_mrloop);
}

/************************************************************
 * command 'sedump' 91  
 ************************************************************/

static int func_sedump(int argc, s08 ** argu);
static option_t opt_sedump[] = {
    { 'l', OPT_DEC,  "file length"},
    { 'f', OPT_STR,  "filename",NULL},
    END_OF_OPTION
};

static command_t cmd_sedump = {
    "sedump",           // command      
    "<filename> [len]", // syntax       
    opt_sedump,         // option       
    "Dump eeprom content to a file",
    "",
    func_sedump,
    CMD_GRP_SEEPROM
};

static int func_sedump(int argc, s08 ** argu)
{
    u32 length = Value32(cmd_sedump,'l');
    s08 * fn = OptionStr(cmd_sedump,'f');
    
    if (argc==2) 
    {
	fn = argu[1];
    } 
    else if (argc==3)
    {
	fn = argu[1];
	length = strtoul(argu[2], &argu[2],global.radix);
    }
    if (fn==NULL)
	return cmd_help(&cmd_sedump);
	
    return eprom_dump(fn, length);
}

/************************************************************
 * command 'sram_test' 92  
 ************************************************************/

/************************************************************
 * command 'dbmode' 93  
 ************************************************************/

/************************************************************
 * command 'asf' 94  
 ************************************************************/

/************************************************************
 * command 'asfcfg'  95 
 ************************************************************/

/************************************************************
 * command 'flshmode'  96
 ************************************************************/
/************************************************************
 * command 'flshmode'  97
 ************************************************************/

/************************************************************
 * command 'cputrace'  98
 ************************************************************/

/************************************************************
 * command 'flshread' 99  
 ************************************************************/

/************************************************************
 * command 'chkpxe'  100
 ************************************************************/

/************************************************************
 * command 'asfprg' 101
 ************************************************************/

/************************************************************
 * command 'do'  102
 ************************************************************/
static int func_do(int argc, s08 **argu);
static option_t opt_do[] = {
    { 'e', OPT_BOOL, "echo command"},
    { 'p', OPT_DEC,  "pause between each command. If a value is entered, it delays for # of ms"},
    END_OF_OPTION
}; 

static command_t cmd_do = {
    "do",              /* command */
    "<filename>",
    opt_do,                 /* option  */
    "Excute command from a file",
    "",
    func_do,
    CMD_GRP_MISC,
    SINGLE_ARGUMENT | NOT_DEVICE_CMD | CHANGES_DEVICE,
};

#define CMD_BUFFER_LEN 80
static int gotoCmd(FILE * ifile, char * str)
{
    s8 * ptr;
    s8 buffer[CMD_BUFFER_LEN];

    while (*str==' ') str++;
    if (!*str)
	return HAS_ERROR;
    ptr = str;
    while (*ptr && *ptr!=' ') ptr++;
    *ptr = 0;
    fseek(ifile, 0, 0);
    while (!feof(ifile))
    {
	if (fgets(buffer,CMD_BUFFER_LEN,ifile))
	{
	    ptr = buffer;
	    while (*ptr==' ') ptr++;
	    if (*ptr==':') /* search for label */
	    {
		if (!strnicmp(++ptr,str,strlen(str)))
		    return NO_ERROR;
	    }
	}
    }    
    eprintf("Cannot find label '%s'\n",str);
    return HAS_ERROR;
}


static int func_do(int argc, s08 **argu)
{
    s8 buffer[CMD_BUFFER_LEN];
	FILE *ifile,*orgLog = global.logfile;
    s8 * str;
    int echo = cmd_optionEntered(&cmd_do,'e');
    int pause = cmd_optionEntered(&cmd_do,'p');
    u32 pause_time = cmd_optionValue(&cmd_do,'p').valueInt;
    int cont = cmd_optionEntered(&cmd_do,'C');
    int retc = NO_ERROR;
    u08 c=0;
    
    if (argc==2)
    {
	if ((ifile=fopen(argu[1],"r")) == NULL)
	{
	    eprintf("Cannot open %s\n",argu[1]);
	    return HAS_ERROR;
	}
	while (!feof(ifile))
	{
	    enable_all_interrupt();
	    if (fgets(buffer,CMD_BUFFER_LEN,ifile))
	    {
		str = buffer + strlen(buffer) - 1;
		if ((*str=='\n' )||(*str=='\r'))
		    *str = '\0';
		// Eliminate the \r that comes before the \n as well 
		str--;
		if ((*str=='\n' )||(*str=='\r'))
		    *str = '\0';
		    
		if (pause)
		{
		    if (pause_time)
		    {
			if (util_wait_ms(pause_time))
			{
			    retc = ERR_ABORT;
			    break;
			}    
		    }
		    else
		    {   
			while (1)
			{
			    printf("%s [y/n?]",buffer);
			    c=tolower(util_getch());
			    printf("\n");
			    if (c=='y' || c=='n' || c==KEY_ESC)
				break;
			}        
			if (c==KEY_ESC)
			{
			    retc = ERR_ABORT;
			    break;
			}    
		    }    
		}
		else if (echo)
		    printf("%s\n",buffer);

		if (retc==ERR_ABORT && scrn_yes("Stop do script?"))
		    break;

		if (c=='n')
		    continue;

		str = buffer;    
		while (*str==' ') str++;
		if (!strnicmp(str,"goto ",5))
		{
		    if (gotoCmd(ifile, str+5)==HAS_ERROR)
			break;
		    continue;    
		}
		
		if ((str[0]==';') || (buffer[0]==':')) /* ingore comment or label */
		    continue;
		    
		if ((retc=cmd_executeString(buffer, NULL)) == CMD_QUIT)
		    break;
		    
		if (util_user_abort() || ((retc!=NO_ERROR) && (!cont)))
		    if (scrn_yes("Stop do script?"))
			break;

	    } 
	}
		fclose(ifile);
	if (!orgLog && global.logfile)
	    func_nolog(argc,argu);
	return retc;          
    }
    return cmd_help(&cmd_do);
}

/************************************************************
 * command 'sleep'  103
 ************************************************************/
static int func_sleep(int argc, s08 **argu);
static command_t cmd_sleep = {
    "sleep",              /* command */
    "[miliseconds]",
    NULL,                  /* option  */
    "suspense process for Excute command from a file",
    "",
    func_sleep,
    CMD_GRP_MISC    
};

static int func_sleep(int argc, s08 **argu)
{
    u32 ms = 0;
    
    if (argc==2)
	ms = strtoul(argu[1],NULL,10);
    else if (argc>2)
	return cmd_help(&cmd_sleep);
    if (ms)
    {
	if(util_wait_ms(ms))
	    return ERR_ABORT;
    }
    else
	util_pause(NULL);
    return NO_ERROR;    
}

/************************************************************
 * command 'fillpattern'  104
 ************************************************************/

/************************************************************
 * command 'woltest'  105
 ************************************************************/

/************************************************************
 * command 'inp'  106
 ************************************************************/
static int func_inp(int argc, s08 **argu);
static option_t opt_inp[] = {
    { 'w', OPT_BOOL, "word size"},
    { 'l', OPT_BOOL, "long word size"},
    END_OF_OPTION
}; 

static command_t cmd_inp = {
    "inp",          /* command */
    "<addr>",
    opt_inp,             /* option  */
    "input port",
    "",
    func_inp,
    CMD_GRP_MISC
};

static int func_inp(int argc, s08 **argu)
{
    if (cmd_optionEntered(&cmd_inp,'l') && cmd_optionEntered(&cmd_inp,'w'))
    {
	eprintf("only one of the 'l' or 'w' option can be used\n");
	return HAS_ERROR;
    }
    if (argc==2)
    {
	u32 addr = strtoul(argu[1],NULL,16);
	
	if (cmd_optionEntered(&cmd_inp,'l'))
	    printf("%08X\n",inpd(addr));
	else if (cmd_optionEntered(&cmd_inp,'w'))
	    printf("%04X\n",inpw(addr));
	else
	    printf("%02X\n",inp(addr));
	return NO_ERROR;    
    }
    return cmd_help(&cmd_inp);
}

/************************************************************
 * command 'outp'  107
 ************************************************************/
static int func_outp(int argc, s08 **argu);
static command_t cmd_outp = {
    "outp",          /* command */
    "<addr> <value>",
    opt_inp,             /* option  */
    "output port",
    "",
    func_outp,
    CMD_GRP_MISC
};

static int func_outp(int argc, s08 **argu)
{
    if (cmd_optionEntered(&cmd_outp,'l') && cmd_optionEntered(&cmd_outp,'w'))
    {
	eprintf("only one of the 'l' or 'w' option can be used\n");
	return HAS_ERROR;
    }
    if (argc==3)
    {
	u32 addr = strtoul(argu[1],NULL,16);
	u32 data = strtoul(argu[2],NULL,16);
	
	if (cmd_optionEntered(&cmd_inp,'l'))
	    outpd(addr,data);
	else if (cmd_optionEntered(&cmd_inp,'w'))
	    outpw(addr,(u16) data);
	else
	    outp(addr,(u8) data);
	return NO_ERROR;    
    }
    return cmd_help(&cmd_outp);
}

/************************************************************
 * command 'inp'  108
 ************************************************************/
/************************************************************
 * command 'dir'  111
 ************************************************************/

/************************************************************
 * command 'erase'  112
 ************************************************************/

/************************************************************
 * command 'pxeprg'  113
 ************************************************************/

/************************************************************
 * command 'txmacdes'   114
 ************************************************************/

/************************************************************
 * command 'txmacsrc'   115
 ************************************************************/

/************************************************************
 * command 'txfill'    116
 ************************************************************/

/************************************************************
 * command 'msitest'   117
 ************************************************************/

/************************************************************
 * command 'qstat' 118  
 ************************************************************/

/************************************************************
 * command 'drvrcfg' 119  
 ************************************************************/

/************************************************************
 * command 'wbuf'    120
 ************************************************************/

/************************************************************
 * command 'rbuf'    121
 ************************************************************/

/************************************************************
 * command 'cpbuf'    122
 ************************************************************/

/************************************************************
 * command 'setbit'   123
 ************************************************************/

/************************************************************
 * command 'clearbit'   124
 ************************************************************/

/************************************************************
 * command 'echo'   125
 ************************************************************/
static int func_echo(int argc, s08 ** argu);
static command_t cmd_echo = {
    "echo",                     /* command */
    "<string>",                 /* syntax  */
    NULL,                       /* option  */
    "echo <string> to screen",
    "",
    func_echo,
    CMD_GRP_MISC
};

static int func_echo(int argc, s08 ** argu)
{
    if(argc > 1)
	printf("\n%s", &argu[1]);
    else
	printf("\n");
	
    return NO_ERROR;
} 

/************************************************************
 * command 'phymse' 125  
 ************************************************************/

/************************************************************
 * command 'pause'   126
 ************************************************************/

/************************************************************
 * command 'q' 127  
 ************************************************************/
static command_t cmd_q = {
    "q",                /* command      */
    "",                 /* syntax       */
    NULL,               /* option       */
    "Exit the System, alias name for quit",  /* title        */
    "",
    NULL,               /* command      */
    CMD_GRP_MISC
};

/************************************************************
 * command 'readbit'   128
 ************************************************************/

/************************************************************
 * command 'romtest'   130
 ************************************************************/

/************************************************************
 * command 'flshdev'   131
 ************************************************************/

/************************************************************
 * command 'gpiotest'   132
 ************************************************************/

/************************************************************
 * command 'verbose'   133
 ************************************************************/

/************************************************************
 * command 'cpudiag'   134  -- 
 ************************************************************/

/************************************************************
 * command 'irt' 135  
 ************************************************************/

/************************************************************
 * command 'ledtest'  136 
 ************************************************************/
static int func_ledtest(int argc, s08 **argu);
static option_t opt_ledtest[] = {
    { 'n', OPT_LOOP,  "iteration"},
    END_OF_OPTION
};

static command_t cmd_ledtest = {
    "ledtest",          /* command */
    "",                 /* syntax  */
    opt_ledtest,        /* option  */
    "Run LED Test",
    "The default iteration is 1. 0 means run forever",
    func_ledtest,
    CMD_GRP_TEST
};

static int func_ledtest(int argc, s08 **argu)
{
	return SB4xDiagTestLEDs(1,20);
}  
/************************************************************
* command 'lbtest'  137 
************************************************************/
static int func_lbtest(int argc, s08 **argu);
static option_t opt_lbtest[] = {
    { 'n', OPT_LOOP,  "iteration"},
	{ 'c', OPT_DEC ,  "max packet count"},
	{ 't', OPT_DEC ,  "packet type: 0 = all 0s, 1 = all 1s\n             2 = 5555, 3 = AAAA\n             4 = 0F0F, 5 = F0F0\n             6 = FF00, 7 = 00FF\n             8 = FFFF0000, 9 = 0000FFFF\n            10 = Inc, 11 = Random"},
	{ 'm', OPT_BOOL,  "mac loop back"},
	{ 'p', OPT_BOOL,  "phy loop back"},
	{ 'e', OPT_BOOL,  "100BT external loop back"},
	{ 'a', OPT_BOOL,  "10BT external loop back"},
    END_OF_OPTION
};

static command_t cmd_lbtest = {
    "lbtest",          /* command */
    "<n>",                 /* syntax  */
    opt_lbtest,        /* option  */
    "m = run MAC Loopback Test, p = run internal Phy Loopback Test, e = run 100 BT external Loopback Test, a = run 10 BT external Loopback Test",
    "",
    func_lbtest,
    CMD_GRP_TEST
};

static int func_lbtest(int argc, s08 **argu)
{
    ULONG Lbtybe;
	ULONG temp;


    if (cmd_optionEntered(&cmd_lbtest,'m'))
		Lbtybe = 0;
	else if (cmd_optionEntered(&cmd_lbtest,'p'))
		Lbtybe = 1;
	else if (cmd_optionEntered(&cmd_lbtest,'e'))
		Lbtybe = 3;
	else if (cmd_optionEntered(&cmd_lbtest,'a'))
		Lbtybe = 4;
	else
		return cmd_help(&cmd_lbtest) ;
	
	temp = SB4xDiagTestLoopBack(1,Lbtybe,cmd_optionValue(&cmd_lbtest,'c').valueInt,cmd_optionValue(&cmd_lbtest,'t').valueInt);
	
	if (temp == NO_ERROR)
		test_passed0();
	else
		test_failed0(temp);
	printf("\n");
    
	return temp;
}     
/************************************************************
* command 'linkstatus'  138 
************************************************************/
static int func_linkstatus(int argc, s08 **argu);

static command_t cmd_linkstatus = {
    "linkstatus",          /* command */
    "",                 /* syntax  */
    NULL,        /* option  */
    "Report link up and link down",
    "",
    func_linkstatus,
    CMD_GRP_TEST
};

static int func_linkstatus(int argc, s08 **argu)
{
	void * dummy;
	dummy=0;
	Link_test(dummy);
	printf("\n");
	return NO_ERROR;
    
}  
 /************************************************************
 * command 'sromutil'   139
 ************************************************************/

static int func_sromutil(int argc, s08 **argu);
static option_t opt_sromutil[] = {
	{ 'b', OPT_BOOL, "Enable bootrom, size encoding: 0=Disable, 1=16KB, 2=32KB, 3=64KB, 4=128KB"},
	{ 'm', OPT_BOOL, "program addr: macaddr sub_venID sub_devID to HW or Bin file"},
    { 'C', OPT_BOOL, "check_crc"},
    { 'c', OPT_BOOL, "fix crc"},
    { 'd', OPT_BOOL, "just dump"},
	{ 'w', OPT_BOOL, "program word"},
	{ 'f', OPT_STR, "output image file names. Must be used with option 'd'.",NULL},
    END_OF_OPTION
};

static command_t cmd_sromutil = {
    "sromutil",
    "",               
    opt_sromutil,      
    "SROM Utilities",
    "",
    func_sromutil,
    CMD_GRP_MISC
};

static int func_sromutil(int argc, s08 **argu)
{  
	int             i = 0, j=0;
    u16             sprom_size = SROM_SIZE;
	u16             *image;                 // In-core copy of the srom 
	char            *emacaddr;
	u16             deviceid=0;
	u16             sub_vendor, sub_id;
    u16             enet_mac[3];
    u16             loc, val;
	u32             val32;
	u8              crc;
	int             todo = 0;
	chip_type_t     chip_type = BCM4401;
	chip_info_t     *ci;
    u16             brsize = 0xffff;
	int             status;
	int             temp =1;
	u32             tempread;
	int         ci_index[8];
	u16 *           tempaddr;
	FILE * file;
	u8 *      buffer;
	s08 * fn;
	FILE *tmpfile;
	ci = &chip_info[1];
	chip_type = ci_index[1];
	

	if (cmd_optionEntered(&cmd_sromutil,'b'))
	{
		temp=2;
		todo = BOOTROM;
		(void) hexstring(argu[1], &brsize, 1, FALSE); 
		if (brsize > 4)
		{
			printf("Invaid boot rom size!\n");
			status= HAS_ERROR;
			goto exit_sub;
		}
	}
	if(cmd_optionEntered(&cmd_sromutil,'m'))
	{
		todo = PROGRAM_HW;
		if (argc < temp + 3)
		{
			printf("Missing parameters < MACddr Sub_VenID Sub_DevID>\n");
			status = HAS_ERROR;
			goto exit_sub;
		}
	}
	else if (cmd_optionEntered(&cmd_sromutil,'C'))
	todo = CHECK_CRC;
	else if (cmd_optionEntered(&cmd_sromutil,'c'))
	todo = FIX_CRC;
	else if (cmd_optionEntered(&cmd_sromutil,'d'))
	todo = JUST_DUMP;
	else if (cmd_optionEntered(&cmd_sromutil,'w'))
	{
		todo = PROGRAM_WORD;
		(void) hexstring(argu[temp], &loc, 1, FALSE);
		(void) hexstring(argu[temp+1], &val, 1, FALSE);   
		if ((loc > 0x3F) || (val > 0xffff))
		{
			printf ("parameters out of range.\n");
			status = HAS_ERROR;
			goto exit_sub;
		}
	      
	}
	else if (brsize == 0xffff)
	{
	    printf("Missing options!\n");
		cmd_help(&cmd_sromutil);
	    status=HAS_ERROR;
	    goto exit_sub;
	}
	

	if (todo == PROGRAM_HW) {
		
	    emacaddr = argu[temp];
		memset(enet_mac, 0, sizeof(enet_mac));
	    (void) hexstring(emacaddr, &enet_mac[0], 3, FALSE);
		temp++;
		(void) hexstring(argu[temp], &sub_vendor, 1, FALSE);
		temp++;
		(void) hexstring(argu[temp], &sub_id, 1, FALSE);
		
	}
  
	image = (uint16 *)malloc(sprom_size*2);
	
	for (i = 0; i < sprom_size/2; i++) {
		tempread = RegRd( SPROM_BASE + (i * 4));
		image[j] = tempread & 0xffff;
		j++;
		image[j] = (tempread & 0xffff0000) / 0x10000;
		j++;
	}

	if (todo == BOOTROM)
	{
		enet_mac[2]=image[SROM_E0_MACLO];
		enet_mac[1]=image[SROM_E0_MACMID];
		enet_mac[0]=image[SROM_E0_MACHI];
		sub_id=image[SROM_SSID];
		sub_vendor=image[SROM_VID];
		goto write;
	}
	if ((todo == PROGRAM_HW) && (brsize == 0xffff))
	{
		brsize = ((image[SROM_COMMPW] >> 9) & 0x7F) +1;
	}
	if (crc8((u8 *)image, SROM_BYTES, CRC8_INIT_VALUE) != CRC8_GOOD_VALUE) 
	{
		
		if (todo == CHECK_CRC)
		{
			printf ("Bad CRC\n");
			status=HAS_ERROR;
			goto exit_sub;
		}
	}
	
	if (todo == CHECK_CRC) {
		printf("Good CRC\n");
		status=NO_ERROR;
		goto exit_sub;
	}

	if (todo == JUST_DUMP) {
		srdump(image, sprom_size, "Srom contents:", chip_type);
		if (cmd_optionEntered(&cmd_sromutil,'f'))
		{
			fn = OptionStr(cmd_sromutil,'f');
			if (fn == NULL) fn = "eeprom.bin";
			strcpy(global.binFileName , fn);
			strncpy(fn + strcspn(fn,".") ,".txt",4);
			strcpy(global.txtFileName , fn);
			
			buffer = (uint8 *)malloc(sprom_size*4);
			for (i = 0; i < SROM_SIZE; ++i) 
			{
				buffer[i*2] = image[i];
				buffer[i*2 +1] = image[i] >> 8;
			}
			if ((file = fopen(global.binFileName, "wb")) == NULL)
			{
				printf("Cannot open eeprom output file...\n");
				return (HAS_ERROR);
			}

			fwrite(buffer,SROM_SIZE*4,1,file);
			fclose(file);
			printf("\n");
			printf("%s updated\n",global.binFileName);

			if (!(tmpfile = fopen(global.txtFileName, "w+")))
			{
				eprintf("Cannot open file, '%s'!\n", global.txtFileName);
				return (HAS_ERROR);
			}
			fprintf (tmpfile, "4401 Srom Image\n");
			fprintf (tmpfile, "Address     Value     Description\n");
			
			
			
			for (i = 0; i < SROM_SIZE; ++i) 
			{
				fprintf (tmpfile, "  %3d        %04x      %s\n",i,image[i],SromTable[i]);
			}
			
			fclose(tmpfile);
			printf("\n");
			printf("%s updated\n",global.txtFileName);
		}
		status=NO_ERROR;
		goto exit_sub;
	}
	if (todo == PROGRAM_WORD) {
	image[loc] = val;
	goto write_it;
    }

	if (todo == FIX_CRC) {
		goto write_it;
	}
write:
	// Clear it 
	for (i = 0; i < sprom_size*2; i++) {
		image[i] = 0xffff;
	}

	// write the global header info 
	wrhdr(image, ci, deviceid, sub_vendor, sub_id, brsize);

	for (i = 0; i < MAX_FN; ++i) {
		if (ci->functions[i].device != 0xffff)
			wrfun(image, i, &ci->functions[i]);
	}

	// XXX: 4402 specific: Enet MAC Address 
	image[SROM_E0_MACLO] = enet_mac[2];
	image[SROM_E0_MACMID] = enet_mac[1];
	image[SROM_E0_MACHI] = enet_mac[0];

	image[SROM_EPHY] = ci->phyval;
	image[SROM_SIZE - 1] = SROM_VERS;

write_it:
	// Calculate crc's XXX: Assumes little-endian host 
	crc = ~crc8((u8 *)image, SROM_BYTES - 1, CRC8_INIT_VALUE);
	image[SROM_SIZE - 1] = (crc << 8) | (image[SROM_SIZE - 1] & 0xff);


	printf("Enabling SPROM writes.\n");

	read_configuration_dword(CURRCARD.bus_number,
				  CURRCARD.device_and_function,
				  CFG_SROM_WRITABLE_OFFSET,&val32);
	write_configuration_dword(CURRCARD.bus_number,
				  CURRCARD.device_and_function,
				  CFG_SROM_WRITABLE_OFFSET,val32 | SROM_WRITEABLE);

	MM_Wait(5000);  
	for (i = 0; i < sprom_size; i++)
		{
			   tempaddr = (U16 *)CURRCARD.base_address + (SPROM_BASE/2) + i; 
			   * tempaddr = image[i];
			MM_Wait(20000); 
		}
	
	// assumes 4-byte aligned offset 
		write_configuration_dword(CURRCARD.bus_number,
				  CURRCARD.device_and_function,
				  CFG_SROM_WRITABLE_OFFSET,val32 );

	MM_Wait(5000);  

	//if (verbose)
		srdump(image, sprom_size, "Programmed:", chip_type);

	// check to see if SPROM programmed correctly 
	j=0;
	for (i = 0; i < sprom_size/2; i++)
		{
			tempread = RegRd(SPROM_BASE + (i*4));
			
			if (((tempread & 0xffff) != image[j]) || 
			   (((tempread & 0xffff0000)/ 0x10000) != image[j+1]))
				break;
			j=j+2;
		}       
	if (i < sprom_size/2) 
	{
	printf("SPROM not programmed correctly!\n");
	printf("Location: %d\n", i);
		status=HAS_ERROR;
	}
	else
	util_init_pci_and_find_tigons (0, 0);

exit_sub:
    
	return status;
}

/**********************************************************/
command_ptr cmds[] = {
    //&cmd_reg,
    &cmd_exit,
    //&cmd_semode,
    //&cmd_seread,
    //&cmd_sewrite,
// 5   
    //&cmd_secfg,
    //&cmd_vpdwrite,
    //&cmd_vpdread,
    //&cmd_vpdtest,
    &cmd_regtest,
// 10   
    &cmd_seprg,
    //&cmd_memsearch,
    &cmd_miitest,
    //&cmd_debug,
    //&cmd_maclpk,
// 15   
    //&cmd_msi,
    //&cmd_memtest,
    &cmd_setest, 
    &cmd_bist,
    &cmd_nictest,
// 20
    //&cmd_intr, 
    &cmd_intrtest,
    &cmd_nicstats,
    //&cmd_ringindex,
    &cmd_blast,
// 25    
    //&cmd_intrctrl,
    //&cmd_gpiowrite,
   // &cmd_gpioread,
    //&cmd_pxecpy,
    //&cmd_pkttest,
// 30    
    //&cmd_loadfw,
    //&cmd_cputest,
    //&cmd_dmaw,
    //&cmd_dmar,
    //&cmd_dmatest,
// 35    
    //&cmd_mbuf,   
    //&cmd_upgfrm,
    //&cmd_txpkt,
    &cmd_device,
    &cmd_version,
// 40 
    //&cmd_loaddrv,
    //&cmd_unloaddrv,
    &cmd_help1,
    &cmd_help2,
    &cmd_phyctrl,
// 45 
    //&cmd_statusblk,
    //&cmd_dma_h,
    //&cmd_dma_d,
    //&cmd_machalt,
    &cmd_read,
// 50
    &cmd_write,
    &cmd_radix,
    &cmd_nolog,
    &cmd_log,
    &cmd_pciinit,
// 55
    &cmd_pciscan,
    //&cmd_sever,
    //&cmd_sechksum,
    &cmd_dos,
    //&cmd_diagcfg,
// 60
    //&cmd_ftq,
    //&cmd_addmc,
    //&cmd_delmc,
    &cmd_mwrite,
    &cmd_mread,
// 65    
    //&cmd_mdev,
    //&cmd_miimode,
    &cmd_reset,
    //&cmd_stsblk,
    //&cmd_cpudtt,
// 70
    //&cmd_cpudrt,
    //&cmd_txcfg,
    //&cmd_rxcfg,
    //&cmd_pmdcfg,
    //&cmd_tprot,
// 75
    //&cmd_pmpcfg,
    //&cmd_pmpd,
    &cmd_teste,
    &cmd_testd,
    &cmd_quit,
// 80
    //&cmd_intt,
    //&cmd_smbus,
    //&cmd_asftest,
    //&cmd_lbertram,
    //&cmd_dbertram,
// 85
    //&cmd_bertstats,
    //&cmd_bustest,
    &cmd_cls,
    //&cmd_rm,
    &cmd_loop,
// 90    
    &cmd_mrloop,
    &cmd_sedump,
	//&cmd_sramtest,
    //&cmd_dbmode,
    //&cmd_asf,
// 95
    //&cmd_asfcfg,
    //&cmd_flshmode,
    //&cmd_flshprg,
    //&cmd_cputrace,
    //&cmd_flshread,
// 100
    //&cmd_chkpxe,    
   // &cmd_asfprg,
	&cmd_do,
    &cmd_sleep,
	//&cmd_fillpattern,
// 105
    //&cmd_woltest,
    &cmd_inp,
    &cmd_outp,
    //&cmd_poll,
    //&cmd_switch,
// 110
    //&cmd_dir,
    //&cmd_erase,
    //&cmd_pxeprg,
    //&cmd_txmacdes,
    //&cmd_txmacsrc,
// 115  
	//&cmd_txfill,
	//&cmd_qstat,
    //&cmd_msitest,
	//&cmd_drvrcfg,
	//&cmd_wbuf,
// 120
	//&cmd_rbuf,
	//&cmd_cpbuf,
	//&cmd_setbit,
	//&cmd_clearbit,
    &cmd_echo, 
// 125    
    //&cmd_phymse,
    //&cmd_pause,
    &cmd_q,
    //&cmd_readbit, 
    //&cmd_romtest,
// 130
    //&cmd_flshdev,
    //&cmd_gpiotest,
    //&cmd_verbose,
    //&cmd_cpudiag,
    //&cmd_irt,
//135
	&cmd_ledtest,
	&cmd_lbtest,
	&cmd_linkstatus,
	&cmd_sromutil,
    NULL,
};

/********************************************************************
 *
 *  Description: This file contains comand line interface 
 *
 *  $Log: /Source/Bcm44xx/b44diag/t3_cmd.c $ 
 * 
 * 16    7/19/02 10:07a Ahui
 * Removed unused code
 * 
 * 15    7/11/02 12:59p Ahui
 * Fixed a bug in labels.
 * 
 * 14    7/08/02 12:25p Ahui
 * Corrected srom labels
 * 
 * 13    6/17/02 2:42p Ahui
 * Mod -b option in sromutil cmd such that it no longer needed to be used
 * with -m option.  
 * 
 * 
 * 12    6/13/02 10:19a Ahui
 * Reduce the trail to 20 in ledtest
 * 
 * 11    6/11/02 4:03p Ahui
 * Added -c option to reset counters.
 * Added pattern select for blast.
 * Added pattern and max packet count selections for lbtest.
 * Added output filename entry for sromutil
 * 
 * 10    5/31/02 10:41a Ahui
 *
 * 8     5/16/02 5:56p Mnima
 * 
 * 7     4/25/02 11:08a Ahui
 * Modified sromutil to dump srom image to txt file.
 * 
 * 3     3/07/02 5:05p Ahui
 * Added sromutil support
 */
