/********************************************************************
 * Copyright(c) 2000 Broadcom Corporation, all rights reserved.
 * Contains proprietary and confidential information.
 *
 * This source file is the property of Broadcom Corporation, and
 * may not be copied or distributed in any isomorphic form without 
 * the prior written consent of Broadcom Corporation. 
 *
 *  FileName: $Header: /Source/Bcm44xx/b44diag/t3_cmd.h 3     3/07/02 5:05p Ahui $
 *
 *  Description: This file contains command line interface 
 *
 *  $Log: /Source/Bcm44xx/b44diag/t3_cmd.h $ 
 * 
 * 3     3/07/02 5:05p Ahui
 * Added sromutil support
 ********************************************************************/ 

#ifndef __T3_CMD_H__
#define __T3_CMD_H__

#include "types.h"
#include "math.h"

#define END_OF_OPTION {'\0'}
#define CheckValue32(cmd,c,var) {if (cmd_optionEntered(cmd,c)) var = cmd_optionValue(cmd,c).value32;}
#define Value32(cmd,c) cmd_optionValue(&cmd,c).value32
#define pValue32(cmd,c) cmd_optionValue(cmd,c).value32
//#define pValue32(cmd,c,def) (cmd_optionEntered(cmd,c) ? cmd_optionValue(cmd,c).value32 : def)
#define OptionStr(cmd,c) cmd_optionValue(&cmd,c).valueStr
#define GetFilename(cmd,fn,cnt) \
{                              \
    fn = (argc>cnt ? argu[cnt] : OptionStr(cmd,'f')); \
    if (fn==NULL)                                     \
        return cmd_help(&cmd);                        \
}

#define NO_DEFAULT 0xdeafdeaf

typedef enum {
   CMD_GRP_VPD,
   CMD_GRP_SEEPROM,
   CMD_GRP_CPU,
   CMD_GRP_DMA,
   CMD_GRP_PACKET,
   CMD_GRP_MII,
   CMD_GRP_MEM,
   CMD_GRP_TEST,
   CMD_GRP_POWER,
   CMD_GRP_IRQ,
   CMD_GRP_MAC,
   CMD_GRP_MISC,
   CMD_GRP_ASF,
   MAX_CMD_GRP
} CMD_GRP_T;
   

typedef union {
    int    valueInt;
    u32    value32;
    s08 *  valueStr;
} optionValue_t;

typedef enum {
    OPT_U32,
    OPT_STR,
    OPT_BOOL,
    OPT_INT,
    OPT_LOOP,
    OPT_DEC,
} OPT_T;
 
typedef struct {
    const s08              key;
    const OPT_T           type;  
    const s08 *           help;
    optionValue_t   defVal,val;
    u08                entered;
} option_t, * option_ptr;

typedef enum {
	SINGLE_ARGUMENT = 0x1,	 // bit 0
    MULTIPLE_DEVICE = 0x2,   // bit 1
	CHANGES_DEVICE  = 0x4,   // bit 2
    NOT_DEVICE_CMD  = 0x8,   // BIT 3
};    

typedef struct command_t {
   const s08 *    cmd;  /* command */
   const s08 * syntax;  /* command syntax */
   option_ptr      opt;  /* points to valued options */
   const s08 *  title;  /* command line title */
   const s08 *  help; 
   int (*func)(int, s08 **);
   CMD_GRP_T group;
   u32	flags;
} command_t, * command_ptr;

extern command_ptr cmds[];
extern command_t cmd_help1;
extern command_t cmd_dos;
extern s08 * cmd_grp_str[];
extern command_t cmd_loop;
extern command_t cmd_txpkt;
#endif
