/********************************************************************
 * Copyright(c) 2000 Broadcom Corporation, all rights reserved.
 * Contains proprietary and confidential information.
 *
 * This source file is the property of Broadcom Corporation, and
 * may not be copied or distributed in any isomorphic form without 
 * the prior written consent of Broadcom Corporation. 
 *
 *
 *  Name: T 3 _ E P R O M . C
 *  FileName: $Header: /Source/Bcm44xx/b44diag/t3_eprom.c 7     7/19/02 10:07a Ahui $
 *
 *******************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <time.h>
#ifndef EFI
#include <graph.h>
#include <io.h>
#endif

#include "general.h"
#include "error.h"
#include "t_io.h"
#include "t3_util.h"
#include "fw_img.h"
#include "protocol.h"
#include "t3_gdata.h"
#include "t3_eprom.h"
#include "general.h"
#include "t3_tests.h"
#include "t3_win.h"
#include "t3_flash.h"
#include "bitbang.h"
#include "Cl4xsb.h"
#define SEEPROM_DELAY() ssprom_delay()


/*************************************************************************
    private routines 
*************************************************************************/    

static int eprom_EepGetData(void)
{
    u32 data = t_readmem32(OFFSETOF(T3_STD_MEM_MAP,Grc.EepromCtrl)) 
               & (CLK_OE | CLK_BIT);                         // save clock line status 
               
    t_writemem32 (OFFSETOF(T3_STD_MEM_MAP,Grc.EepromCtrl), data);
    
    data = t_readmem32(OFFSETOF(T3_STD_MEM_MAP,Grc.EepromCtrl));
    t_writemem32 (OFFSETOF(T3_STD_MEM_MAP,Grc.EepromCtrl), data | DATA_IN_BIT);
    return (data & DATA_IN_BIT) ? 1 : 0;
}

/*************************************************************************/

static int eprom_legacy_bitbang_write(u32 dev,u32 addr,u32 data)
{    
    return eepBitBangWriteWord((u08) dev, (u16) addr, data, eprom_EepSetClock, eprom_EepSetData, eprom_EepGetData);
}


/*************************************************************************/
static int eprom_legacy_auto_write(u32 device,u32 addr,u32 data)
{
    u32 i;
    
    t_writemem32(OFFSETOF(T3_STD_MEM_MAP,Grc.EepromData),data);

    data = t_readmem32(OFFSETOF(T3_STD_MEM_MAP,Grc.EepromAddr));
    t_writemem32(OFFSETOF(T3_STD_MEM_MAP,Grc.EepromAddr),
           data | SEEPROM_ADDR_COMPLETE);

    data &= ~(SEEPROM_ADDR_ADDRESS_MASK | SEEPROM_ADDR_DEV_ID_MASK);
    data &= ~SEEPROM_ADDR_RW_MASK;

    t_writemem32(OFFSETOF(T3_STD_MEM_MAP,Grc.EepromAddr),
           data |
           SEEPROM_ADDR_DEV_ID(device) |
           SEEPROM_ADDR_ADDRESS(addr) |
           SEEPROM_ADDR_START |
           SEEPROM_ADDR_WRITE);

    for (i = 0; i < SEEPROM_WAIT;i++)
    {
        data = t_readmem32(OFFSETOF(T3_STD_MEM_MAP,Grc.EepromAddr));
        if (data & SEEPROM_ADDR_COMPLETE)
            return NO_ERROR;
    }
    eprom_init(); 
    return HAS_ERROR;
}        


/*************************************************************************/
static void foo ()
{
  return;
}

/*************************************************************************
    public routines 
*************************************************************************/    
void eprom_EepSetClock(int set)
{
    u32 data = t_readmem32(OFFSETOF(T3_STD_MEM_MAP,Grc.EepromCtrl)) 
               & (DATA_OE_BIT | DATA_OUT_BIT | DATA_IN_BIT) /* save data line status */
               | CLK_OE;                                    /* clock output enable */
    if (set)
        data |= CLK_BIT;
    t_writemem32 (OFFSETOF(T3_STD_MEM_MAP,Grc.EepromCtrl), data);
}

/*************************************************************************/

void eprom_EepSetData(int set)
{
    u32 data = t_readmem32(OFFSETOF(T3_STD_MEM_MAP,Grc.EepromCtrl)) 
               & (CLK_OE | CLK_BIT)                         // save clock line status 
               | DATA_OE_BIT;
                                              // write data output enable 
    if (set)
        data |= DATA_OUT_BIT;
    t_writemem32 (OFFSETOF(T3_STD_MEM_MAP,Grc.EepromCtrl), data);
}

/*************************************************************************/
void  eprom_init()
{
  /* Use AUTO mode by defaults */

  /* Initialize clock period */
  t_writemem32(OFFSETOF(T3_STD_MEM_MAP,Grc.EepromAddr),
               SEEPROM_ADDR_CLK_PERD(SEEPROM_CLOCK_PERIOD) |
               SEEPROM_ADDR_FSM_RESET);
  util_wait_ms(1);
  
  t_writemem32(OFFSETOF(T3_STD_MEM_MAP,Grc.LocalCtrl),
               t_readmem32(OFFSETOF(T3_STD_MEM_MAP,Grc.LocalCtrl)) |
               GRC_MISC_LOCAL_CTRL_AUTO_SEEPROM);

#ifdef IKOS
  t_writemem32(OFFSETOF(T3_STD_MEM_MAP,Grc.SepromDelay),0x8d8);
#else
  t_writemem32(OFFSETOF(T3_STD_MEM_MAP,Grc.SepromDelay),0xa2c2a);
#endif
}


/*************************************************************************/
u32 eprom_read_seprom(u32 addr)
{
  u32 temp;
  u32 data;
  //int failed = eprom_read_word(addr,&temp);

  temp=EpromRd(addr);
  /* First of all, convert data to little-endian first */
  data = BYTE_SWAP(temp);

  return data;
}


/*************************************************************************/

/*************************************************************************/
int eprom_read_word(u32 addr, u32 *data)
{
	*data = EpromRd(addr);
	return (NO_ERROR);
}

/*************************************************************************/
void eprom_fill_blk(u32 start, u32 end,u32 wr_data)
{
    u32 i;
    int device;
    int address;
    u32 temp;
    int row,col;

    temp = BYTE_SWAP(wr_data);
    if (!flash_oldMethod())
    {
        flash_fill_blk(start, end, wr_data, 1);
    }
    else 
    {
        /* First of all, convert data to big-endian first */
        printf("Programming %04X",start);
        win_wherexy(&row, &col);
        col-=4;
        for (i = start; i <= end; i+=4) 
        {
            if (!(i & 0xf))
            {
                win_gotoxy(row,col);
                printf("%04X",i);
            }
            device  = i / SEEPROM_CHIP_SIZE;
            address = i % SEEPROM_CHIP_SIZE;
			EpromWr(address,temp);
            //if (eprom_write_word(device,address, temp)==HAS_ERROR)
            //{
                //eprintf("\nFailed writing 0x%x to 0x%x (device = %d)\n",
                     //temp,address,device);
                //return;
            //}
      
            if (util_user_abort())
                break;
        }
        win_gotoxy(row,col);
        printf("%04X\n",i);
    }    
}


/*************************************************************************/

int eprom_write_word(u32 device,u32 eeprom_addr,u32 wr_data)
{
    if (!flash_oldMethod())
        return flash_program(eeprom_addr, 1, &wr_data, 0);
    else
    {
        int retc;
        
        if (eeprom_addr>=0x10000)
            return HAS_ERROR;

        lockFlash();
        if (t_readmem32(OFFSETOF(T3_STD_MEM_MAP,Grc.LocalCtrl)) & GRC_MISC_LOCAL_CTRL_AUTO_SEEPROM)
            retc = eprom_legacy_auto_write(device, eeprom_addr, wr_data);
        else
            retc = eprom_legacy_bitbang_write(device, eeprom_addr, wr_data);
        releaseFlash();
        return retc;
    }    
}

/*************************************************************************/
void ssprom_delay()
{
#ifdef IKOS
  util_wait_ms(10);
#else
  u32 now;
  u32 target;

  now = t_readmem32(OFFSETOF(T3_STD_MEM_MAP,Grc.Timer));
  target = now + 10000;
  if (target > now)
    {
      /* No wrap */
      while (t_readmem32(OFFSETOF(T3_STD_MEM_MAP,Grc.Timer)) < target) {}
    }
  else
    {
      while (t_readmem32(OFFSETOF(T3_STD_MEM_MAP,Grc.Timer)) > target) {}
    }
#endif
}

/*************************************************************************/

int eprom_write_blk(u32 start,u32 *buffer,u32 num_words)
{
    int device;
    u32 address;
    int i;

    if (!flash_oldMethod())
        return flash_program(start, num_words, buffer, 0);
    else
    {
        for (i = 0; i < num_words ; i++,start+=4)
        {
            device  = start / SEEPROM_CHIP_SIZE;
            address = start % SEEPROM_CHIP_SIZE; 
            if (eprom_write_word(device,address,buffer[i])==HAS_ERROR)
                return test_failed1(ERR_EEP_WRITE,start);
        }    
    }
    return DIAG_SUCCESS;
}

/*************************************************************************/
int eprom_read_blk(u32 address,u32 *buffer,u32 num_words)
{
    int i;
	
    for (i = 0; i < num_words ; i++, address+=4)
    {
        if (eprom_read_word(address,buffer++)==HAS_ERROR)
            return test_failed1(ERR_EEP_READ,address);
        if (util_user_abort()) 
            return test_failed0(ERR_ABORT);
    }
    return NO_ERROR;
}

/*************************************************************************/
/*
 * waits 'time' microseconds. FIX:dcb
 */
void eprom_wait (u32 time)
{
  //delay(1);
  int i;
  while (time--)
    for (i=0; i<100; i++)
      foo();
}

/*************************************************************************/
int eprom_dump(s08 * fn, u32 length)
{
    int ofile=-1;
    u08 *buffer;
	int i, j;

    if (!length)
        length = CURRCARD.eeprom_size;

    if ((buffer = (u08 *)malloc(length)) == NULL)
    {
        eprintf("Cannot allocate memory!\n");
        return DIAG_FAILURE;
    }

    if ((ofile = open(fn, O_TRUNC| O_CREAT | O_RDWR | O_BINARY, S_IRWXU)) == -1)
    {
        eprintf("Cannot open %s\n",fn);
        free(buffer);
    }

    printf("Reading contents of serial eeprom (length=0x%X (%u)) ...",length,length);
	printf("\n");
    if (eprom_read_blk(0,(u32 *)buffer,length/4) < 0)
    {
        eprintf("failed!\n");
        close(ofile);
        free(buffer);
        return DIAG_FAILURE;
    }
	j=0;
    for (i=0; i < length; i++)
	{
		printf("%02x%02x ",buffer[i+1],buffer[i]);
		i++;
		j++;
		if (j == 7) {
			printf("\n");
			j=0;
		}

	}
    printf("OK\n");
    printf("Writing contents of serial eeprom to file ... ");
	
    if (write(ofile,buffer,length) != length)
        eprintf("failed!\n");
    else
        printf("OK\n");

    free(buffer);
    close(ofile);
    return NO_ERROR;
} 

/*******************************************************************
 *
 *  Description: This file contains serial EEPROM utility routines
 *               routines to checkout T3.
 *
 *  AUTHOR: Kevin Tran
 *
 *  HISTORY:    Kevin Tran        06/01/2000   Created
 *  $Log: /Source/Bcm44xx/b44diag/t3_eprom.c $
 * 
 * 7     7/19/02 10:07a Ahui
 * Removed unused code
 * 
 * 6     4/25/02 11:06a Ahui
 * 
 * 3     3/07/02 5:05p Ahui
 * Added sromutil support
 *******************************************************************/
