/********************************************************************
 * Copyright(c) 2000 Broadcom Corporation, all rights reserved.
 * Contains proprietary and confidential information.
 *
 * This source file is the property of Broadcom Corporation, and
 * may not be copied or distributed in any isomorphic form without 
 * the prior written consent of Broadcom Corporation. 
 *
 *
 *  Name: T 3 _ E R R O R . C
 *
 *  Description: This file contains error support routines.
 *  point.
 *
 *  AUTHOR: Kevin Tran
 *
 *  HISTORY:    Kevin Tran        06/01/2000   Created
 *
 *******************************************************************/

#include "general.h"
#include "error.h"
#include "t3_util.h"

//extern unsigned int lineno; 	/* Global line number */

u08 t_err_type;		/* global error type */
u32 t_errl1 = 0;
u32 t_errl2 = 0;
u32 t_errl3 = 0;
u32 t_errl4 = 0;
u32 t_errl5 = 0;
u32 t_errl6 = 0;
s08 *t_errs1 = NULL;
s08 *t_errs2 = NULL;
s08 *t_errs3 = NULL;

/*
 *      Error message printers
 */

void
fatal_error (s08 *message, s08 *s, int k)
{
        printf("\n*** FATAL ERROR - ");
        printf(message, s, k);
        printf("\n");
}

void
error (s08 *message, s08 *s, int k)
{
}

void
warning (s08 *message, int line)
{
      	printf("*** WARNING - Line %d: %s\n", line, message);
}

void
fatal_test_error ()
{
	test_error ();
	util_system_exit(1);
}

void
test_warning ()
{
	wprintf("*** WARNING - ");
	if (t_errs1)
	   wprintf(t_errs1, t_errl1);
	wprintf("\n");
}

void
test_error ()
{
	s08 buf[16];
	int i;

	eprintf("*** ERROR - ");
	switch (t_err_type) {
	case BAD_CKSUM:
	   if (t_errs1)
              eprintf("%s\n", t_errs1);
	   eprintf ("  Expected : %08lx\n  Got      : %08lx\n",
	    t_errl1, t_errl2);
	   break;
	case NO_PCI_BIOS:
	case NO_TIGON_BASEADDR:
	case NO_TIGON_INTLINE:
	case NO_ASIC_VERSION:
	case NO_TIGON:
	case NO_SHARED_ACCESS:
	case NO_DMA_MASTER:
	   if (t_errs1)
              eprintf("%s\n", t_errs1);
	   break;
	case BAD_CONSTANT:
	case BAD_DATALINE:
	case ADDR_LINE_SHORT:
	case BAD_MAGIC_NUMBER:
	case BAD_DATA:
	   if (t_errs1)
	      eprintf(t_errs1);
	   if (t_errl5)
	      eprintf("  parity error detected!\n");

	   sprintf (buf, "%08lx", t_errl2);   /* construct 00x00xxx notation */
	   for (i=0; i<8; i++)
	   {
	      if (((u32)0xf << 4*(7-i)) & t_errl6)
		 buf[i] = 'x';
	   }
	   t_errl6 = 0;

	   eprintf ("  Address  : %08lx\n  Expected : %s\n  Got      : %08lx\n",
	    t_errl1, buf, t_errl3);
	   break;
	case CONF_BYTE:
	   if (t_errs1)
	      eprintf(t_errs1);

	   eprintf ("  Address  : %08lx\n  Expected : %02lx\n  Got      : %02lx\n",
	    t_errl1, t_errl2, t_errl3);
	   break;
	case CONF_HALF:
	   if (t_errs1)
	      eprintf(t_errs1);

	   eprintf ("  Address  : %08lx\n  Expected : %04lx\n  Got      : %04lx\n",
	    t_errl1, t_errl2, t_errl3);
	   break;
	case RW_TEST_FAILURE:
	   if (t_errs1)
	      eprintf(t_errs1, t_errs2);

	   eprintf ("  Address  : %08lx\n  W/R bits : %08lx\n  Mask     : %08lx\n",
	    t_errl1, t_errl2, t_errl3);
	   break;
	case TIMER_FAILURE:
	   if (t_errs1)
	      eprintf(t_errs1, t_errl1);
	   break;
	case MAILBOX_FAILURE:
	   if (t_errs1)
	      eprintf(t_errs1, t_errl1, t_errl2);
	   break;
	default:
	   eprintf("Illegal error code\n");
	   break;
	}
}

/* DMA errors have too many parameters to report as abve */
void
test_err_firstdma (s08 *s,
		   u32 addr,
		   u08 exp,
		   u08 res,
		   u32 blen,
		   u32 doff,
		   u32 soff,
		   u32 itr)
{
	eprintf("*** ERROR - ");
	if (s)
	   eprintf(s);

	eprintf ("  Src Offset: %08lx  Dest Offset: %08lx  DMA len: %08lx (iteration %u)\n",
	 soff, doff, blen, itr);
	test_err_dma (addr,exp,res);
}

void
test_err_dma (u32 addr,
	      u08 exp,
	      u08 res)
{
	eprintf ("  Address   : %08lx  Expected   : %02x        Got    : %02x\n",
	 addr, exp, res);
}

/* required by byacc */
void
yyerror(s08 *error)
{
}

/* end of error.c */
