/********************************************************************
 * Copyright(c) 2000 Broadcom Corporation, all rights reserved.
 * Contains proprietary and confidential information.
 *
 * This source file is the property of Broadcom Corporation, and
 * may not be copied or distributed in any isomorphic form without 
 * the prior written consent of Broadcom Corporation. 
 *
 *
 *  Name: T 3 _ H I S T . C
 *  FileName: $Header: /Source/Bcm44xx/b44diag/t3_hist.c 3     3/07/02 5:05p Ahui $
 *
 *  Description: This file contains routines to keep track history
 *  of commands.
 *
 *  AUTHOR: Kevin Tran
 *
 *  HISTORY:    Kevin Tran        06/01/2000   Created
 *  $Log: /Source/Bcm44xx/b44diag/t3_hist.c $
 * 
 * 3     3/07/02 5:05p Ahui
 * Added sromutil support
 *
 *******************************************************************/

#include <stdio.h>
#ifndef EFI
#include <graph.h>
#include <conio.h>
#include <bios.h>
#include <signal.h>
#endif
#include <ctype.h>

#include "t3_hist.h"
#include "general.h"
#include "t3_gdata.h"
#include "t3_intr.h"
#include "t3_comm.h"

#define NORMAL_CURSOR	0xd0e
#define BLOCK_CURSOR	7

/* global variables */

s08 (*hist)[HISTORY_LINES][MAX_LINE_LENGTH];	/* 16k */
static int col;					/* starting column */
sig_atomic_t	break_flag;

/* change to menu mode
#ifdef ALTEON
*/
void init_history ()
{
    hist = malloc (HISTORY_LINES * MAX_LINE_LENGTH);
}

void
cleanup_history ()
{
	free (hist);
}

void
abs_gotox (int x)
{
	struct rccoord old = _gettextposition ();
	_settextposition (old.row, x+1);
}
void
abs_gotoxy(int x,int y)
{

  _settextposition (y, x);
}

static void
gotox (int x)
{
    win_gotoCol(col + x);
} 

/* clears a line on the screen */
static void
clear_line (int line)
{
	int i, j = strlen((*hist)[line]);
	gotox (0);
	for (i=0; i<j; i++)
	   printf (" ");
	gotox (0);
}

/* prints a line to the screen */
static void
print_line (int line)
{
	gotox (0);
	_outtext ((*hist)[line]);
}

/* copies one line in memory to another line */
static void
copy_line (int from, int to)
{
	clear_line (to);
	strcpy ((*hist)[to], (*hist)[from]);
	print_line (to);
}

/*
 * Moves 'len' bytes starting at position 'p1' in line to position 'p2' in
 * line.  Overlapping moves are correctly handled.
 */
static void
move_in_line (int line, int p1, int len, int p2)
{
	s08 tmp[MAX_LINE_LENGTH], ch;
	int i;

	for (i=0; i<len; i++)		/* "safe" move */
	   tmp[i] = (*hist)[line][p1+i];
	
	for (i=0; i<len; i++)
	   (*hist)[line][p2+i] = tmp[i];
	
	gotox (p1);
	for (i=0; i<len; i++)
	   printf (" ");
	gotox (p2);
	for (i=0; i<len; i++)
	{
	   ch = (*hist)[line][p2+i];
	   if (isprint(ch))
	      printf ("%c", ch);
	   else
	      printf (" ");
	}
}

/*
 * Gets next line from the user, keeping track of previous lines and allowing
 * the user to use editing keys, arrow keys, etc. in much the same way as a
 * decent unix shell or a 'doskey' like program.
 *
 * Specifically the key definitions are:
 *   UP_ARROW		Last line in history
 *   DOWN_ARROW		Next line in history
 *   LEFT_ARROW		non-destructive move one character left
 *   RIGHT_ARROW	non-destructive move one character right
 *
 *   ESC		erase line (start over)
 *   CTRL-U		erase line (start over)
 *
 *   ERASE		delete one character to the left
 *   CTRL-H		delete one character to the left
 *
 *   DEL		delete character under cursor
 *
 *   RETURN		accept line
 *
 *   INSERT		toggle insert/typeover mode
 */
s08 * get_line (int (*callback)(void))
{
    static int  line = 0;
    static int  mode = 1;	/* 0 = typeover, 1 = insert */
    int         back = line;
    static bool back_limit = TRUE;
    int         x = 0;
    int	        maxx = 0;
    int         old_line;
    bool        stop = FALSE;
    int         i = 0;
    s08        *p;
    int         row;    
    
    win_wherexy(&row, &col);
#ifdef EFI    
    printf(" ");
#endif    

    for (p=(*hist)[line]; p < &(*hist)[line][MAX_LINE_LENGTH];)
        *p++ = '\0';
// fix !*(*hist)[line] = '\0';

    enable_all_interrupt();
    while (1) 
    {
        if (callback!=NULL)
            if (callback())
                break;
                
        fflush (stdout);
        i = _bios_keybrd (_KEYBRD_READY);
            
        if (i==-1)
            Uprintf("<ctrl-c>\n");
        else
        {
            if (i)
                i = _bios_keybrd (_KEYBRD_READ);
            else if (com_isChar())
                i = com_getch();
                
            if (i) 
            {
                switch (i) {
                case ARROW_UP:
                    if (back == 0)
                    {
                        if (!back_limit)	/* can roll under */
                        {
                            back = HISTORY_LINES-1;
                            copy_line (back, line);
                            maxx = x = strlen((*hist)[line]);
                            gotox (x);
                        }
                    }
                    else
                    {
                        back--;
                        copy_line (back, line);
                        maxx = x = strlen((*hist)[line]);
                        gotox (x);
                    }
                    break;
                
                case ARROW_DOWN:
                    if (back < line-1)	/* can go further forward */
                    {
                        back++;
                        copy_line (back, line);
                        maxx = x = strlen((*hist)[line]);
                        gotox (x);
                    }
                    else
                    {
                        x = maxx = 0;
                        clear_line (line);
                        gotox (x);
                    }
                    break;
                
                case ARROW_LEFT:
                case BACKSPACE:
                    if (x > 0)
                    {
                        gotox (--x);
                        com_putch(BACKSPACE);
                    }
                    break;
                
                case ARROW_RIGHT:
                    if (x < maxx)
                        gotox (++x);
                    break;
                
                case ESC:
                case KEY_ESC:
                case CTRL_U:
                    x = maxx = 0;
                    clear_line (line);
                    gotox (x);
                    break;
                
                case CTRL_C:
                    break;		/* fix : do something */
                
                case ERASE:
                case CTRL_H:
                    if (x > 0)
                    {
                        move_in_line (line, x, maxx-x+1, x-1);
                        maxx--;
                        gotox (--x);
                    }
                    break;
                
                case DEL:
                    if (maxx > x)
                    {
                        move_in_line (line, x+1, maxx-x, x);
                        maxx--;
                        gotox (x);
                    }
                    break;
                
                case RET:
                case RETURN:
                    stop = TRUE;
                    x = maxx;
                    (*hist)[line][x] = '\0';
                    Uprintf ("\n");
					if(global.Rs232)
						util_wait_ms(500);
                    break;
                
                case INSERT:
                    if (mode)
                        _settextcursor (BLOCK_CURSOR);
                    else
                        _settextcursor (NORMAL_CURSOR);
                    mode ^= 1;
                    break;
                
                default:
                    if (mode == 0)		/* insert mode */
                    {
                        move_in_line (line, x, maxx-x+1, x+1);
                        maxx++;
                        gotox (x);
                    }
                    (*hist)[line][x++] = (s08)i;
                    if (x > maxx) maxx = x;
                        Uprintf ("%c", (s08)i);
                }  /* switch */           
            } /* if */
        }    
        if (stop)
            break;
    }
    old_line = line;
    if ((*hist)[line][0] != '\0')  /* if line was null, don't save it */
    {
        if (line++ >= HISTORY_LINES)
        {
            line = 0;
            back_limit = FALSE;
        }
    }
    return ((*hist)[old_line]);
}

/* checks for user break. (ctrl-c, ctrl-break, escape) */
int user_break ()
{
//	int i;
    
#ifdef EFI
    return efi_user_break();
#else
    if (break_flag)
    {
        break_flag = 0;
        return 1;
    }
	return 0;
#endif
} 

/***********************************************************
 * sets up "interrupt" for ctrl-c 			   
 * (fix: does not work yet!!)			           
 ***********************************************************/
void break_handler (int sig_number);
void
init_break ()
{
#ifndef EFI
   signal (SIGINT, break_handler);
   signal (SIGBREAK, break_handler);
#endif
   break_flag = 0;
}

void
break_handler (int sig_number)
{
#ifndef EFI
   signal (SIGINT, break_handler);
   signal (SIGBREAK, break_handler);
#endif
   if (global.manuf)
       break_flag = 1;
}

void
clear_break ()
{
   break_flag = 0;
}

/* end of history.c */

