/****************************************************************************
 * Copyright(c) 2000 Broadcom Corporation, all rights reserved.
 * Contains proprietary and confidential information.
 *
 * This source file is the property of Broadcom Corporation, and
 * may not be copied or distributed in any isomorphic form without 
 * the prior written consent of Broadcom Corporation. 
 *
 *
 *  Name: T 3 _ I N T R . C
 *  FileName: $Header: /Source/Bcm44xx/b44diag/t3_intr.c 7     7/19/02 10:07a Ahui $
 *
 *  Description: This file contains interrupt utility routines.
 *
 *  AUTHOR: Kevin Tran
 *
 *  HISTORY:    Kevin Tran        07/01/2000   Created
 *  $Log: /Source/Bcm44xx/b44diag/t3_intr.c $
 * 
 * 7     7/19/02 10:07a Ahui
 * Removed unused code
 * 
 * 6     4/25/02 11:05a Ahui
 * 
 * 3     3/07/02 5:05p Ahui
 * Added sromutil support
 *******************************************************************/

#include <stdio.h>
#ifndef EFI
#include <conio.h>
#include <dos.h>
#endif
#include <ctype.h>
#include "t_io.h"
#include "t3_util.h"
#include "t3_gdata.h"
#include "t3_win.h"
#include "pci.h"
#include "um.h"
#include "t3_diag.h"
#include "Cl4xsb.h"

#define CTRL_C_INTR 0x23
#define INTERRUPT_STACK_SIZE  (128 * 1024)

typedef enum {
   INTR0, INTR1, INTR2, INTR3, INTR4, INTR5, INTR6,INTR7,
   INTR8, INTR9,INTR10,INTR11,INTR12,INTR13,INTR14,INTR15, 
   MAX_INTR
} INTR_t;

/****************************************************************
          static variables 
 ****************************************************************/
 
static u08 org_mask_21, org_mask_a1, org_mask_saved=0;
static u08 run_mask_21, run_mask_a1, run_mask_saved=0;

static void (__interrupt __far *old_isr[MAX_INTR])() = {
   NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,
   NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL
};

//static void (__interrupt __far *old_ctrl_c_isr)() = NULL;

static u08 pc_interrupt[MAX_INTR] = {
    0x8,   // IRQ0
    0x9,   // IRQ1
    0,          /* 2 is special case */
    0xb,   // IRQ3
    0xc,   // IRQ4
    0xd,   // IRQ5
    0xe,   // IRQ6
    0xf,   // IRQ7
    0x70,  // IRQ8
    0x71,  // IRQ9
    0x72,  // IRQ10
    0x73,  // IRQ11
    0x74,  // IRQ12
    0x75,  // IRQ13
    0x76,  // IRQ14
    0x77,  // IRQ15
};

static u32 interrupt_stack[INTERRUPT_STACK_SIZE/4];
static u16 old_ss;
static u32 old_esp;
static void __far *stack_ptr;
#define CHAIN_INTR(x) if (x!=NULL) _chain_intr(x)

/***************************************************************
          interrupt service routine 
 ***************************************************************/
#ifdef EFI
isr_wrapper (int intr)
#else
static isr_wrapper (int intr)
#endif
{
    u08 dev, save_currcard;
	u32 value;
    
    save_currcard = get_currcard_index();
    for (dev = 0; dev < get_device_detected(); dev++)
    {
        if (global.t_cards[dev].interrupt_line == intr) 
        {
            global.current_card = dev;
			value=RegRd( REGOFFSET(intstatus));
            if (value && I_TO)
            {
				RegWr(0x1F94,0);
				RegWr( REGOFFSET(intmask), 0);
				RegWr( REGOFFSET(gptimer), 0);
				RegWr( REGOFFSET(intstatus), I_TO);
				//global.t_cards[dev].isr_callback();
                CURRCARD.gottestint=TRUE;
				CURRCARD.inttest=FALSE;	
			}
        }
    }
    global.current_card = save_currcard;
#ifndef EFI
    /* send end-of-interrupt signal to 8259 */
    outp(0x20, 0x20);
    if (intr > 7)
        outp(0xa0, 0x20);
#endif
}

#ifndef EFI
static void __interrupt isr_wrapper0 (void)
{
    _asm 
    { 
        mov     old_ss, ss
        mov     old_esp, esp
        lss     esp,stack_ptr
    };
    isr_wrapper(INTR0);
    _asm
    {
        mov     ss,old_ss
        mov     esp,old_esp
    };
    CHAIN_INTR(old_isr[INTR0]);
}

static void __interrupt isr_wrapper1 (void)
{
    _asm 
    { 
        mov     old_ss, ss
        mov     old_esp, esp
        lss     esp,stack_ptr
    };
    isr_wrapper(INTR1);
    _asm
    {
        mov     ss,old_ss
        mov     esp,old_esp
    };
    CHAIN_INTR(old_isr[INTR1]);
}

static void __interrupt isr_wrapper3 (void)
{
    _asm 
    { 
        mov     old_ss, ss
        mov     old_esp, esp
        lss     esp,stack_ptr
    };
    isr_wrapper(INTR3);
    _asm
    {
        mov     ss,old_ss
        mov     esp,old_esp
    };
    CHAIN_INTR(old_isr[INTR3]);
}

static void __interrupt isr_wrapper4 (void)
{
    _asm 
    { 
        mov     old_ss, ss
        mov     old_esp, esp
        lss     esp,stack_ptr
    };
    isr_wrapper(INTR4);
    _asm
    {
        mov     ss,old_ss
        mov     esp,old_esp
    };
    CHAIN_INTR(old_isr[INTR4]);
}

static void __interrupt isr_wrapper5 (void)
{
    _asm 
    { 
        mov     old_ss, ss
        mov     old_esp, esp
        lss     esp,stack_ptr
    };
    isr_wrapper(INTR5);
    _asm
    {
        mov     ss,old_ss
        mov     esp,old_esp
    };
    CHAIN_INTR(old_isr[INTR5]);
}

static void __interrupt isr_wrapper6 (void)
{
    _asm 
    { 
        mov     old_ss, ss
        mov     old_esp, esp
        lss     esp,stack_ptr
    };
    isr_wrapper(INTR6);
    _asm
    {
        mov     ss,old_ss
        mov     esp,old_esp
    };
    CHAIN_INTR(old_isr[INTR6]);
}

static void __interrupt isr_wrapper7 (void)
{
    _asm 
    { 
        mov     old_ss, ss
        mov     old_esp, esp
        lss     esp,stack_ptr
    };
    isr_wrapper(INTR7);
    _asm
    {
        mov     ss,old_ss
        mov     esp,old_esp
    };
    CHAIN_INTR(old_isr[INTR7]);
}

static void __interrupt isr_wrapper8 (void)
{
    _asm 
    { 
        mov     old_ss, ss
        mov     old_esp, esp
        lss     esp,stack_ptr
    };
    isr_wrapper(INTR8);
    _asm
    {
        mov     ss,old_ss
        mov     esp,old_esp
    };
    CHAIN_INTR(old_isr[INTR8]);
}

static void __interrupt isr_wrapper9 (void)
{
    _asm 
    { 
        mov     old_ss, ss
        mov     old_esp, esp
        lss     esp,stack_ptr
    };
    isr_wrapper(INTR9);
    _asm
    {
        mov     ss,old_ss
        mov     esp,old_esp
    };
    CHAIN_INTR(old_isr[INTR9]);
}

static void __interrupt isr_wrapper10 (void)
{
    _asm 
    { 
        mov     old_ss, ss
        mov     old_esp, esp
        lss     esp,stack_ptr
    };
    isr_wrapper(INTR10);
    _asm
    {
        mov     ss,old_ss
        mov     esp,old_esp
    };
    CHAIN_INTR(old_isr[INTR10]);
}

static void __interrupt isr_wrapper11 (void)
{
    _asm 
    { 
        mov     old_ss, ss
        mov     old_esp, esp
        lss     esp,stack_ptr
    };
    isr_wrapper(INTR11);
    _asm
    {
        mov     ss,old_ss
        mov     esp,old_esp
    };
    CHAIN_INTR(old_isr[INTR11]);
}

static void __interrupt isr_wrapper12 (void)
{
    _asm 
    { 
        mov     old_ss, ss
        mov     old_esp, esp
        lss     esp,stack_ptr
    };
    isr_wrapper(INTR12);
    _asm
    {
        mov     ss,old_ss
        mov     esp,old_esp
    };
    CHAIN_INTR(old_isr[INTR12]);
}

static void __interrupt isr_wrapper13 (void)
{
    _asm 
    { 
        mov     old_ss, ss
        mov     old_esp, esp
        lss     esp,stack_ptr
    };
    isr_wrapper(INTR13);
    _asm
    {
        mov     ss,old_ss
        mov     esp,old_esp
    };
    CHAIN_INTR(old_isr[INTR13]);
}

static void __interrupt isr_wrapper14 (void)
{
    _asm 
    { 
        mov     old_ss, ss
        mov     old_esp, esp
        lss     esp,stack_ptr
    };
    isr_wrapper(INTR14);
    _asm
    {
        mov     ss,old_ss
        mov     esp,old_esp
    };
    CHAIN_INTR(old_isr[INTR14]);
}

static void __interrupt isr_wrapper15 (void)
{
    _asm 
    { 
        mov     old_ss, ss
        mov     old_esp, esp
        lss     esp,stack_ptr
    };
    isr_wrapper(INTR15);
    _asm
    {
        mov     ss,old_ss
        mov     esp,old_esp
    };
    CHAIN_INTR(old_isr[INTR15]);
}
#endif /* ifndef EFI */


/*
 * enable interrupt 
 * 
 * input irq - interrupt number to be enabled 
 *
 */
void enable_interrupt(u08 irq)
{

    u08 val, chipaddr = 0x21;
    
    outp(0x20, 0x20);     /* send end-of-interrupt signal to 8259 */
                                
    /* sets up interrupt mask */
    if (irq > 7)
    {
        /* the interrupt, may be above 7        */
        outp(0xa0, 0x20); 
        val = inp(0x21);          /* manually enable cascaded 8259 */
        val &= ~(u08)4;
        outp(0x21, val);
        chipaddr = 0xa1;
        irq -= 8;
    }

    val = inp(chipaddr);
    val &= ~(1 << irq);
    outp(chipaddr, val);

}

/*
 *  disable interrupt
 * 
 *  input irq - interrupt number to be disabled
 * 
 */
void disable_interrupt(u08 irq)
{

    u08 val, chipaddr=0x21;

    outp(0x20, 0x20);     /* send end-of-interrupt signal to 8259 */
    
    /* sets up interrupt mask */
    if (irq > 7)
    {
        /* the interrupt, may be above 7        */
        outp(0xa0, 0x20); 
        val = inp(0x21);          /* manually enable cascaded 8259 */
        val &= ~(u08)4;
        outp(0x21, val);
        chipaddr = 0xa1;
        irq -= 8;
    }
  
    val = inp(chipaddr);
    val |= (1 << irq);
    outp(chipaddr, val);

}

//void t3_install_handler(int irq_num,void (far *handler)())

/*
 * takeover interrupt
 * 
 * input intr - interrupt number to take over
 * 
 */

void takeoverIntr(u08 intr)
{
#ifndef EFI
    u08 vect = pc_interrupt[intr];

    if (old_isr[intr] == NULL)   /* never tookover before? */
    {
        old_isr[intr] = _dos_getvect(vect);
        switch (intr) {
            case INTR0: _dos_setvect(vect,isr_wrapper0); break;
            case INTR1: _dos_setvect(vect,isr_wrapper1); break;
            case INTR3: _dos_setvect(vect,isr_wrapper3); break;
            case INTR4: _dos_setvect(vect,isr_wrapper4); break;
            case INTR5: _dos_setvect(vect,isr_wrapper5); break;
            case INTR6: _dos_setvect(vect,isr_wrapper6); break;
            case INTR7: _dos_setvect(vect,isr_wrapper7); break;
            case INTR8: _dos_setvect(vect,isr_wrapper8); break;
            case INTR9: _dos_setvect(vect,isr_wrapper9); break;
            case INTR10: _dos_setvect(vect,isr_wrapper10); break;
            case INTR11: _dos_setvect(vect,isr_wrapper11); break;
            case INTR12: _dos_setvect(vect,isr_wrapper12); break;
            case INTR13: _dos_setvect(vect,isr_wrapper13); break;
            case INTR14: _dos_setvect(vect,isr_wrapper14); break;
            case INTR15: _dos_setvect(vect,isr_wrapper15); break;
        }
    }
    enable_interrupt(intr);
#endif
}

void restore_interrupt(u08 irq_num)
{
#ifndef EFI
    u08 val = org_mask_21, irq = irq_num;
    
    if (old_isr[irq_num] != NULL)
    {
        _dos_setvect (pc_interrupt[irq_num], old_isr[irq_num]);
        old_isr[irq_num] = NULL;

        if (irq_num > 7)
        {
            irq -= 8;
            val = org_mask_a1;
        }
        if (val & (1 << irq)) /* disabled originally? */
            disable_interrupt(irq_num);
    }
#endif
}

void init_dev_interrupt()
{
    run_mask_21 = inp(0x21);
    run_mask_a1 = inp(0xa1);
	run_mask_saved = 1;
}        

/* Some how our interrupt is intermittently masked out when
   run in IKOS PC! This is a hack. */
void enable_all_interrupt()
{
    if (run_mask_saved)
    {
        outp(0x21,run_mask_21);
        outp(0xa1,run_mask_a1);
    }    
}    

void enable_poll_mode()
{
  CURRCARD.poll_mode = 1;
}

void disable_poll_mode()
{
  CURRCARD.poll_mode = 0;
}

void intr_init()
{

    stack_ptr = (void __far *)&interrupt_stack[INTERRUPT_STACK_SIZE-1];

	if (!org_mask_saved)
	{
        org_mask_21 = inp(0x21);
        org_mask_a1 = inp(0xa1);
		org_mask_saved = 1;
	}

}

/*
 * interrupt clean up routine
 *
 *    1. Restore all interrupts
 *    2. Restore original system interrupt mask 
 */
void intr_cleanup()
{

    u08 intr;

    for (intr = 0; intr < MAX_INTR; intr++)
	    restore_interrupt(intr);

	if (org_mask_saved)
	{
		outp(0x21,org_mask_21);
		outp(0xa1,org_mask_a1);
		org_mask_saved = 0;
	}

}



