/********************************************************************
 * Copyright(c) 2000 Broadcom Corporation, all rights reserved.
 * Contains proprietary and confidential information.
 *
 * This source file is the property of Broadcom Corporation, and
 * may not be copied or distributed in any isomorphic form without 
 * the prior written consent of Broadcom Corporation. 
 *
 *
 *  Name: T 3 _ P R T F . C
 *  FileName: $Header: /Source/Bcm44xx/b44diag/t3_prtf.c 4     7/19/02 10:07a Ahui $
 *
 *  Description: This file contains printf utility routines.
 *
 *  AUTHOR: Kevin Tran
 *
 *  HISTORY:    Kevin Tran        06/01/2000   Created
 *  $Log: /Source/Bcm44xx/b44diag/t3_prtf.c $
 * 
 * 4     7/19/02 10:07a Ahui
 * Removed unused code
 * 
 * 3     3/07/02 5:05p Ahui
 * Added sromutil support
 *******************************************************************/

#include <stdarg.h>
#ifndef EFI
#include <io.h>
#include <graph.h>
#endif
#include "t3_prtf.h"
#include "general.h"
#include "t3_gdata.h"
#include "t3_win.h"
#include "t3_comm.h"

#define STDPRN_FILENO   4

static t3_print(s08 * buff)
{
    if (win_empty())
	    _outtext (buff);
    else
        win_print(buff);

    if (global.verbose & V_PRINTER)  /* printer is turned on */
        write(STDPRN_FILENO,global.pbuf,strlen(global.pbuf));

#ifndef EFI
    com_puts(buff);
#endif
}

/* unconditional printf */
int Uprintf (s08 *fmt, ...)
{
	va_list arglist;
	int retval;

    va_start (arglist, fmt);
    retval = vsprintf (global.pbuf, fmt, arglist);
    _outtext (global.pbuf);
#ifndef EFI
    com_puts(global.pbuf);
#endif
    if (global.logfile) fprintf (global.logfile, global.pbuf);
    va_end (arglist);
    return retval;
}

/* replaces printf */
int Printf (s08 *fmt, ...)
{
	va_list arglist;
	int retval;

	if (global.verbose & V_CONSOLE)
	{
        va_start (arglist, fmt);
        retval = vsprintf (global.pbuf, fmt, arglist);
        t3_print(global.pbuf);
        if (global.logfile) fprintf (global.logfile, global.pbuf);
        va_end (arglist);
        return retval;
	}
    return 0;
}

/* print out for debug information */
int Dprintf (s08 *fmt, ...)
{
	va_list arglist;
	int retval;

	if (global.verbose & V_DEBUG)
	{
	    va_start (arglist, fmt);
	    retval = vsprintf (global.pbuf, fmt, arglist);
        t3_print(global.pbuf);
	    if (global.logfile) fprintf (global.logfile, global.pbuf);
	    va_end (arglist);
        return retval;
	}
    return 0;
}

/* prints out to printer */
int Pprintf (s08 *fmt, ...)
{
	va_list arglist;
	int retval;

    va_start (arglist, fmt);
    retval = vsprintf (global.pbuf, fmt, arglist);
    write(STDPRN_FILENO,global.pbuf,strlen(global.pbuf));
    va_end (arglist);
    return retval;
}

/* prints errors */
int Eprintf (s08 *fmt, ...)
{
	va_list arglist;
	int retval;

	if (global.verbose & V_ERROR)
	{
        va_start (arglist, fmt);
        retval = vsprintf (global.pbuf, fmt, arglist);
        t3_print (global.pbuf);
        if (global.logfile) fprintf (global.logfile, global.pbuf);
        va_end (arglist);
        return retval;
	}
    return 0;
}

/* prints warnings */
int Wprintf (s08 *fmt, ...)
{
	va_list arglist;
	int retval;

	if (global.verbose & V_WARNING)
	{
        va_start (arglist, fmt);
        retval = vsprintf (global.pbuf, fmt, arglist);
        t3_print (global.pbuf);
        if (global.logfile) fprintf (global.logfile, global.pbuf);
        va_end (arglist);
        return retval;
    }
    return 0;
}

/* I/O print */
int Iprintf (s08 *fmt, ...)
{
	if (global.verbose & V_IO)
	{
        va_list arglist;
        int retval;

        va_start (arglist, fmt);
        retval = vsprintf (global.pbuf, fmt, arglist);
        t3_print (global.pbuf);
        if (global.logfile) fprintf (global.logfile, global.pbuf);
        va_end (arglist);
        return retval;
	}
    return 0;
}

/* prints based on parameter info */
int Vprintf (int verbose, s08 *fmt, ...)
{
	if (verbose && (global.verbose & V_CONSOLE))
	{
        va_list arglist;
        int retval;

        va_start (arglist, fmt);
        retval = vsprintf (global.pbuf, fmt, arglist);
        t3_print (global.pbuf);
        if (global.logfile) fprintf (global.logfile, global.pbuf);
        va_end (arglist);
        return retval;
	}
    return 0;
}



/* end of printf.c */
