/****************************************************************************
 * Copyright(c) 2000 Broadcom Corporation, all rights reserved.
 * Contains proprietary and confidential information.
 *
 * This source file is the property of Broadcom Corporation, and
 * may not be copied or distributed in any isomorphic form without 
 * the prior written consent of Broadcom Corporation. 
 *
 *
 *  Name: T 3 _ S C R N . H
 *
 *  Description: This file contains command line interface macros
 *  and data structures.
 *
 *  AUTHOR: Kevin Tran
 *
 *  HISTORY:    Kevin Tran        06/01/2000   Created
 *
 *******************************************************************/

#ifndef __T3_SCRN_H__
#define __T3_SCRN_H__

#ifndef EFI
#include <graph.h>
#endif
#include "general.h"

#ifdef EFI
#define CHAR CHAR16
#else
#define CHAR u08
#endif

#define WHITE       15
#define YELLOW      14
#define LIGHT_CYAN  11
#define GRAY        7
#define BROWN       6
#define RED         4
#define CYAN        3
#define GREEN       2
#define BLUE        1
#define BLACK       0
#define FLASH       8

#define BACKGROUND(c) (c<<4)
#define ATTRIB(f,b) (f|BACKGROUND(b))
#define scrn_homeCursor(s) scrn_gotoxy(s,0,0)

#define NORMAL_COLOR	GRAY	
#define DIR_COLOR	    LIGHT_CYAN
#define CMD_COLOR	    CYAN	
#define HELP_COLOR	    YELLOW	
#define BANNER_COLOR	WHITE
#define PASS_COLOR	    GREEN	
#define FAIL_COLOR	    RED	

#define SCREEN_BASE(x) (x->buffer)
#define SCREEN_END(x)  (SCREEN_BASE(x) + (x->csize * x->rsize))
#define pSCREEN &(__scrn)
#define SCREEN (__scrn)
#define MSGPOSITIONS 5,0,8,79

#ifndef EFI
    #define L                /* No need for unicode extension on constants */

    #define BOXDRAW_HORIZONTAL                    '\xc4'
    #define BOXDRAW_VERTICAL                      '\xb3'
    #define BOXDRAW_DOWN_RIGHT                    '\xda'
    #define BOXDRAW_DOWN_LEFT                     '\xbf'
    #define BOXDRAW_UP_RIGHT                      '\xc0'
    #define BOXDRAW_UP_LEFT                       '\xd9'
    #define BOXDRAW_VERTICAL_RIGHT                '\xc3'
    #define BOXDRAW_VERTICAL_LEFT                 '\xb4'
    #define BOXDRAW_DOWN_HORIZONTAL               '\xc2'
    #define BOXDRAW_UP_HORIZONTAL                 '\xc1'
    #define BOXDRAW_VERTICAL_HORIZONTAL           '\xc5'

    #define BOXDRAW_DOUBLE_HORIZONTAL             '\xcd'
    #define BOXDRAW_DOUBLE_VERTICAL               '\xba'

    #define BOXDRAW_DOWN_RIGHT_DOUBLE             '\xc9'
    #define BOXDRAW_DOWN_DOUBLE_RIGHT             '\xd6'
    #define BOXDRAW_DOUBLE_DOWN_RIGHT             '\xd5'

    #define BOXDRAW_DOWN_LEFT_DOUBLE              '\xbb'
    #define BOXDRAW_DOWN_DOUBLE_LEFT              '\xb7'
    #define BOXDRAW_DOUBLE_DOWN_LEFT              '\xb8'

    #define BOXDRAW_UP_RIGHT_DOUBLE               '\xc8'
    #define BOXDRAW_UP_DOUBLE_RIGHT               '\xd3'
    #define BOXDRAW_DOUBLE_UP_RIGHT               '\xd4'

    #define BOXDRAW_UP_LEFT_DOUBLE                '\xbc'
    #define BOXDRAW_UP_DOUBLE_LEFT                '\xbd'
    #define BOXDRAW_DOUBLE_UP_LEFT                '\xbe'

    #define BOXDRAW_VERTICAL_RIGHT_DOUBLE         '\xcc'
    #define BOXDRAW_VERTICAL_DOUBLE_RIGHT         '\xc6'
    #define BOXDRAW_DOUBLE_VERTICAL_RIGHT         '\xc7'

    #define BOXDRAW_VERTICAL_LEFT_DOUBLE          '\xb9'
    #define BOXDRAW_VERTICAL_DOUBLE_LEFT          '\xb5'
    #define BOXDRAW_DOUBLE_VERTICAL_LEFT          '\xb6'

    #define BOXDRAW_DOWN_HORIZONTAL_DOUBLE        '\xcb'
    #define BOXDRAW_DOWN_DOUBLE_HORIZONTAL        '\xd1'
    #define BOXDRAW_DOUBLE_DOWN_HORIZONTAL        '\xd2'

    #define BOXDRAW_UP_HORIZONTAL_DOUBLE          '\xca'
    #define BOXDRAW_UP_DOUBLE_HORIZONTAL          '\xcf'
    #define BOXDRAW_DOUBLE_UP_HORIZONTAL          '\xd0'

    #define BOXDRAW_VERTICAL_HORIZONTAL_DOUBLE    '\xce'
    #define BOXDRAW_VERTICAL_DOUBLE_HORIZONTAL    '\xd8'
    #define BOXDRAW_DOUBLE_VERTICAL_HORIZONTAL    '\xd7'
#endif

#define ULCH1 '\xda'
#define URCH1 '\xbf'
#define VCH1  '\xb3'
#define HCH1  '\xc4'
#define LRCH1 '\xd9'
#define LLCH1 '\xc0'
#define UPTCH1 '\xc2'
#define DNTCH1 '\xc1'
#define LTTCH1 '\xc3'
#define RTTCH1 '\xb4'
#define ULCH2 '\xc9'
#define URCH2 '\xbb'
#define VCH2  '\xba'
#define HCH2  '\xcd'
#define LRCH2 '\xbc'
#define LLCH2 '\xc8'

#if 0
const CHAR s_line_set[] = {BOXDRAW_UP_LEFT, BOXDRAW_HORIZONTAL,
                                BOXDRAW_UP_RIGHT, BOXDRAW_VERTICAL,
                                BOXDRAW_DOWN_LEFT, BOXDRAW_DOWN_RIGHT};
const CHAR d_line_set[] = {BOXDRAW_UP_LEFT_DOUBLE, BOXDRAW_DOUBLE_HORIZONTAL,
                                BOXDRAW_UP_RIGHT_DOUBLE, BOXDRAW_DOUBLE_VERTICAL,
                                BOXDRAW_DOWN_LEFT_DOUBLE, BOXDRAW_DOWN_RIGHT_DOUBLE};
#endif

typedef enum 
{
    SINGLE_LINE,
    DOUBLE_LINE
} line_t;

typedef s16 * scrnPtr;


typedef struct
{
    CHAR ch;
    u08 attrib;
} ScreenNode, *ScreenPtr;

typedef struct 
{
    s16 r,c;
} ScreenPoint, *pScreenPoint;

typedef struct
{
    ScreenPoint ul;
    ScreenPoint lr;
} ScreenArea, *pScreenArea;

typedef enum {
    HIDE_CURSOR,
    THIN_CURSOR,
    THICK_CURSOR,
} CURSOR_t;

typedef struct
{
//    ScreenPoint      pos;
    ScreenPtr     buffer;
    ScreenPtr  cursorPtr;
    ScreenPoint   cursor;
    ScreenArea      area;
    int     rsize, csize;
    u08 attrib;
} Screen, * pScreen;

extern Screen __scrn;

scrnPtr scrn_scrnPtr(pScreen s, ScreenPoint p);
void scrn_xyPrintf(pScreen scrn, int x, int y, s08 *fmt, ...);
void scrn_writeXYCh(pScreen scrn, int x1,int y1,CHAR c);
void scrn_writeCh(pScreen scrn, CHAR c, int scroll);
ScreenPtr scrn_saveScreen(int,int,int,int);
ScreenPtr scrn_save(void);
void scrn_restoreScreen(int,int,int,int,ScreenPtr);
void scrn_restore(ScreenPtr);
void scrn_xyPrint(pScreen scrn, int row,int col,s08 * msg);
void scrn_printPad(pScreen scrn, s08 * st, int len);
void scrn_xyPrintPad(pScreen scrn, int row,int col,s08 * msg, int len);
void scrn_xyPrintfPad(pScreen scrn, int x, int y, int len, s08 *fmt, ...);
void scrn_box(pScreen scrn, line_t type, int ur,int uc, int lr, int lc);
void scrn_printf(pScreen scrn, s08 *fmt, ...);
void scrn_xyPprintf(pScreen scrn, int, int, s08 *fmt, ...);
void scrn_print(pScreen scrn, s08 * str);
void scrn_gotoxy(pScreen scrn,int r, int c);
ScreenPtr scrn_moveCursur(pScreen p, int i);
void scrn_clear(pScreen scrn);
s08 scrn_dispWait(s08 *fmt, ...);
void scrn_hideCursor();
void scrn_showThickCursor();
void scrn_showCursor();
void scrn_absPoint(pScreen scrn, pScreenPoint p);
void scrn_userPointToScrnPoint(pScreen s, pScreenPoint up);
void scrn_userPointToAbsPoint(pScreen s, pScreenPoint up);
//void scrn_scrnPointToAbsPoint(pScreen s, pScreenPoint sp);
void scrn_setConsole(pScreen scrn, int r1, int c1, int r2, int c2);
void scrn_line(pScreen s, line_t border, line_t line,int r,int c,int w);
void scrn_replicate(pScreen, CHAR,int);
void scrn_addShadow(int,int,int,int);
int  scrn_yes(s08 *);
ScreenPtr scrn_dispMsg(s08 *);
u08 scrn_setAttrib(pScreen, u08);
void scrn_boxOnlyWithShadow(line_t type, int ur,int uc, int lr, int lc);
void scrn_boxOnly(pScreen scrn, line_t type, int ur,int uc, int lr, int lc);
void scrn_boxWithShadow(line_t type, int ur,int uc, int lr, int lc);
void scrn_init();
void refreshScreen(int x1, int y1, int x2, int y2);
#endif /* __T3_SCRN_H__ */
