/********************************************************************
 * Copyright(c) 2000 Broadcom Corporation, all rights reserved.
 * Contains proprietary and confidential information.
 *
 * This source file is the property of Broadcom Corporation, and
 * may not be copied or distributed in any isomorphic form without 
 * the prior written consent of Broadcom Corporation. 
 *
 *  FileName: $Header: /Source/Bcm44xx/b44diag/t3_tests.h 8     6/11/02 11:52a Ahui $
 *
 *  Description: This file contains test control routines
 *
 *  $Log: /Source/Bcm44xx/b44diag/t3_tests.h $
 * 
 * 8     6/11/02 11:52a Ahui
 * Added test_pasted0 function.
 * Added error code to support fmac option.
 * 
 * 7     5/29/02 7:41p Ahui
 * 
 * 5     4/25/02 11:04a Ahui
 * Enabled C3 external tests.
 * 
 * 3     3/07/02 5:05p Ahui
 * Added sromutil support 
 *****************************************************************/
 
#ifndef __T3_TESTS_H__
#define __T3_TESTS_H__

#define DOT_POSITION 38
#define check_valid_nic(n) if ((n=UM_GetNIC())==NULL) return test_failed0(ERR_BAD_NIC)
#define check_abort() if (util_user_abort()) return test_failed0(ERR_ABORT)
#define check_nic() if (UM_GetNIC()==NULL) return test_failed0(ERR_BAD_NIC)
#define check_not_chip_running(n,d) if ((d=n->DeviceInst)!=NULL) test_failed0(ERR_CHIP_RUNNING)
#define checkDriverNotLoaded()                  \
{                                               \
    pNIC_BLK pNicBlk;                           \
                                                \
    check_valid_nic(pNicBlk);                   \
    if (pNicBlk->DeviceInst != NULL)            \
        return test_failed0(ERR_CHIP_RUNNING);  \
}
#define checkDriverIsLoaded()                   \
{                                               \
    pNIC_BLK pNicBlk;                           \
                                                \
    check_valid_nic(pNicBlk);                   \
    if (pNicBlk->DeviceInst == NULL)            \
        return test_failed0(ERR_CHIP_NOT_RUNNING);\
}

/* 
    Every time, there is test change, for example, add or delete a test from the structure, 
    we should change this version.
    At the start up, the program is trying to load configuration from "diagcfg.bin", if there is 
    structure change, (when the version does not match, it should go back to default.)
*/
#define TEST_VERSION 0x3
    
typedef enum {
    GROUP_REG,
    //GROUP_MEM,
    GROUP_MISC,
    GROUP_DATA,
    MAX_TEST_GROUP
} GROUP_t;

typedef enum {
    REG_INDIRECT,
    REG_CONTROL,
    REG_INTERRUPT,
    REG_BIST,
    MAX_REG_TEST
} REG_SUB_t;

typedef enum {
    MEM_SPAD,
    //MEM_BD,
    //MEM_DMA,
    //MEM_MBUF,
    //MEM_MBUF_DMA,
    //MEM_SPECIAL,
    //MEM_EXT,
	//MEM_BD_REL,
    MAX_MEM_TEST
} MEM_SUB_t;

typedef enum {
	MISC_LED,
    MISC_EEPROM,
	MISC_MII,
	MISC_LINK,
    //MISC_CPU,
    //MISC_DMA,
    //MISC_VPD,
    //MISC_ASF,
    //MISC_ROM,
    MAX_MISC_TEST
} MISC_SUB_t;

typedef enum {
    DATA_MAC,
    DATA_PHY,
    DATA_RJ45_100,
	DATA_RJ45_10,
    //DATA_MSI,
    MAX_DATA_TEST
} DATA_SUB_t;

#define HAS_ERROR 1

typedef enum {
    NO_ERROR,                   // 0
    ERR_IND_REG_ERR,            // 1
    ERR_CHIP_RUNNING,           // 2
    ERR_BAD_NIC,                // 3
    ERR_READ_ONLY_CLEAR,        // 4
    
    ERR_READ_ONLY_SET,          // 5
    ERR_READ_WRITE_NOT_CLEAR,   // 6
    ERR_READ_WRITE_NOT_SET,     // 7
    ERR_BIST,                   // 8
    ERR_INTERRUPT,              // 9
    
    ERR_ABORT,                  // 10
    ERR_DMA_TXDATA,             // 11
    ERR_DMA_RXDATA,             // 12
    ERR_TXDMA,                  // 13
    ERR_RXDMA,                  // 14
    
    ERR_MEM,                    // 15
    ERR_MEM2,                   // 16
    ERR_EEP_WRITE,              // 17
    ERR_EEP_READ,               // 18
    ERR_EEP_DATA,               // 19
    
    ERR_FILE_OPEN,              // 20
    ERR_LOOPBACK_TYPE,          // 21
    ERR_IMAGE_SIZE,             // 22
    ERR_MALLOC,                 // 23
    ERR_CPU_RESET,              // 24

    ERR_CPU_RELEASE,            // 25
    ERR_CPU_TEST,               // 26
    ERR_DMA_RANGE,              // 27    
    ERR_DMA_DATA,               // 28
    ERR_RX_LEN,                 // 29

    ERR_PHY_TOO_MANY_REG,       // 30
    ERR_VPD_WRITE,              // 31
    ERR_VPD_DATA,               // 32
    ERR_NO_LINK,                // 33
    ERR_DATA_TX,                // 34

    ERR_DATA_TX_MISSING,        // 35
    ERR_DATA_RX_MISSING,        // 36
    ERR_INVALID_TEST,           // 37
    ERR_EEPROM_CHECKSUM,        // 38
    ERR_READING_WOL_PXE,        // 39
    
    ERR_WRITING_WOL_PXE,        // 40
    ERR_NO_EXT_SRAM,            // 41
    ERR_DMA_LEN,                // 42
    ERR_FILE_TOO_BIG,           // 43
    ERR_INVALID,                // 44
    
    ERR_WRITE,                  // 45
    CMD_QUIT,                   // 46
    ERR_CPU_MEM_ERR,            // 47
    ERR_COMMAND,                // 48
    ERR_OPTION,                 // 49
    
    ERR_CHIP_NOT_RUNNING,       // 50
    ERR_NO_REG_DEF,             // 51
    ERR_ASF_RST,                // 52
    ERR_ASF_ATTN_LOC,           // 53
    ERR_ASF_RST_VAL,            // 54
    
    ERR_ASF_PA_TIMER,           // 55
    ERR_ASF_PA_CLEAR,           // 56
    ERR_ADF_NO_STAMP,           // 57
    ERR_ADF_NO_TIMER,           // 58
    ERR_ASF_EVENT,              // 59
    
    ERR_EEP_FILESIZE,           // 60
    ERR_MAGIC_VALUE,            // 61
    ERR_EEP_FMT,                // 62
    ERR_EEP_BOOTVER,            // 63
    ERR_EEP_CANNOT_WRITE,       // 64

    ERR_EEP_CANNOT_READ,        // 65
    ERR_BAD_CHECKSUM,           // 66
    ERR_BAD_MAGIC_VALUE,        // 67
    ERR_MAC,                    // 68
    ERR_BUS,                    // 69
    
    ERR_SPEC_MEM,               // 70
    ERR_NOT_SUPPORT,            // 71
	ERR_BAD_CRC,				// 72
	ERR_MII_ERR_BITS_SET,		// 73
	ERR_INIT_MAC,				// 74
    
    ERR_FW_FILE_FORMAT,         // 75
    ERR_RESET_TX_CPU,           // 76
    ERR_RESET_RX_CPU,           // 77
    ERR_INVALID_MAC_ADDR,       // 78
    ERR_MAC_REG,                // 79
    
    ERR_BOOTCHECKSUM,           // 80
    ERR_VPD_READONLY,           // 81
    ERR_VPD_READ,               // 82
    ERR_MEM_READ,               // 83
    ERR_MEM_WRITE,              // 84
    
    ERR_PXE_PGM,                // 85
    ERR_PXE_VFY,                // 86
    ERR_EXT_MEM_EXE_TIMOUT,     // 87
    ERR_EXT_MEM_SIZE,           // 88
    ERR_RESET_TIMEOUT,          // 89
    
    ERR_MSI_ERR_NOTCLEAR,       // 90
    ERR_MSI_DATA,               // 91
	ERR_MEM_POOL_INIT,          // 92
	ERR_MEM_POOL_UNINIT,        // 93
    ERR_PCI_REGS_WIDTH,         // 94
    
    ERR_LINK_STATUS,            // 95
    ERR_PHY_INTERRUPT,          // 96
    ERR_EEP_BIT_BANG,           // 97
    ERR_ROM_SIZE,               // 98
    ERR_ROM_DATA,               // 99
    
    ERR_ROM_ENABLE,             // 100
    ERR_GPIO,                   // 101
    ERR_GPIO5704,               // 102
    ERR_BIST_NOT_DONE,          // 103
    ERR_BIST_DATA_MISCMP,       // 104
    ERR_CPU_NO_RESPONSE,        // 105
    ERR_CPU_ERR_CODE,           // 106
	ERR_RX_CORR,				// 107
	ERR_BAD_CPU_CFG,			// 108
	ERR_BIST_SBTMSTATE,         // 109 
	ERR_RX_HEADER,              // 110
	ERR_RX_CONT,                // 111
	ERR_RX_INSTATUS,            // 112
	ERR_EEP_CRC,				// 113
    ERR_MAC_ADDR_ENDED,         // 114
    ERR_ILLEGAL_MAC_ADDR,       // 115 
    MAX_ERR                     
};

/*************************************************
 * function test_doAllTests
 *   performs all tests
 *
 * input: req_iteration -- requested iteration
 *        devMap -- it passes the device need to be tested. This passes in
 *                  bit map. If bit0 is set, it indicates device 0 is selected
 * output: none
 *
 * return: test result
 *************************************************/
int test_doAllTests(u32 req_iteration, u32 devMap, u32 prog_OWL_PXE, int eepProg);

/**************************************************************
 * function test_checkTestStr
 * reads a test select as string in enable/disable the test.
 *
 * input: enable - 1:enable, 0:disable tests
 *        p      - parameter string
 *
 * output: prints out error message if error is detected
 *
 * return: 0 or no error, 1 for error detected
 *
 * side effects: none
 *************************************************************/
int test_checkTestStr(int enable, s08 * p);

/*************************************************
 * function test_runTest
 *   save as do_test, this is just engineering version of test
 *   the difference is it prints '\n' if test passes. 
 *
 * input: grp -- test group
 *        id - test id
 *        para -- test parameter
 *        abs -- if true, always run test regardless of enable status
 *
 * output: none
 *
 * return: test result
 *
 * side effect: Once this function is called, it set "tested" flag
 *              internally to indicate this test has been run before
 *
 *************************************************/
int test_runTest(u32 req_iteartion, int grp,int id,void * para);

/*************************************************
 * function test_testGroup
 *   engineering version of test_group. The difference is it prints out
 *   new line if it passes the tests.
 *
 * input: req_iteration -- requested iteration
 *        devMap -- it passes the device need to be tested. This passes in
 *                  bit map. If bit0 is set, it indicates device 0 is selected
 * output: none
 *
 * return: test result
 *************************************************/
int test_testGroup(u32 req_iteration, int grp, u32 sel);

/*************************************************
 * function test_maxTest
 *   returns a constant number of tests available in that group
 *
 * input: grp - group number
 *  
 * output: none
 *
 * return: The number of tests availabe in that group
 * 
 **************************************************/
int test_maxTest(int grp);

/*************************************************
 * function test_title
 *   returns test title
 *
 * input: grp -- test group
 *        id  -- test id
 *
 * output: none
 *
 * return: the number of tests
 *
 *************************************************/
const s08 * test_title(int grp, int id);
 
/*************************************************
 * function test_groupTitle
 *   returns test group title
 *
 * input: grp -- test group
 *
 * output: none
 *
 * return: the number of tests
 *
 *************************************************/
const s08 * test_groupTitle(int grp);
 
/*************************************************
 * function test_enable 
 *   Enables the test
 *
 * input: grp, id - test id
 * 
 * output: none
 *
 * return: none
 *************************************************/
int test_enable(int grp, int test);

/*************************************************
 * function test_enableGrpTests 
 *   Enables the test, parameter are passed as bit mapped
 *
 * input: grp, bits
 * 
 * output: none
 *
 * return: none
 *************************************************/
int test_enableGrpTests(int grp, u32 sel);

/*************************************************
 * function test_disable 
 *   disable the test
 *
 * input: id - test id
 * 
 * output: none
 *
 * return: none
 *************************************************/
int test_disable(int grp, int test);

/*************************************************
 * function test_disableGrpTests 
 *   Disables the test, parameter are passed as bit mapped
 *
 * input: grp, bits
 * 
 * output: none
 *
 * return: none
 *************************************************/
int test_disableGrpTests(int grp, u32 sel);

/*************************************************
 * function test_enabled
 *   return zero if test is not enabled else it returns
 *          a non-zero number 
 *
 * input: grp,id - test id
 *  
 * output: none
 *
 * return: non-zero if enabled, zero if disabled
 * 
 **************************************************/
int  test_enabled(int grp, int id);

/*************************************************
 * function test_failed
 *   performs all tests
 *
 * input: errCode - error code
 *        para list - arguments
 *
 * output: none
 *
 * return: error code
 *
 *************************************************/
int test_failed(int errCode, int p1, int p2, int p3, int p4, int p5, int p6, int p7);
#define test_failed0(e) test_failed(e,0,0,0,0,0,0,0)
#define test_failed1(e,p) test_failed(e,(int)(p),0,0,0,0,0,0)
#define test_failed2(e,p1,p2) test_failed(e,(int)(p1),(int)(p2),0,0,0,0,0)
#define test_failed3(e,p1,p2,p3) test_failed(e,(int)(p1),(int)(p2),(int)(p3),0,0,0,0)
#define test_failed4(e,p1,p2,p3,p4) test_failed(e,(int)(p1),(int)(p2),(int)(p3),(int)(p4),0,0,0)
#define test_failed5(e,p1,p2,p3,p4,p5) test_failed(e,(int)(p1),(int)(p2),(int)(p3),(int)(p4),(int)(p5),0,0)
#define test_failed6(e,p1,p2,p3,p4,p5,p6) test_failed(e,(int)(p1),(int)(p2),(int)(p3),(int)(p4),(int)(p5),(int)(p6),0)
#define test_failed7(e,p1,p2,p3,p4,p5,p6,p7) test_failed(e,(int)(p1),(int)(p2),(int)(p3),(int)(p4),(int)(p5),(int)(p6),(int)(p7))

/*****************************************************************
 * test_selectedGrpTests(int grp)
 *   returns the selected test bit map
 *   
 *****************************************************************/
u32  test_selectedGrpTests(int grp);

/*************************************************
 * function test_passed
 *   display 'passed' and return NO_ERROR
 *
 * input: none
 *  
 * output: none
 *
 * return: NO_ERROR
 **************************************************/
int test_passed(void);
int test_passed0(void);
int link_status(u08 stat);

/*************************************************
 * function test_notAvailable
 *   display 'N/A' and return NO_ERROR
 *
 * input: none
 *  
 * output: none
 *
 * return: NO_ERROR
 **************************************************/
int test_notAvailable(void);

/*************************************************
 * test_bitString 
 *    convert bits into a string
 *
 * side effect: Since this routine uses static buffer to return the string, 
 *              it cannot be called in one single statement:
 *       
 *              printf("the bit string is %s and %s",bitSTring(1), test_bitString(0xff)); -- won't work
 *
 *              should be changed to:
 *
 *              printf("the bit string is %s", test_bitString(1));        -- Good 
 *              printf(" and %s",test_bitString(0xff));
 *************************************************/
const s08 * test_bitString(u32 data32);

/*************************************************
 * Show current test setting's dots
 *************************************************/
void test_showDot(void);

/*************************************************
 * function test_setGrpTests
 *   Use the parameter as is to disable or enable the tests
 *
 * input: grp, id - test id
 * 
 * output: none
 *
 * return: none
 *************************************************/
void test_setGrpTests(int grp, u32 tests);


//void reset_tests(void);
/*************************************************
 * function test_tested
 *   returns true if the test is run previousely
 *
 * input: grp,id -- test id
 *  
 * output: none
 *
 * return: zero if the test has never been tested
 *         otherwise, returns non-zero value
 **************************************************/
//int  test_tested(int grp, int id);
void startTest(void);


/**************************************************/
set_current_test(int g,int i);

/*****************************************************************
 * showIteration
 *     if req_iteartion is not 1, then show the itearation 
 *
 * in - iteration : current iteration
 *      req_iteration : requested iteration
 *****************************************************************/ 
void showIteration(u32 iter, u32 req_iteration);
int test_setError(int code);
#endif /* __T3_TESTS_H__ */
