/********************************************************************
 * Copyright(c) 2000 Broadcom Corporation, all rights reserved.
 * Contains proprietary and confidential information.
 *
 * This source file is the property of Broadcom Corporation, and
 * may not be copied or distributed in any isomorphic form without 
 * the prior written consent of Broadcom Corporation. 
 *
 *
 *  Name: T 3 _ U T I L . C
 *  FileName: $Header: /Source/Bcm44xx/b44diag/t3_util.c 13    7/19/02 10:07a Ahui $
 *
 *  Description: This file contains utility routines and diagnostics
 *               routines to checkout T3.
 *
 *  AUTHOR: Kevin Tran
 *******************************************************************/
/** include files **/
#include <stdio.h>
#include <stdarg.h>
#include <ctype.h>
#include <fcntl.h>
#include <sys/stat.h>
#include <time.h>

#ifndef EFI
#include <conio.h>
#include <bios.h>
#include <graph.h>
#include <io.h>
#endif
#include "Cl4xsb.h"
#include "general.h"
#include "error.h"
#include "t_io.h"
#include "t3_util.h"
#include "t3_mii.h"
#include "fw_img.h"
#include "protocol.h"
#include "t3_gdata.h"
#include "eeprom.h"
#include "t3_scrn.h"
#include "t3_tests.h"
#include "t3_eprom.h"
#include "t3_win.h"
#include "t3_cli.h"
#include "pci.h"
#include "t3_hist.h"
#include "t3_eecfg.h"
#include "t3_intr.h"
#include "um.h"
#include "t3_tx.h"
#include "t3_var.h"
#include "t3_flash.h"
#include "t3_comm.h"

s08 *test_speed_str[]=
{
  "Triple",
  "10",
  "100",
  "1000"
};



s08 *phy_speed_str[] =
{
  "Unknown",
  "10Base-T Half Duplex",
  "10Base-T Full Duplex",
  "100Base-TX Half Duplex",
  "100Base-T4",
  "100Base-TX Full Duplex",
  "1000Base-T Half Duplex",
  "1000Base-T Full Duplex",
  "1000 TBI Full Duplex",
};

						   
int my_delay = 100;
u32 tigon_tx = FALSE;

#define CPUA_CON_STRCT (RX_CPU_CODE_SPACE + SYSTEM_RAM - SYSTEM_VAR_SIZE - \
                        SYSTEM_STACK_SIZE + OFFSETOF(SystemAVar, comm.prn))
#define CPUB_CON_STRCT (TX_CPU_CODE_SPACE + SYSTEM_RAM - SYSTEM_VAR_SIZE - \
                        SYSTEM_STACK_SIZE + OFFSETOF(SystemBVar, comm.prn))
#define CPU_TX_COLOR YELLOW
#define CPU_RX_COLOR GREEN
                       
/*****************************************/
/*********** private functions ***********/
/*****************************************/

#define DEV_ROW 0
#define DEV_COL 0
#define DEV_NAME_SIZE 80

typedef struct
{
    s08 name[DEV_NAME_SIZE+1];
    wEdCmd_t cli;
} devt, *pDev;

/************************************************************
  private functions 
 ************************************************************/
 
/***********************************************************************/
static void check_message(u32 offset, u08 attrib)
{
    db_print_t buff;
    u32 * msg = (u32 *) buff.db_data_out;

    buff.db_ctrl = util_read_isram(offset+OFFSETOF(db_print_t,db_ctrl));
    if (buff.db_ctrl & DB_CTRL_BUSY)
    {
        util_read_isram_blk(offset+OFFSETOF(db_print_t,db_para1), &buff.db_para1, (CBUFSIZE + 8) >> 2);
        util_write_isram(offset+OFFSETOF(db_print_t,db_ctrl), 0);
        while (1)
        {
            *msg = BYTE_SWAP(*msg);
            /* end of string? */
            if (!(*msg & 0xff) || !(*msg & 0xff00) || 
                !(*msg & 0xff0000) || !(*msg & 0xff000000) || 
                ((u32) msg - (u32) buff.db_data_out >= CBUFSIZE))
                break;
            msg++;
        }
        win_setTextColor(attrib);
        printf((s08 *) buff.db_data_out,buff.db_para1,buff.db_para2);
        win_setTextColor(GRAY);
    }
}  
 /*****************************************************************************************
 *          util_ReadMacAddrFile
 *****************************************************************************************/
int util_ProcessMacAddr(s08 *buff)
{
    s08 *tempStr, *cmd = buff;
    
    /* remove extra cr. Make sure is NULL terminated */
    tempStr = buff + strlen(buff) - 1;
    if ((*tempStr=='\n' )||(*tempStr=='\r'))
        *tempStr = '\0';

    /* remove extra trailing spaces */
    for (tempStr = buff + strlen(buff) - 1; (tempStr >= buff) && *tempStr==' '; tempStr--)
                *tempStr = '\0';

    /* skip spaces */
    for (cmd=buff;*cmd==' ';cmd++);

    /* if comment line, then ignore */
    if (*cmd==';' || *cmd=='\r' || !*cmd ||
           *cmd=='\n' || (cmd[0]=='/' && cmd[1]=='/'))
        return NO_ERROR;
    
    /* find '=' */
    for (buff = cmd; *buff && *buff!='='; buff++);
    
    if (!*buff)
    {
        printf("syntax error: <cmd> = <argument>\n");
        return HAS_ERROR;
    }

    for (*buff++ = '\0';*buff==' ';buff++);
    if (!*buff)
    {
        eprintf("argument is missing\n");
        return HAS_ERROR;
    }

    /* get command token */
    for (tempStr = cmd; *tempStr; tempStr++)
        if (!(isalpha(*tempStr) || isdigit(*tempStr) || *tempStr=='_'))
        {
            *tempStr = '\0';
            break;
        }
    
    if (!strcmpi("mac_addr_pref", cmd)) 
        strncpy(&global.mac_address[0], buff, 6);
    else if (!strcmpi("mac_addr_start",cmd)) 
        strncpy(&global.mac_address[6], buff, 6);
    else if (!strcmpi("mac_addr_end", cmd)) 
        strncpy(&global.mac_address_end[0], buff, 6);
    
    return NO_ERROR;
}     
/********************************************************************************
 *          util_ReadMacAddrFile
 *          return:     Nothing
 *          argument:   0 for read 
 *                      1 for write
 ********************************************************************************/
int util_AccessMacAddrFile(u08 AccessMode)
{
    FILE *file;
    s08 buff[256];
    s08 temp[6];
    u32 line=0;
    
    if (util_fileExists(global.MacAddrFileName))
    {
        if(AccessMode)
        {
            file = fopen(global.MacAddrFileName, "w");
            strncpy(temp, global.mac_address, 6);
            fprintf(file, "mac_addr_pref    =   %06X\n", strtoul(temp, NULL, 16));
            fprintf(file, "mac_addr_start   =   %06X\n", strtoul(&global.mac_address[6], NULL, 16)+1);
            fprintf(file, "mac_addr_end     =   %06X\n", strtoul(global.mac_address_end, NULL, 16));
        }
        else
        {    
            file = fopen(global.MacAddrFileName, "r");
            while (!feof(file))
            {
                if (fgets(buff,256,file))
                {
                    line++;
                    if (util_ProcessMacAddr(buff)==HAS_ERROR)
                    {
                        eprintf("Error detected at line %d\n", line);
                        fclose(file);
                        return HAS_ERROR;
                    }
                }
            }
        }
        fclose(file);
        
        if(strtoul(&global.mac_address[6], NULL, 16) > strtoul(global.mac_address_end, NULL, 16))
            return test_failed0(ERR_MAC_ADDR_ENDED);
        else if((strtoul(&global.mac_address[6], NULL, 16)==0)||(strtoul(global.mac_address_end, NULL, 16)==0))
            return test_failed0(ERR_INVALID_MAC_ADDR);
    }    
    else
        return test_failed1(ERR_FILE_OPEN, global.MacAddrFileName);
    
    return NO_ERROR;
}


/***********************************************************************/

/*****************************************/
static void UTIL_devPrompt(pWin w)
{
    pDev devPtr = (pDev) w->myStruct;
    pScreen s = &w->scrn;
    int editSize;
    int r,c;

    r = w->scrn.cursor.r;
    c = w->scrn.cursor.c;
    win_setTextColor(GRAY);
    scrn_writeCh(s,'>',0);
    win_partialRefresh(w, r, c, 1);
    editSize = s->csize - s->cursor.c;
    if (editSize > MAX_LINE_LENGTH)
        editSize = MAX_LINE_LENGTH;
    win_editCmdInit(w, &devPtr->cli, s->cursor.r, s->cursor.c, editSize);
}

/*****************************************/
static WIN_MSG dev_handler(pWin w, win_eType type, int event)
{
    pDev devPtr = (pDev) w->myStruct;
    
    switch (type) {
    case ePOLL:
        break;
        
    case eKEY:
        if ((win_editCmdInkey(w, &devPtr->cli, event)) == EDIT_GOOD)
        {
//            showWinNames();
            if (cmd_line_interface(devPtr->cli.buff)==CMD_QUIT)
                win_shutDown();
            else    
                UTIL_devPrompt(w);
        }
        break;
         
    case eSHUTDOWN:
        free(w->myStruct);
        break;
    }
    return WIN_NO_ACTION;
}
 
/***********************************************************************/
static int UTIL_get_pci_info(int bus,int device, int func,pPCI_DEVICE_INFO pci_info)
{
  u32 data32;
  u16 data16;

  if (read_configuration_word(bus,((device << 3) | func),0,&data16) != 0)
    {
      return DIAG_FAILURE;
    }

  pci_info->vendor_id = data16;
  if (read_configuration_word(bus,((device << 3) | func),0x2,&data16) != 0)
    {
      return DIAG_FAILURE;
    }

  pci_info->device_id = data16;
  if (read_configuration_dword(bus,((device << 3) | func),0x8,&data32) != 0)
    {
      return DIAG_FAILURE;
    }
  pci_info->class = (data32 >> 8);
  pci_info->revision = (u08)(data32 & 0xff);

  if (read_configuration_dword(bus,((device << 3) | func),0x10,&data32) != 0)
    {
      return DIAG_FAILURE;
    }
  pci_info->base_addr_l = data32;

  if (read_configuration_dword(bus,((device << 3) | func),0x14,&data32) != 0)
    {
      return DIAG_FAILURE;
    }
  pci_info->base_addr_h = data32;

  if (read_configuration_dword(bus,((device << 3) | func),0x3c,&data32) != 0)
    {
      return DIAG_FAILURE;
    }

  pci_info->irq = (u08)(data32 & 0xff);

  return DIAG_SUCCESS;
}

/***********************************************************************
 *
 * Tests to see if tigon card of logical number 'i' is present, and if
 * so, fills in the card_t structure values for the card in 'tigon'.
 *
 * returns TRUE = found card number 'i'
 *         FALSE = not found
 */
static bool UTIL_config_card(card_t *tigon)
{
    u32 value;

    /* Get ASIC version Number */
    value = RegRd(REGOFFSET(sbconfig.sbidhigh));
	
    tigon->ASIC_version = value  & 0xf;
  
    tigon->base_address_offset = PCI_CS_BASE_ADDRESS_0;
    read_configuration_dword(tigon->bus_number,
                           tigon->device_and_function,
                           tigon->base_address_offset,
                           &tigon->base_address);
	//printf("base addr %8x\n", tigon->base_address);
    tigon->base_address &= 0xfffffff0;
  
    read_configuration_dword(tigon->bus_number,
                           tigon->device_and_function,
                           PCI_CS_INTERRUPT_LINE,
                           &value);
	//printf("interrupt_line %8x\n", value);
    tigon->interrupt_line = value & 0xff;
	//util_install_default_handler();

    takeoverIntr(tigon->interrupt_line);
  
    read_configuration_dword(tigon->bus_number,
                           tigon->device_and_function,
                           PCI_CS_COMMAND,&value);
	write_configuration_dword(tigon->bus_number,
                           tigon->device_and_function,
                           PCI_CS_COMMAND,value | 4 | 2);
	tigon->cmd_stat = value | 6;  // save for possible restoration
	
	//printf("COMMAND %8x\n",value |4|2);
	// get various other parameters we might need for a restore later
    if (read_configuration_dword (tigon->bus_number,
                                tigon->device_and_function,
                                PCI_CS_CACHE_LINE_SIZE,
                                &value) == DIAG_SUCCESS)
    {
        tigon->bist_lat = value;
    }
	read_configuration_dword(tigon->bus_number,
			  tigon->device_and_function,
			  PCI_CS_VENDOR_ID,
			  &value);
    tigon->vendor_id = value & 0xffff;
    tigon->device_id = (value >> 16) & 0xffff;
	reset();
	
	

	
    return TRUE;
}

            
/***********************************************************************/


/***********************************************************************/
static void UTIL_initDev(card_t * dev)
{
    dev->phy_device = 1;
    
    UTIL_config_card(dev);
    
    {

    	global.device_detected++; 
        
    	if (!global.noinit)
        	util_hw_init(1);
  		global.current_card++;
        
	}
}
 
/***********************************************************************/
static void UTIL_find_dev(u16 vendor, u16 device, s08 * title)
{
    int i;
    card_t * dev;

    for (i=0;i<MAX_DEVICES;i++)
    {
        dev = &CURRCARD;
	    if (find_pci_device (device, vendor, i,&(dev->bus_number),
				    &(dev->device_and_function)) != DIAG_SUCCESS)
            return;  /* should return instead of continue */
        if (!(dev->device_and_function & 0x7))
		{
			dev->dev_str = title;
			UTIL_initDev(dev);
		}
    }
}    



/***********************************************************************/
static void UTIL_vendor_devid_to_str(u16 vendor, u16 device_id, s08 *string)
{

  switch (vendor)
    {
    case BROADCOM_VENDOR_ID:
      strcpy(string,"Broadcom 440x ");
      break;

    default:
      strcpy(string,"Unknown Vendor ");
      break;
    }


}


/***********************************************************************/
static u32 UTIL_getClockScale()
{
#define NUM_CLOCK_SAMPLE   5
  double prev;
  double now;
  u32 temp;
  u32 sample[NUM_CLOCK_SAMPLE];
  u32 total;
  int i;

  for (i = 0; i < NUM_CLOCK_SAMPLE; i++)
    {
      temp = clock();
      prev = util_get_clock();  
      delay(500);
      now = util_get_clock();  
      sample[i] = (u32)((now - prev)/500000.0);
    }
  
  total = 0;
  for (i = 0; i < NUM_CLOCK_SAMPLE; i++)
    total += sample[i];

  return (total/NUM_CLOCK_SAMPLE);
}

/***********************************************************************/
/*
 * reads tigon sram space via moveable sram window
 */
u32 util_read_direct_mem(u32 addr)
{
    return *((u32 *)addr);
}

/***********************************************************************/
u08 util_read_direct_mem_byte(u32 addr)
{
    return *((u08 *)addr);
}

/***********************************************************************/
u16 util_read_direct_mem_short(u32 addr)
{
    return *((u16 *)addr);
}

/***********************************************************************/
static void UTIL_write_direct_mem(u32 addr, u32 data)
{
  *((u32 *)addr) = data;
}

/***********************************************************************/
static void UTIL_write_direct_mem_byte(u32 addr, u08 data)
{
  *((u08 *)addr) = data;
}

/***********************************************************************/
static void UTIL_write_direct_mem_short(u32 addr, u16 data)
{
  *((u16 *)addr) = data;
}

    
/***********************************************************************/

/***********************************************************************/
/*********** public functions  *****************************************/
/***********************************************************************/
/***********************************************************************/
static void get_seeprom_size()
{
	CURRCARD.eeprom_size = 128;
	
}


/***********************************************************************/
int util_isDigit(s08 c)
{
    return global.radix==16 ? isxdigit(c) : isdigit(c);
}

/***********************************************************************/
int util_openLog(s08 * logFileName)
{    
	FILE *tmpfile;
	time_t t;

	if (!(tmpfile = fopen(logFileName, "w+")))
	{
        eprintf("Cannot open log file, '%s'!\n", logFileName);
        return 1;
	}
	else
	{
        strcpy(global.logFileName,logFileName);    
	    printf("started logfile %s \n", logFileName);
	    global.logfile = tmpfile;   /* so that previous line does NOT get send to logfile */
	}
	t = time(NULL);
	fprintf (global.logfile, "Log file created by %s %s\n",APP_NAME, versionStr);
	fprintf (global.logfile, "on: %s", asctime(localtime(&t)));
	fprintf (global.logfile, "---------------------------------------------\n");
	return 0;
} 


/***********************************************************************/
int util_openBin(s08 * binFileName)
{    
	FILE *tmpfile;
	time_t t;

	if (!(tmpfile = fopen(binFileName, "w+")))
	{
        eprintf("Cannot open eeprom file %s \n",  global.binFileName);
        return 1;
	}
	else
	{
        strcpy(global.binFileName,binFileName);    
	    printf("started eeprom file %s \n",  global.binFileName);
	}
	t = time(NULL);
	fprintf (global.logfile, "eeprom file created by %s %s\n",APP_NAME, versionStr);
	fprintf (global.logfile, "on: %s", asctime(localtime(&t)));
	fprintf (global.logfile, "---------------------------------------------\n");
	return 0;
} 


/***********************************************************************/
int util_fileExists(s08 * logFileName)
{    
	FILE *tmpfile;

	if ((tmpfile = fopen(logFileName, "r")))
	{
        fclose(tmpfile);
        return 1;
	}
    return 0;
}

/***********************************************************************/
#ifndef IKOS

/***********************************************************************/

#endif


/***********************************************************************/

/***********************************************************************/
int util_resetPhy()
{         
	/* some chips have internal phy, some don't */
	if (!(RegRd( REGOFFSET(devcontrol)) & DC_IP)) {
		RegWr( REGOFFSET(enetcontrol), EC_EP);
	} else if (RegRd( REGOFFSET(devcontrol)) & DC_ER) {
		RegAnd( REGOFFSET(devcontrol), ~DC_ER);
		MM_Wait(10000);
	}

	/* XXX: hard-coded 4402/4401 value */
	RegWr( REGOFFSET(mdiocontrol), 0x8d);

    miireset();
	return NO_ERROR;

}



/***********************************************************************/
void util_intr_info(void)
{
  printf("Interrupt Count      : %d\n",CURRCARD.intr_count);
#ifndef EFI
  printf("IPC MASK             : 0x%02x 0x%02x\n",inp(0x21),inp(0xa1));
  outp(0x20,0xb);
  outp(0xa0,0xb);
  printf("IPC IS1 IS2          : 0x%02x 0x%02x\n",inp(0x20),inp(0xa0));
  outp(0x20,0xa);
  outp(0xa0,0xa);
  printf("IPC IRR1 IRR2        : 0x%02x 0x%02x\n",inp(0x20),inp(0xa0));
  printf("IPC ILCR1 ILCR2      : 0x%02x 0x%02x\n",inp(0x4d0),inp(0x4d1));
#endif
  if (global.clk_scale != 0)
    {
      printf("Worst Intr. Latency  : %.0lf CPU clocks/%.0lf uS\n",
             global.worst_interrupt_latency,global.worst_interrupt_latency/global.clk_scale);
    }
}

/***********************************************************************/
void util_pci_cfg_write_word(u32 address,u32 data,int *failed)
{
    card_t *card = get_currcard();

    *failed = 0;
    if (card!=NULL && !write_configuration_dword(card->bus_number,
                                card->device_and_function,
                                address,
                                data))
        return;
                                     
    *failed = 2;
    printf("write configuration dword error\n");
}

/***********************************************************************/


/***********************************************************************/
void util_pci_cfg_write_short(u32 address,u16 data,int *failed)
{
    card_t *card;

    card = get_currcard();
    *failed = 0;
    if (card !=NULL && !write_configuration_word(card->bus_number,
                               card->device_and_function,
                               address,
                               data))
        return;                           
    *failed = 2;
}


/***********************************************************************/

/***********************************************************************/
/***********************************************************************/
int util_user_abort()
{
    int c;
    
    if (!user_break())
    {
        if (!util_kbhit())
            return 0;
        
        c = util_getch();
        if (!((c == KEY_CTL_C && global.manuf) || (c==KEY_ESC && !global.manuf)))
            return 0;
    }
    if (c==KEY_ESC)
        printf("\n");
    return 1;
}

int util_wait_ms(int milliseconds)
{
#ifdef EFI
    delay(milliseconds);
#else
    u32 now;

    now = clock();

    while (clock() < (now+milliseconds)) 
    {
        if (util_user_abort())
            return 1;
        
    }
    return 0;
#endif
}

/***********************************************************************
 *
 * writes values to tigon sram space
 */
void util_write_sram (u32 addr, u32 data)
{
#ifdef EFI
    int efi = 1;
#else
    int efi = 0;
#endif

    if (efi || NOCFG())
    {
        t_writemem32(T3_SRAM_WINB_REG,addr);
        addr = t_readmem32(T3_SRAM_WINB_REG);
        t_writemem32(T3_SRAM_DATA_REG,data);
    }    
    else
    {
        write_configuration_dword(CURRCARD.bus_number,
                              CURRCARD.device_and_function,
                              T3_SRAM_WINB_REG,addr);
        write_configuration_dword(CURRCARD.bus_number,
                              CURRCARD.device_and_function,
                              T3_SRAM_DATA_REG,data); 
    }                        
}

/***********************************************************************/
/*
 * writes values to tigon sram space
 */
void util_write_isram (u32 addr, u32 data)
{
#ifdef EFI
    int efi = 1;
#else
    int efi = 0;
#endif

    if (efi || NOCFG())
    {
        t_writemem32(T3_REG_BASE_ADDR,addr);
        addr = t_readmem32(T3_REG_BASE_ADDR);
        t_writemem32(T3_REG_DATA_ADDR,data);
    }
    else
    {
        write_configuration_dword(CURRCARD.bus_number,
                              CURRCARD.device_and_function,
                              T3_REG_BASE_ADDR,addr);
        write_configuration_dword(CURRCARD.bus_number,
                              CURRCARD.device_and_function,
                              T3_REG_DATA_ADDR,data);
    }                          
}


/***********************************************************************/
int util_pause(s08 * msg)
{
    s08 c;
    
    s08 * defmsg = "Press any key to continue...";
    
    if (msg==NULL)
        msg = defmsg;
    printf("%s",msg);
    c = win_getkey(1);
    printf("\n");
    return c;
}

/***********************************************************************/
int util_pcicfg_init(void)
{
  u16 vendor_id;
  u16 PciSpeed;
  u16 device_id;
  u32 value;
  s08 string[80];
  card_t *card;
  

  card = get_currcard();

  read_configuration_word(card->bus_number,
                          card->device_and_function,
                          PCI_CS_VENDOR_ID,
                          &vendor_id);
  read_configuration_word(card->bus_number,
                          card->device_and_function,
                          PCI_CS_DEVICE_ID,
                          &device_id);
  read_configuration_dword(card->bus_number,
                          card->device_and_function,
                          PCI_CS_STATUS,
                          &value);
  
	if (value & 0x2) 
	{
		PciSpeed = 33;
	}
	else
	{
		PciSpeed = 66;
	}
  if ((vendor_id == 0xffff) || (device_id == 0xffff))
    {
      printf("No NIC is detected on this slot!\n");
      return DIAG_FAILURE;
    }

  card->loadd_count = 0;
  printf("Initializing PCI Configuration Space\n");
  UTIL_vendor_devid_to_str(vendor_id,device_id,string);
  printf("%s NIC is detected\n",string);
  printf("Bus Number       : %d\n",card->bus_number);
  printf("Device/Funtion   : %d/%d\n",(card->device_and_function >> 3) & 0x1f,
         (card->device_and_function  & 0x7));
  printf("Base Address     : 0x%08x\n",card->base_address);
  printf("IRQ              : %d\n",card->interrupt_line);

  write_configuration_dword(card->bus_number,
                            card->device_and_function,
                            card->base_address_offset,
                            card->base_address);
  write_configuration_dword(card->bus_number,
                            card->device_and_function,
                            card->base_address_offset+4,
                            0);

  /* Enable Memory space and bus master */
  write_configuration_word(card->bus_number,
                           card->device_and_function,
                           PCI_CS_COMMAND,
                           0x6);
  write_configuration_byte(card->bus_number,
                           card->device_and_function,
                           PCI_CS_INTERRUPT_LINE ,
                           card->interrupt_line);

  //UTIL_get_bus_type(&bus_width,&bus_speed,&bus_type);
  printf("Bus Type         : %s\n","Conventional PCI");
  printf("Bus Speed        : %d MHz\n",PciSpeed);
  
  CURRCARD.dma_workaround_done = 0;
  CURRCARD.eeprom_size = 0;
  util_hw_init(1);

  return DIAG_SUCCESS;
}

/***********************************************************************/
int util_pci_scan(void)
{
  PCI_DEVICE_INFO pci_info;
  int bus;
  int device;
  int func;
  u16 data;
  u16 device_id;

  printf("Scanning PCI devices ... \n");
  /*12345678901234567890123456789012345678901234567890123456789012345678901234567890*/
  printf("Bus Dev Func  Vendor ID Device ID    Class     Base/IO Address    IRQ\n");
  printf("=== === ====  ========= =========  ==========  =================  ===\n");

  for (bus = 0; bus < 256; bus++)
    {
      for (device = 0; device < 32; device++)
        {
          device_id = 0;
          for (func = 0; func < 8; func ++)
            {
              if (read_configuration_word(bus,
                                          ((device << 3) | func),
                                          0x0,                                    
                                          &data) == 0)
                {
                  if (data == 0xffff)
                    continue;

                  
                  if (UTIL_get_pci_info(bus,device,func,&pci_info) != 0)     
                    continue;
                  
                  if (device_id != 0)
                    {
                      if (pci_info.device_id == device_id)
                        continue;
                    }

                  device_id = pci_info.device_id;
                  printf("%2d%4d%4d",bus,device,func);
                  printf("       %04X      %04X     %02X:%02X:%02X   %08X:%08X  %d\n",
                         pci_info.vendor_id,
                         pci_info.device_id,
                         (pci_info.class >> 16)& 0xff,
                         (pci_info.class >> 8) & 0xff,
                         pci_info.class & 0xff,
                         pci_info.base_addr_h,
                         pci_info.base_addr_l,
                         pci_info.irq);
                }
            }
        }
    }

  return DIAG_SUCCESS;
}


/***********************************************************************/
int util_DriverLoaded()
{
    return UM_GetNIC()->DeviceInst != NULL;            
}

/***********************************************************************/
int util_CheckDriverLoaded()
{
    dprintf("### util_CheckDriverLoaded\n");
    if (util_DriverLoaded())
        return 1;
        
    Vprintf(!global.Rs232, "Dev %d:Driver is not loaded\n",global.current_card);
    return 0;
}


int util_hw_init(int pci_init)
{
    card_t *card;
    u32 value;
    static in_hw_init = 0;

#ifdef EFI
    pNIC_HW_CONFIG p_hwcfg;
#else
    //NIC_SHARED_DATA nic_data;
#endif

    /* to prevent recursive, return if this was called from hw_init */
    if (in_hw_init)
        return NO_ERROR; 
        
    in_hw_init = 1; /* indicate we are in hw_init contest */

    card = get_currcard();
    
    /* restore cache_line and letency timer value */
    if (read_configuration_dword (card->bus_number,
                               card->device_and_function,
                               PCI_CS_CACHE_LINE_SIZE,
                                &value) == DIAG_SUCCESS)
	
	write_configuration_dword (card->bus_number,
                               card->device_and_function,
                               PCI_CS_CACHE_LINE_SIZE,
                               card->bist_lat);
    

    /* Get current ASIC revision */
    read_configuration_dword(card->bus_number,
                           card->device_and_function,
                           REGOFFSET(sbconfig.sbidhigh),
                           &value);
    card->ASIC_version = value  & 0xF;  

    read_configuration_dword(card->bus_number,
			  card->device_and_function,
			  PCI_CS_VENDOR_ID,
			  &value);
    card->vendor_id = value & 0xffff;
    card->device_id = (value >> 16) & 0xffff;
	

      /* get commmand register */
    read_configuration_dword(card->bus_number,
                           card->device_and_function,
                           PCI_CS_COMMAND,
                           &value);
    write_configuration_dword(card->bus_number,
                            card->device_and_function,
                            PCI_CS_COMMAND,value | 6);
    

  
//    value = t_readmem32(OFFSETOF(T3_STD_MEM_MAP,PciCfg.ClockCtrl)) ;
    /* always enable RX/TX CPU clock as default */

    read_configuration_dword(card->bus_number,
							card->device_and_function,
							PCI_CS_SUBSYSTEMVENDORID,
							&value);
    card->subsystem_vendor_id = value & 0xffff;
    card->subsystem_device_id =(value >> 16) & 0xffff;


        card->external_memory_size = 0;

    if (!CURRCARD.eeprom_size)
        get_seeprom_size();
    
    in_hw_init = 0; /* indicate we are exiting hw_init contest */

    return NO_ERROR;
}

/***********************************************************************/


/***********************************************************************/
double util_get_clock()
{
  u32 low_clk=0;    /* assign value here to avoid compiter warning */
  u32 high_clk=0;   /* assign value here to avoid compiter warning */
  double data;

#ifdef EFI
  data = efi_get_time();
#else
  _asm 
    {
      .586
      rdtsc
      mov low_clk,eax
      mov high_clk,edx
    };

  data = (double)high_clk;
  data = data * 4294967296;
  data += (double)low_clk;
#endif


  return data;
}

/***********************************************************************/
void util_wait_1us(int wait)
{
#ifdef EFI
    clock_t current;
    int period;

    period = (wait/1000);
    if (period == 0)
        period = 1;
    delay(period);
#else
    double current;
    double period;

    period = MICRO_SECONDS(wait);
    current = util_get_clock();  
    while (util_get_clock() < (current + period));
#endif
}

/***********************************************************************/
void util_printTabularByte(u32 beg, u32 end, u08 (*func)(u32))
{
  u32 i;
  u08 j;
        
  if (beg & 0xf)                           // not nicely aligned
    {
      if (end > ((beg & ~0xf) + 0xf))    // will go more than 1 line
        {
          printf("%08x: ", beg & ~0xf);
          for (j=(beg&0xf); j; --j)
            printf ("   ");
        }
      else
        printf("%08x: ", beg);
    }
  
  for (i=beg; i<=end; i++)
    {
      if (!(i&0xf))
        printf("%08x: ", i);

      printf("%02x ", func(i));
      if ((i &0xf) == 0xf)
        {
          printf ("\n");
          if (util_user_abort ())
            break;
        }
    }

  if ((end&0xf) != 0xf)
    printf ("\n");
}

/***********************************************************************/
void util_printTabularShort(u32 beg, u32 end, u16 (*func)(u32))
{
  u32 i;
  u08 j;
        
  if (beg & 0xf)                           // not nicely aligned
    {
      if (end > ((beg & ~0xf) + 0xf))    // will go more than 1 line
        {
          printf("%08x: ", beg & ~0xf);
          for (j=(beg&0xf); j; --j)
            printf ("     ");
        }
      else
        printf("%08x: ", beg);
    }
  
  for (i=beg; i<=end; i+=2)
    {
      if (!(i&0xf))
        printf("%08x: ", i);

      printf("%04x ", func(i));
      if ((i &0xf) == 0xe)
        {
          printf ("\n");
          if (util_user_abort ())
            break;
        }
    }

  if ((end&0xf) != 0xf)
    printf ("\n");
}

/***********************************************************************/
/*
 *  Prints memory in tabular format.  Printing rules are easier to show by
 *  example :-).  Assume memory looks like the following:
 *
 * 0: 00112233 44556677 8899aabb ccddeeff 01234567 89abcdef feedface deadbeef
 *40: 33221100 77665544 bbaa9988 ffeeddcc 76543210 fedcba98 facefeed beefdead
 *      ...
 *
 *  print 0 4 results in:
 * 0: 00112233
 *
 *  print 8 0x10 results in:
 * 0: 8899aabb ccddeeff 01234567 89abcdef
 *
 *  print 4 0x3c results in:
 * 0:          44556677 8899aabb ccddeeff 01234567 89abcdef feedface deadbeef
 *40: 33221100 77665544 bbaa9988 ffeeddcc 76543210 fedcba98 facefeed beefdead
 *
 *  print 0x10 0x14 results in:
 * 0:                                     01234567 89abcdef feedface deadbeef
 *40: 33221100 
 *
 *  Function to read memory by words is passed in.
 */
void util_print_tabular (u32 beg, u32 end, u32 (*func)(u32))
{
  u32 i;
  u08 j;
        
  if (beg&0x1f)                           // not nicely aligned
    {
      if (end > ((beg & ~0x1f) + 0x1c))    // will go more than 1 line
        {
          printf("%06lx:", beg&~0x1f);
          for (j=(beg&0x1f) >> 2; j; --j)
            printf ("         ");
        }
      else
        printf("%06lx:", beg);
      if (util_user_abort ())
            return;
    }

  for (i=beg; i<=end; i+=4)
    {
      if (!(i&0x1f))
        printf("%06lx:", i);
      printf(" %08lx", func(i));
        
      if ((i&0x1c) == 0x1c)
        {
          printf ("\n");
        }
      if (util_user_abort ())
            return;
    }

  if ((end&0x1c) != 0x1c)
    printf ("\n");
}

#ifdef OLD
/***********************************************************************/
/*
 * reads values from tigon register space
 */
u32 util_read_reg (u32 addr)
{
        addr = addr % 0x1000;
        return t_readmem32 ((u16)addr);
}
#endif

/***********************************************************************/
/*
 * prints values from tigon register space in nice, readable format, with
 * register names printed out wherever possible
 */
u32 util_read_reg(u32 addr)
{
  return t_readmem32(addr);
}

/***********************************************************************/
void util_fill_miiregs(u32 beg, u32 end, u16 data)
{
  int i;

  if (beg > 0x1f)
    beg = 0x1f;

  if (end > 0x1f)
    end = 0x1f;

  for (i=beg; i<=end; i++)
    {
      miiwr((UCHAR)i, data);
      if (util_user_abort())
        return;
    }
}

/***********************************************************************/
void util_fill_conf (u32 beg, u32 end, u32 data, int type)
{
  u32 i;
        
  switch (type)
    {   
    case 4:
      for (i=beg; i<=end; i+=4) 
    	{
    	  t_writeconf32 ((u16)i,data);
    	  return;
    	}
      break;

    case 2:
      for (i = beg; i <= end; i += 2)
        {
          t_writeconf16(i,(u16)data);
        }
      break;

    case 1:
      for (i = beg; i <= end; i++)
        {
          t_writeconf08(i,(u08)data);
        }
      break;
      
    default:
      break;
    }
}

/***********************************************************************/
u32 util_read_sram (u32 addr)
{
    u32 data;
#ifdef EFI           
    int efi = 1;
#else
    int efi = 0;
#endif
    
    if (efi || NOCFG())
    {
        t_writemem32(T3_SRAM_WINB_REG,addr);
        addr = t_readmem32(T3_SRAM_WINB_REG);
        data = t_readmem32(T3_SRAM_DATA_REG);
    }    
    else
    {
        write_configuration_dword(CURRCARD.bus_number,
                              CURRCARD.device_and_function,
                              T3_SRAM_WINB_REG,addr);
    	read_configuration_dword(CURRCARD.bus_number,
                             CURRCARD.device_and_function,
                             T3_SRAM_DATA_REG,&data); 
    }                         
    return data;
}

/***********************************************************************/
u32 util_read_sram_blk(u32 addr, u32 *buffer,int num_word)
{
  int i;

  if (buffer == NULL)
    return 1;

  for (i = 0; i < num_word; i++)
    {
      *buffer++ = util_read_sram(addr);
      addr += 4;
    }

  return 0;
}





/***********************************************************************
 *
 * reads tigon internal sram space
 */
u32 util_read_isram (u32 addr)
{
    u32 data;
#ifdef EFI
    int efi = 1;
#else
    int efi = 0;
#endif

    if (efi || NOCFG())
    {
        t_writemem32(T3_REG_BASE_ADDR,addr);
        addr = t_readmem32(T3_REG_BASE_ADDR);
        data = t_readmem32(T3_REG_DATA_ADDR);
    }    
    else
    {
      	write_configuration_dword(CURRCARD.bus_number,
                              CURRCARD.device_and_function,
                              T3_REG_BASE_ADDR,addr);
      	read_configuration_dword(CURRCARD.bus_number,
  	                         CURRCARD.device_and_function,
  	                         T3_REG_DATA_ADDR,&data);
    }                         
  	return data;
}

/***********************************************************************/
u32 util_read_isram_blk(u32 addr, u32 *buffer,int num_word)
{
  int i;

  if (buffer == NULL)
    return 1;

  for (i = 0; i < num_word; i++)
    {
      *buffer++ = util_read_isram(addr);
      addr += 4;
    }

  return 0;
}


/***********************************************************************/
/*
 * writes values to tigon register space
 */
void util_write_reg (u32 addr, u32 data)
{
  t_writemem32 ((u16)addr, data);
}

/***********************************************************************/
/*
 * writes 32-bit data pattern to region of tigon register space
 */
void util_fill_reg (u32 beg, u32 end, u32 data)
{
  u32 i;
  
  for (i=beg; i<=end; i+=4) 
    {
      util_write_reg (i, data);
      if (util_user_abort())
        return;
    }
}

/***********************************************************************/
void util_show_device(int i)
{
    u32 value;
	//u32 temp;
    u08 mac[8];
    u08 org_card = global.current_card;
    //u32 unit;
	u16 PciSpeed;
	
    util_switch_card(i);
	read_configuration_dword(global.t_cards[i].bus_number,
                           global.t_cards[i].device_and_function,
                           PCI_CS_STATUS,
                           &value);
	if (value & 0x2) 
	{
		PciSpeed = 33;
	}
	else
	{
		PciSpeed = 66;
	}
   
    
    //("Dev  Brd:Rv   Bus         Spd BaseAddr Irq  EEP MAC          Fmw PXE WOL PXESpd\n");
    //("--- ------- ------------- --- -------- --- ---- ------------ --- --- --- ------\n");
    //x B1   xx:xx     PCIX-64    66  xxxxxxxx xx  64k  xxxxxxxxxxxx 1.3 yes no  auto        */
    
    win_setTextColor(global.current_card==org_card ? WHITE : GRAY);
//    printf("bus:%02X dev:%02X function:%02X\n",global.t_cards[i].bus_number,global.t_cards[i].device_and_function>>3, global.t_cards[i].device_and_function & 7 );
	printf ("%d %s %02X:%02X:%d PCI  %3d %08X %2d  %3d",
			i,
            global.t_cards[i].dev_str,
            //ChipRevision(),
			//MetalRevision(),
			global.t_cards[i].bus_number,
			global.t_cards[i].device_and_function >>3,
			global.t_cards[i].device_and_function & 0x7,
			PciSpeed,
			global.t_cards[i].base_address,
			global.t_cards[i].interrupt_line,
			global.t_cards[i].eeprom_size);

			eprom_read_blk(76,(u32 *)&mac,2);
			printf(" %02X:%02X:%02X:%02X:%02X:%02X\n",
                   mac[3],mac[2],mac[5],mac[4],mac[7],mac[6]);

            win_setTextColor(GRAY);

    win_setTextColor(GRAY);
    util_switch_card(org_card);
}

/***********************************************************************/
#define SEL_DEV_X1 5
#define SEL_DEV_Y1 2
#define SEL_DEV_W  49
void util_select_devices()
{
    u32 i=0;
    
    printf("  Brd: Bus          Spd BaseAddr Irq SRM MAC          \n");
    printf("  ---- ------------ --- -------- --- --- ------------ \n");
/*             x    xxxxxxxxxx X 128 xxxxxxxxxxxx no        */
    while (i<global.device_detected)
      	util_show_device(i++);
}
    


/***********************************************************************/
int util_switch_card(int dev)
{
#ifndef WIN_CLI
    s08 buff[10];
#endif 

	if (dev >= global.device_detected)
	{                                                                         
		printf("Invalid device %d requested",dev);
        if (global.device_detected>1)
            printf(", valid device number is 0-%d\n",global.device_detected-1);
        else
            printf("\n");
		return 0;
	}

    if (dev != global.current_card)
    {
        global.current_card = dev;
    }
#ifndef WIN_CLI
    sprintf(buff,"%d:>",dev);
    cmd_setPrompt(buff);
#endif 
    return 1;
}

/***********************************************************/

/***********************************************************
 * looks for PCI bus, tigon cards, and creates data        *
 * structures for each card (base address, etc.)           *
 ***********************************************************/
void util_init_pci_and_find_tigons (int start_card,int force_overwrite_vendor_id)
{
    //if (force_overwrite_vendor_id)
        //UTIL_overwrite_vendor_id();
        
  	global.current_card = global.device_detected = 0;
    
    UTIL_find_dev(BROADCOM_VENDOR_ID, BROADCOM_DEVID_4401, "4401");
	UTIL_find_dev(BROADCOM_VENDOR_ID, BROADCOM_DEVID_4402, "4402");
	UTIL_find_dev(BROADCOM_VENDOR_ID, BROADCOM_DEVID_FFFF, "4401");
	UTIL_find_dev(BROADCOM_VENDOR_ID, BROADCOM_DEVID_0000, "4401");
    
    //if (global.pciDev)
        //UTIL_find_dev(global.pciDev >> 16, global.pciDev & 0xffff, "440x");
    
  	if (!global.device_detected)
    {
        if (global.manuf)
        {
            scrn_dispMsg("No Broadcom 4401 card has been detected in the system");

            printf("\n\n\n");
            util_system_exit(-1);
        }
		else
        	return;
    }
    init_dev_interrupt();
    
    //if (global.ref)
        //findReferenceCard();
    
    global.current_card = 0; /* default to device 0 (in case start_card is invalid) */
    util_switch_card(start_card);
    
    if (!global.noinit)
        util_select_devices();
} 

/***********************************************************************/
void util_install_default_handler()
{
    //CURRCARD.isr_callback = default_handler;
}

/***********************************************************************/
/*
 * prints values from tigon serial eeprom space in nice, readable format.
 */
void util_print_miiregs(u32 beg, u32 end)
{
  int i,j;
  U16 data;

  if (beg > 0x1f)
    beg = 0x1f;

  if (end > 0x1f)
    end = 0x1f;

  if (beg & 0x7)                           // not nicely aligned
    {
      if (end > ((beg & ~0x7) + 0x7))    // will go more than 1 line
        {
          printf("%02x:", beg &~0x7);
          for (j = (beg&0x7); j; --j)
            printf ("     ");
        }
      else
        printf("%02x:", beg);
    }

  for (i=beg; i<=end; i++)
    {
      if (!(i & 0x7))
        printf("%02x:", i);
		data = miird((UCHAR) i);
      
      printf(" %04x", data);

      if ((i & 0x7) == 0x7)
        printf ("\n");
    }

  printf ("\n");
}

/***********************************************************************/
void util_fill_direct_mem(u32 beg, u32 end, u32 data,int type)
{
    u32 i;
  
    switch (type)
    {   
    case 4:
        for (i = beg; i <= end; i += 4)
        {
            UTIL_write_direct_mem(i,data);
        }
        break;

    case 2:
        for (i = beg; i <= end; i += 2)
        {
            UTIL_write_direct_mem_short(i,data);
        }
        break;

    case 1:
        for (i = beg; i <= end; i++)
        {
            UTIL_write_direct_mem_byte(i,data);
        }
        break;
      
    default:
        break;
    }
}



/*************************************************************
 * function util_print_banner
 * 
 * input: none
 * 
 * output: none
 *
 * return: none
 *
 * side effects: none
 *************************************************************/
void util_print_banner(void)
{
    win_clearscreen();
    printf("*****************************************************************\n");
    printf(" Copyright(c) 2002 Broadcom Corporation\n");
    printf(" %s %s (%s)\n", global.manuf ? MANU_NAME : APP_NAME, versionStr,dateStr);
    printf("*****************************************************************\n\n");
    _settextcolor (NORMAL_COLOR);
}

/***********************************************************************/
int util_system_exit(int exitCode)
{   
    
#ifdef EFI
    terminate_efi();
#endif
#ifndef WIN_CLI     
    intr_cleanup();
#endif    
    win_shutDown();
    if (!global.manuf)
        cleanup_history ();

    if (global.video_mode)
    {
    	_setvideomode (_DEFAULTMODE);
    	_settextcolor (GRAY);
    }
    scrn_showCursor();
#ifdef EFI
    /* EFI doesn't like the return value to be negative */
    if (exitCode < 0)
        exitCode *= -1;
#endif        
    exit(exitCode);
    return 0; // makes compiler happy
}

/***********************************************************************/
void util_system_init(void)
{
    intr_init();
    scrn_init();
    hist_init();
#ifndef WIN_CLI     
    init_history();
#endif    
    if (!global.manuf)
    {
        _clearscreen(_GCLEARSCREEN);
    }
    if (global.video_mode)
    {
        _setvideomoderows (_TEXTC80, 50);
//        _settextposition (0, 0);
//        _settextcolor (GRAY);
    }
    win_init();
#ifdef EFI
    if (init_efi() != EFI_SUCCESS) 
    {
        printf("ERROR: EFI Init failed\n");
        util_system_exit(0);
    }
#endif
}

/***********************************************************************/

/*************************************************************
 * function util_init_default_cfg 
 * Initialize test tx/rx fifos and other diagnostic environment
 * 
 * input: none
 * 
 * output: none
 *
 * return: none
 *
 * side effects: diagnostic environmnet
 *************************************************************/
void util_init_default_cfg(void)
{
    global.clk_scale = UTIL_getClockScale();

}

/*************************************************************
 * function util_printer_ready
 * 
 * input: none
 * 
 * output: none
 *
 * return: true if printer is ready to print
 *         false if user hit ESC to abort
 *
 *************************************************************/
int util_printer_ready()
{
    unsigned short status;
    u16 ch;

    while (1)
    {
#ifdef EFI
        scrn_dispMsg( "No printer support in EFI platform.");
        return FALSE;         /* No printer support on EFI yet */
#else
        status = _bios_printer( _PRINTER_STATUS, 0, 0 );
        if (status & 0x80)   // NEED TO FIND OUT WITCH BIT TO CHECK
            return TRUE;
        ch = scrn_dispWait( "Please turn on the printer, press any key when ready....", status );
        if (ch==KEY_ESC)
            return FALSE;
#endif
    }
}


/*************************************************************
 * function util_beep
 *      generate a beep sound
 * 
 * input: none
 * 
 * output: none
 *
 * return: beep
 *************************************************************/
void util_beep()
{
#ifndef EFI
    _asm {
        mov dl,7
        mov ah,2
        int 0x21
    }
#endif
}

/*************************************************************
 * device window function 
 * 
 *************************************************************/
int util_activateDevice(int dev)
{
    static int r = DEV_ROW;
    static int c = DEV_COL;
    int high = SCREEN.rsize - r + DEV_ROW;
    int width = SCREEN.csize - c + DEV_COL;
    pDev devPtr;
    pWin w;
    u16 devKey[MAX_DEVICES] = {KEY_ALT_0, KEY_ALT_1, KEY_ALT_2, KEY_ALT_3,
           KEY_ALT_4, KEY_ALT_5, KEY_ALT_6, KEY_ALT_7};

    if (dev<0 || dev >= global.device_detected)
        return 0;
    util_switch_card(dev);
    if (!win_ActivateKey(devKey[dev]))
    {
        if ((devPtr = (pDev) malloc(sizeof(devt)))==NULL)
            return 0;
        sprintf(devPtr->name,"DEV %d:%X:%02X:%s(%c%d),0x%08x,IRQ %d",
            dev,
            global.t_cards[dev].bus_number,
            global.t_cards[dev].device_and_function>>3,
            global.t_cards[dev].dev_str,
            'A'+T3_CHIP_REV(global.t_cards[dev].ASIC_version),
            T3_METAL_REV(global.t_cards[dev].ASIC_version),
            global.t_cards[dev].base_address,
            global.t_cards[dev].interrupt_line);
        w = win_new(devPtr, devKey[dev], dev_handler,
              devPtr->name, r++, c++, high--, width--, ATTRIB(YELLOW,BLUE));
        if (w==NULL)
        {
            free(devPtr);
            return 0;
        }
        UTIL_devPrompt(w);
    }
    return 1;
}

/*
 * util_readFileMemAlloc 
 *      open a file and read data into a buffer
 *
 * input: filename - input filename string
 *        offset   - offset to read
 *        len      - len to read, 0 = all
 *        limit    - size limit
 *
 * sideeffect:   
 *        it allocates buffer memory and return that value, therefore, the buffer
 *        MUST be freed after use
 * 
 * return: NO_ERROR or error_code
 */
int util_readFileMemAlloc(s08 * filename,int offset,u32 len,u32 limit,u32 * size, u08 **buffer, int verbose)
{
    int ifile;
    struct stat file_stats;
#ifdef EFI
    int i, buf;
#endif
    
    *buffer = NULL;
    if ((ifile = open(filename,O_RDONLY | O_BINARY)) == -1)
    {
        return test_failed1(ERR_FILE_OPEN,filename);
    }      

    /* Get input info */
    fstat(ifile,&file_stats);
    if (file_stats.st_size > limit)
    {
        close(ifile);
        return test_failed2(ERR_FILE_TOO_BIG,file_stats.st_size,limit);
    }
    
    if (verbose) 
        printf("\nFile Size   : %d\n",file_stats.st_size);
    if (!len)
       len = file_stats.st_size;
    if ((*buffer = (u08 *)malloc(file_stats.st_size))==NULL)
    {
        close(ifile);
        return test_failed0(ERR_MALLOC);
    }
    
#ifdef EFI     /* lseek() may not work in this case in EFI */
    for (i=0; i<offset; i++) read(ifile, &buf, 1);
#else
    lseek(ifile,offset,SEEK_SET);
#endif
    *size = read(ifile,*buffer,len);
    if (verbose)
    {
        printf("Offset      : 0x%04x\n",offset);
        printf("Bytes Read  : %d\n",*size);
    }
    close(ifile);
    return NO_ERROR;
}

/*
 * util_readFile
 *      open a file and read data into a buffer, the buffer must have enough size for 'len' to read
 *
 * input: filename - input filename string
 *        offset   - offset to read
 *        len      - len to read, 0 = all
 *        limit    - size limit
 *
 * output: size - the size read
 * 
 * return: error_code
 */
int util_readFile(s08 * filename, int offset, u32 len, u32 * size, u08 *buffer)
{
    int ifile;
#ifdef EFI
    int i, buf;
#endif
    
    if ((ifile = open(filename,O_RDONLY | O_BINARY)) == -1)
    {
        return test_failed1(ERR_FILE_OPEN,filename);
    }      

#ifdef EFI     /* lseek() may not work in this case in EFI */
    for (i=0; i<offset; i++) read(ifile, &buf, 1);
#else
    lseek(ifile,offset,SEEK_SET);
#endif
    *size = read(ifile,buffer,len);
    close(ifile);
    return NO_ERROR;
}

#define READSTR_ROW 3
util_readStr(s08 * msg, s08 * buffer, int len, int hexOnly, int spaceAllowed)
{
    ScreenPtr savedScreen;
    int width = strlen(msg);
    struct rccoord oldxy;
    s08 attrib;
    int boxCol,col;
    int ch;
    int pos=0,i;
    int firstKey = 1;
    int insertMode = 1;
    int editing = 1;

    /* get the max of (len, strlen(buffer)) for box width */
    if (width < len)
        width = len;
        
    /* add width with frame at both side & make sure it is within limit */
    width += 6; 
    if (width>(SCREEN.csize-2)) 
        width = SCREEN.csize-2;
    
    /* save screen info */    
    oldxy = _gettextposition();
    attrib = scrn_setAttrib(pSCREEN,ATTRIB(WHITE,BLUE));
    boxCol = (SCREEN.csize-width)/2;
    savedScreen = scrn_saveScreen(READSTR_ROW, boxCol, READSTR_ROW+3, boxCol+width);
    
    /* draw dialog */
    scrn_boxWithShadow(SINGLE_LINE, READSTR_ROW, boxCol, READSTR_ROW+2, boxCol+width-2);
    scrn_setAttrib(pSCREEN,ATTRIB(WHITE,RED));
    scrn_xyPrintf(pSCREEN, READSTR_ROW, (SCREEN.csize-strlen(msg))/2 - 1, " %s ",msg);
    scrn_setAttrib(pSCREEN, ATTRIB(BLACK,GRAY));
    col = (SCREEN.csize-len)/2 - 1;
    scrn_showCursor();
    while (editing)
    {
        scrn_xyPrintPad(pSCREEN, READSTR_ROW+1, col, buffer, len);
#ifdef EFI
        _settextposition(READSTR_ROW + 1, col + 0 + pos);
#else
        _settextposition(READSTR_ROW + 2, col + 1 + pos);
#endif
        switch ((ch = win_getkey(1))) {
        case KEY_RETURN:
            editing = 0;
            break;
        
        case KEY_ESC:
            if (scrn_yes("abort input?"))
                editing = 0;
            break;
        
        case KEY_LEFT:
            if (pos)
                pos--;
            break;
           
        case KEY_RIGHT:
            if (buffer[pos] && (pos<len-1))
                pos++;
            break;
        
        case KEY_END:
            while (pos<len && !buffer[pos])
                pos++;
            if (pos==len)
                pos--;
            break;
    
        case KEY_HOME:
            pos=0;
            break;
        
        case KEY_INSERT:
            if ((insertMode = !insertMode))
                scrn_showCursor();
            else
                scrn_showThickCursor();
            break;
        
        case KEY_BS:
            if (pos)
            {
                for (i=pos;i<=len;i++)
                {
                    buffer[i-1]=buffer[i];
                    if (!buffer[i])
                        break;
                }
                pos--;
            }
            break;

        case KEY_DEL:
            if (buffer[pos])
            {
                for (i=pos;i<len;i++)
                {
                    buffer[i]=buffer[i+1];
                    if (!buffer[i])
                        break;
                }
            }
            break;
        
        default:
            if (!hexOnly || isxdigit(ch) || (spaceAllowed && ch==' '))
            {
                if (!pos && firstKey)
                    buffer[0]=0;
                firstKey = 0;
                if (insertMode)
                {
                    for (i=strlen(buffer);i>=pos;i--)
                        if (i<len-1)
                            buffer[i+1]=buffer[i];
                }
                else
                {
                    if (!buffer[pos])
                        buffer[pos+1]=0;
                }        
                buffer[pos++] = ch;
                if (pos==len)
                    pos--;
                break;
            }
        }
    }
    
    /* restore orginal screen setup */
    scrn_restoreScreen(READSTR_ROW, boxCol, READSTR_ROW+3, boxCol+width, savedScreen);
    //savedScreen = scrn_saveScreen(READSTR_ROW, col, READSTR_ROW+3, col+width);
    free(savedScreen);
    _settextposition(oldxy.row, oldxy.col);
    scrn_setAttrib(pSCREEN,attrib);
    return (ch==KEY_RETURN);
}    
    
u08 util_char2num(u08 c)
{
    c = toupper(c);
    return isdigit(c) ? c-'0' : c-'A'+10;
}

int util_readMacAddress()
{
#define MAX_MAC_BUFF 17
    s08 buffer[MAX_MAC_BUFF+1];
    u08 mac[6];
    u08 * buff,ch,sum;

//    u08 MacAddress = 0;
    int i;

    while (1)
    {    
        /* empty string */
        buffer[0]=0; 
        if (global.optMacCmdLine)
            strncpy(buffer, global.mac_address, 13);
		else if (global.optMacProg)
		{  
			if (!util_readStr("Please enter new Mac Address",buffer,MAX_MAC_BUFF,1,1))
			{
				scrn_dispWait("Invalid MAC address");
				return 0;
			}
		}
		
        sum = 0;
        buff = (u08*) buffer;
        i=0;
        if ((global.optMacCmdLine) || (global.optMacProg))
		{
			/* if length is less than 12, do not update prefix & use only last 6 digits as mac address */
			if (strlen(buffer)<12)
			{
        		mac[0] = 0x0;
				mac[1] = 0xFF;
				mac[2] = 0xFF;
				i = 3;
				/* only care about last 6 digit */
				buff += (strlen(buffer)-6);
			}
            
			for (;i<6;i++)
			{
			/* make sure there is more input */
        		if (!*buff)
				{
            		/* it is error if no more input */
					sum=0;
					break;
				}
                
				ch = 0;
				while (*buff && !isxdigit(*buff))
            		buff++;
				if (isxdigit(*buff))
				{
            		ch = util_char2num(*buff++);
					if (isxdigit(*buff))
					ch = (ch << 4) | util_char2num(*buff++);
				}
				mac[i] = ch;
				sum |= ch;
			}
            
			if ((mac[0] & 1) || !sum || ((mac[3]==0) && (mac[4]==0) && (mac[5]==0)))
			{
        		scrn_dispWait("Invalid MAC address");
				return 0;
			}
		}
        SB4xDiagprgEEPROM (mac);  
        return 1;                
    }
}

/*****************************************************************************/
void util_open_device_windows()
{
    int dev;

// temp for debugging
    global.device_detected = 2;
    
    for (dev=0;dev<global.device_detected;dev++)
        if (!util_activateDevice(dev))
            break;
}

/*****************************************************************************/
pause()
{
    _outtext ("Press any key...");
    util_getch();
}

/*****************************************************************************/


/*****************************************************************************/

/*****************************************************************************/
int util_getPass(s08 * msg, s08 * buff, int len)
{
    
	u08 c;
    int l=0;
    
    len--;
    printf(msg);
    while (l<len)
    {
        c = util_getch();
        if (c=='\r')
            break;
        *buff++ = c;
        printf("*");
        l++;
    }    
    *buff='\0';
    printf("\n");
	
    return l;
}

/*****************************************************************************/

/*****************************************************************************/

/*****************************************************************************/
int util_is5703_or_later(void)
{
    u32 rev = T3_ASIC_REV(CURRCARD.ASIC_version);
    return ( rev == T3_ASIC_REV_5703 || rev == T3_ASIC_REV_5704);
}


/*************************************************************/
int util_poll_cpu_messages(void)
{
    if (global.pollCPU)
    {
        check_message(CPUA_CON_STRCT,CPU_RX_COLOR);
        check_message(CPUB_CON_STRCT,CPU_TX_COLOR);
    }
    return 0;
}

/*************************************************************/
u32 util_power(u32 base,u32 n)
{
    u32 answer = 1;
    while (n--)
        answer *= base;
    return answer;    
} 
 
/*************************************************************/
int util_kbhit()
{
    return kbhit() || com_isChar();
}

/*************************************************************/
int util_getch()
{
    while (1)
    {
        if (kbhit())
            return getch();
        if (com_isChar())
            return com_getch();
    }
}


/*****************************************************************************
 *  HISTORY:    Kevin Tran        06/01/2000   Created
 *  $Log: /Source/Bcm44xx/b44diag/t3_util.c $
 * 
 * 13    7/19/02 10:07a Ahui
 * Removed unused code
 * 
 * 12    7/11/02 1:02p Ahui
 * Clean up unused code.
 * 
 * 11    7/08/02 12:22p Ahui
 * Modify code to support new card srom and mac address progarmming.
 * 
 * 10    6/13/02 10:02a Ahui
 * Clean up unused code
 * 
 * 9     6/11/02 11:50a Ahui
 * Added functions to support fmac option
 * 
 * 8     5/16/02 5:59p Mnima
 * use input eeprom name
 * 
 * 7     4/25/02 11:03a Ahui
 * 
 * 3     3/07/02 5:05p Ahui
 * Added sromutil support
 *******************************************************************/

