/****************************************************************************
 * Copyright(c) 2000 Broadcom Corporation, all rights reserved.
 * Contains proprietary and confidential information.
 *
 * This source file is the property of Broadcom Corporation, and
 * may not be copied or distributed in any isomorphic form without 
 * the prior written consent of Broadcom Corporation. 
 *
 *
 *  Name: T 3 _ W I N . H
 *
 *  Description: This file contains command line interface macros
 *  and data structures.
 *
 *  AUTHOR: Kevin Tran
 *
 *
 *******************************************************************/

#ifndef __T3_WIN_H__
#define __T3_WIN_H__

#ifndef EFI
#include <graph.h>
#include <bios.h>
#endif
#include "t3_scrn.h"

#define HISTORY_LINES	20
#define MAX_LINE_LENGTH	80

#define BIOSKEY
#ifdef BIOSKEY
  #define FUNC_KEY(x) (x << 8)
#else
  #define FUNC_KEY(x)   (0x100 | x)
#endif
#define KEY_BS        0x8
#define KEY_RETURN    0xd
#define KEY_ESC       0x1b
#define KEY_ALT_0     (FUNC_KEY(0x81))
#define KEY_ALT_1     (FUNC_KEY(0x78))
#define KEY_ALT_2     (FUNC_KEY(0x79))  
#define KEY_ALT_3     (FUNC_KEY(0x7a))  
#define KEY_ALT_4     (FUNC_KEY(0x7b))  
#define KEY_ALT_5     (FUNC_KEY(0x7c))  
#define KEY_ALT_6     (FUNC_KEY(0x7d))  
#define KEY_ALT_7     (FUNC_KEY(0x7e))  
#define KEY_ALT_8     (FUNC_KEY(0x7f))  
#define KEY_ALT_9     (FUNC_KEY(0x80))  
#define KEY_ALT_A     (FUNC_KEY(0x1e))
#define KEY_ALT_B     (FUNC_KEY(0x30))
#define KEY_ALT_C     (FUNC_KEY(0x2e))
#define KEY_ALT_D     (FUNC_KEY(0x20))
#define KEY_ALT_E     (FUNC_KEY(0x12))
#define KEY_ALT_F     (FUNC_KEY(0x21))
#define KEY_ALT_G     (FUNC_KEY(0x22))
#define KEY_ALT_H     (FUNC_KEY(0x23))
#define KEY_ALT_I     (FUNC_KEY(0x17))
#define KEY_ALT_J     (FUNC_KEY(0x24))
#define KEY_ALT_K     (FUNC_KEY(0x25))
#define KEY_ALT_L     (FUNC_KEY(0x26))
#define KEY_ALT_M     (FUNC_KEY(0x32))
#define KEY_ALT_N     (FUNC_KEY(0x31))
#define KEY_ALT_O     (FUNC_KEY(0x18))
#define KEY_ALT_P     (FUNC_KEY(0x19))
#define KEY_ALT_Q     (FUNC_KEY(0x10))
#define KEY_ALT_R     (FUNC_KEY(0x13))
#define KEY_ALT_S     (FUNC_KEY(0x1f))
#define KEY_ALT_T     (FUNC_KEY(0x14))
#define KEY_ALT_U     (FUNC_KEY(0x16))
#define KEY_ALT_V     (FUNC_KEY(0x2f))
#define KEY_ALT_W     (FUNC_KEY(0x11))
#define KEY_ALT_X     (FUNC_KEY(0x2d))
#define KEY_ALT_Y     (FUNC_KEY(0x15))
#define KEY_ALT_Z     (FUNC_KEY(0x2c))
#define KEY_ALT_F1    (FUNC_KEY(0x68))
#define KEY_ALT_F2    (FUNC_KEY(0x69))
#define KEY_ALT_F3    (FUNC_KEY(0x6a))
#define KEY_ALT_F4    (FUNC_KEY(0x6b))
#define KEY_ALT_F5    (FUNC_KEY(0x6c))
#define KEY_ALT_F6    (FUNC_KEY(0x6d))
#define KEY_ALT_F7    (FUNC_KEY(0x6e))
#define KEY_ALT_F8    (FUNC_KEY(0x6f))
#define KEY_ALT_F9    (FUNC_KEY(0x70))
#define KEY_ALT_F10   (FUNC_KEY(0x71))
#define KEY_ALT_F11   (FUNC_KEY(0x8b))
#define KEY_ALT_F12   (FUNC_KEY(0x8c))

#define KEY_CTL_RIGHT (FUNC_KEY(0x74))
#define KEY_CTL_LEFT  (FUNC_KEY(0x73))
#define KEY_CTL_UP    (FUNC_KEY(0x8d))
#define KEY_CTL_DOWN  (FUNC_KEY(0x91))

#define KEY_F1        (FUNC_KEY(0x3b))
#define KEY_F2        (FUNC_KEY(0x3c))
#define KEY_F3        (FUNC_KEY(0x3d))
#define KEY_F4        (FUNC_KEY(0x3e))
#define KEY_F5        (FUNC_KEY(0x3f))
#define KEY_F6        (FUNC_KEY(0x40))
#define KEY_F7        (FUNC_KEY(0x41))
#define KEY_F8        (FUNC_KEY(0x42))
#define KEY_F9        (FUNC_KEY(0x43))
#define KEY_F10       (FUNC_KEY(0x44))
#define KEY_F11       (FUNC_KEY(0x85))
#define KEY_F12       (FUNC_KEY(0x86))

#define KEY_PGUP      (FUNC_KEY(0x49))
#define KEY_PAGEUP    (FUNC_KEY(0x49))
#define KEY_PGDN      (FUNC_KEY(0x51))
#define KEY_PAGEDN    (FUNC_KEY(0x51))
#define KEY_END       (FUNC_KEY(0x4f))
#define KEY_HOME      (FUNC_KEY(0x47))
#define KEY_LEFT      (FUNC_KEY(0x4b))
#define KEY_RIGHT     (FUNC_KEY(0x4d))
#define KEY_UP        (FUNC_KEY(0x48))
#define KEY_DOWN      (FUNC_KEY(0x50))
#define KEY_INSERT    (FUNC_KEY(0x52))
#define KEY_INS       (FUNC_KEY(0x52))
#define KEY_DELETE    (FUNC_KEY(0x53))
#define KEY_DEL	      (FUNC_KEY(0x53))

#define KEY_CTL_C     0x3
#define KEY_CTL_T	  0x14
#define KEY_CTL_Y	  0x19

/*
typedef enum {
    NORMAL_WIN,
    WIN_NO_FRAME,
} WIN_t;   
*/

typedef enum {
    WIN_NO_ACTION,
    WIN_ACTION_CLOSE,
} WIN_MSG;    

typedef enum 
{
    eKEY,
    eMOUSE,
    ePOLL,
    eSHUTDOWN,
} win_eType;

typedef struct wNode * pWin;
typedef WIN_MSG (*win_eHandler)(pWin, win_eType, u16 event);
typedef struct wNode
{
    Screen scrn;
    ScreenPoint pos;
    int   csize;
    int   rsize;
    ScreenPoint viewPos;    
    s08 * name;
    int   hotKey;
    win_eHandler func;
    void * myStruct;
    struct wNode * next, * prev;
//    WIN_t type;
    u08   showCursor;
    u08   dev;
} wNode, * pWin;

#define W32HEXDIGIT 8

typedef struct {
    s08 * buff;
    ScreenPoint origin;
    int len;
    int pos;
    int firstKey;
    u08 spaceAllowed;
} wEdit_t, * pWEdit;

typedef struct {
    s08 buff[MAX_LINE_LENGTH+1];
    ScreenPoint origin;
    int len;
    int pos;
} wEdCmd_t, * pWEdCmd;

typedef struct {
    wEdit_t ed;
    u32  data;
    s08 format[8];
    s08 buffer[W32HEXDIGIT+1];
    u08 attrib;
} wHexNum_t, * pWHexN;

typedef enum {
    EDIT_ABORT,
    EDIT_GOOD,
    EDIT_NOACTION,
} EDIT_t;

int win_shutDown();
void win_init();

/*
 * function win_new 
 *
 * do not use malloc for name, because it will not be freeed.
 * the myStruct will be freeed automatically. (internal scrn's buffer, too)
 */
pWin win_new(void *myStruct, u16 hotkey, win_eHandler func, s08 *name,
             int pos_r,int pos_c,int rsize,int csize,int attrib);
pWin win_close(pWin w);
int  win_empty(void);
u16  win_getkey(int);
void win_homeCursor();
void win_gotoxy(int,int);
int win_editHex(pWin w,int x,int y, u32 * data, int len);
int win_editHexStr(pWin w,int x,int y, s08 * buffer, int len, int spaceAllowed);
void win_partialRefresh(pWin, int row, int col, int len);
void win_areaRefresh(pWin);
void win_refresh(pWin);
EDIT_t win_editHexInkey(pWin w, pWEdit ed, u16 ch);
void win_editHexInit(pWin w, pWEdit hex, int r, int c, int len, int spaceAllowed);
EDIT_t win_editCmdInkey(pWin w, pWEdCmd ed, u16 ch);
void win_editCmdInit(pWin w, pWEdCmd hex, int r, int c, int len);
EDIT_t win_readHexNumInkey(pWin w, pWHexN hexN, u16 ch);
void win_readHexNumInit(pWin w, int r, int c, pWHexN hexN, int defData, int len, int attrib);
pWin win_newBasic(int r, int c, int h, int w, u08 attrib);
int win_ActivateKey(u16 key);
int win_viewRowSize(void);
void win_setTextColor(u08 a);
void win_gotoCol(int x);
void win_gotoxy(int x,int y);
void win_wherexy(int * x, int * y);
s08 * win_gets(void);
void win_clearscreen(void);
int  win_select(int r, int c, int width, s08 * title, s08 * selectStr[], u32 * sel, int max);
void win_activate(pWin w);
void win_print(s08 * buff);
#ifndef WIN_CLI
void win_noWinCli(pWin w);
void hist_init();
#endif

#endif /* __T3_SCRN_H__ */
