/****************************************************************************
 * Copyright(c) 2000 Broadcom Corporation, all rights reserved.
 * Contains proprietary and confidential information.
 *
 * This source file is the property of Broadcom Corporation, and
 * may not be copied or distributed in any isomorphic form without 
 * the prior written consent of Broadcom Corporation. 
 *
 *
 *  Name:  U M . H
 *
 *  Description: This file contains data structure and macro definitions
 *               for Upper Module (UM).
 *
 *  AUTHOR: Kevin Tran
 *
 *  HISTORY:    Kevin Tran        06/01/2000   Created
 *
 ******************************************************************************/

#ifndef UM_H
#define UM_H

#include "mm.h"

#include "types.h"

/******************************************************************************/
/* Constants. */
/******************************************************************************/

#define DEFAULT_NUM_OF_MAP_REGISTERS                100     /* per ring. */
#define DEFAULT_RX_PACKET_DESC_LOW_WATER_MARK       40

/******************************************************************************/
/* UM_DEVICE_BLOCK */
/******************************************************************************/

#define MAX_DATA_BLKS    128

#define TCP_UDP_CHKSUM_OFFLOAD   (1 << 0)
#define IP_CHKSUM_OFFLOAD        (1 << 1)
#define VLAN_INSERTION           (1 << 2)
#define TCP_SEGMENTATION_OFFLOAD (1 << 3)
#define DONT_GEN_CRC             (1 << 4)
#define IP_FRAGMENT              (1 << 5)
#define IP_FRAGMENT_END          (1 << 6)

#define MAX_NUM_NIC MAX_DEVICES

#define NUM_OF_RESOURCES    20

typedef struct NIC_STATS
{
  struct 
  {
    U32 TxCount;
    U32 TxEnqCount;
  }TxStats[T3_MAX_SEND_RCB_COUNT];

  struct
  {
    U32 RxCount;
    U32 RxCrcErr;
    U32 RxLengthMismatch;
  }RxStats[T3_MAX_RCV_RETURN_RCB_COUNT];

  U32 OutOfMemory;
  U32 RxInvalidSeqNumber;
  U32 RxTprotPacketCnt;
  U32 RxDiagPacketCnt;
  U32 RxIpPacketCnt;
  U32 RxArpPacketCnt;
  U32 RxOtherPacketCnt;
  U32 RxInvalidLen;
  U32 RxReplicateFrame;
  U32 RxHwBadTcpUdpChecksum;
  U32 RxHwBadIpChecksum;
  U32 RxSwBadTcpUdpChecksum;
  U32 RxSwBadIpChecksum;
  U32 TooManyFragPkt;
  U32 RxInvalidRawChecksum;
#ifdef T3_JUMBO_USE_EXT_RX_BD
  U32 RxExtBdUsed;
#ifdef T3_CHECK_SPLIT_HEADER
  U32 HdrSplitted;
#endif
  U32 TxFailedDueToQFull;
#endif
}NIC_STATS,*pNIC_STATS;

typedef struct NIC_BLK
{
  NIC_STATS stats;
  NIC_STATS rate;
  void *DeviceInst;
}NIC_BLK,*pNIC_BLK;

typedef struct TTP_TX_BLOCK
{
  U16 TotalLen;
  U16 DataBlkCount;
  U16 Flags;
  U16 VlanTag;
  U8 *BufferPtr;
  struct DataBlk
  {
    U16 BlkLen;
    U8  *pPhyAddr;
  }DataBlock[MAX_DATA_BLKS];
}TTP_TX_BLOCK,*pTTP_TX_BLOCK;

typedef struct 
{
  /* OS independent device structure. */
  LM_DEVICE_BLOCK LmDevice;  /* must be the first member of the struct */

  /* PCI "slot" number. */
  U32 Slot;
  /* PCI bus number. */
  U32 BusNumber;
  U32 DeviceFunc;

  /* Filter mask */
  U32 FilterMask;
  U8 *UsedResource[NUM_OF_RESOURCES];
  U32 Handle[NUM_OF_RESOURCES];
} UM_DEVICE_BLOCK, *PUM_DEVICE_BLOCK;


/******************************************************************************/
/* Packet descriptor. */
/******************************************************************************/

typedef struct 
{
  LM_PACKET LmPacket;     /* Must be the first element of this structure. */

	unsigned char *pMemVirt;
	unsigned long MemPhy;
  union 
  {
    /* Tx info. */
	
    struct 
    {
      pTTP_TX_BLOCK pTxDataBlk; 
      double time_stamp;
    } Tx;
  }u;

} UM_PACKET, *PUM_PACKET;

#define MAX_RX_IND_USERS      4
#define MAX_TX_COMP_IND_USERS 4

/* Function prototypes */

pNIC_BLK UM_GetNIC(void);

int t3_shutdown();

int  UM_FreeResources(PUM_DEVICE_BLOCK pDevice);
dbg_printf();

#endif /* UM_H */

